// ***************************************************************************************
// File: session2.cpp
// Description: session management
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: The session.cpp file was too big for some compilers (Solaris)
//           So it has been divided into session.cpp, session1.cpp,
//           session2.cpp and so on...
// ***************************************************************************************


#include "nvs.h"

#ifdef NV_SERVER
	#ifdef WIN32
		#ifdef _DEBUG
			#include <stdlib.h>
			#include <crtdbg.h>
			#define NVNEW new( _NORMAL_BLOCK, __FILE__, __LINE__)
		#else // _DEBUG
			#define NVNEW new
		#endif // _DEBUG
	#else	//WIN32
		#define NVNEW new
	#endif	//WIN32
#else	//NV_SERVER
	#define NVNEW new
#endif	//NV_SERVER

using namespace std;

#ifdef PB_PERMISSION_LIST
extern NvMutex ProtectConsoleLog;				// protect console output defeined in nvs.cpp
#endif

//SYSTEM LOG CREATE
bool NvSession::LogCreate(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get variable name that should be in command parameters
	string LogName = CommandToExecute->Parameters.GetValue("LOG");
	CleanString(LogName);

	if(!LogName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LOG:CREATE:LOG");
		return false;
	}

	// By default, we have an application log
	
	bool bServiceLog = false;
	bool bSystemLog = false;
	bool bApplicationLog = true;

	NvService *Service = NULL;
	
	
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);
	if(ServiceName.length())
	{
		bApplicationLog = false;
		if(ServiceName == "SYSTEM")
			bSystemLog = true;
		else
			bServiceLog = true;
	}
		
	if(bSystemLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_LOG_ADMIN"))
			return false;
	if(bServiceLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_LOG_ADMIN"))
			return false;
	if(bApplicationLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_LOG_ADMIN"))
			return false;

	if(bServiceLog && ServiceName.length())
	{
		Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
		if(Service == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
			return false;
		}
	}

	NvLogList *LogList = &(Application->Logs);
	if(bSystemLog)
		LogList = &(GetMain()->Logs);
	if(bServiceLog)
		LogList = &(Service->Logs);


	string sErase = CommandToExecute->Parameters.GetValue("ERASE");
	CleanString(sErase);
	MakeUpper(sErase);
	bool bErase = false;
	if(sErase == "YES")
		bErase = true;

	// Get variable name that should be in command parameters
	string ImportFileName = "";
	if(CommandToExecute->Parameters.KeyExist("FILE"))
	{
		string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
		CleanString(FileObjectName);
		MakeUpper(FileObjectName);

		if(!FileObjectName.length())
			FileObjectName = "LOG_FILE";

		NvObjFile *FileObject = (NvObjFile *)InContainer->GetObject(FileObjectName.c_str());
		if(FileObject != NULL)
		{
			if(FileObject->GetType() != NV_OBJTYPE_FILE)
			{
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, FileObjectName);
				return false;
			}
			ImportFileName = FileObject->GetFilePathName();
		}
	}

	if(!LogList->Create(LogName.c_str(), true, bErase, ImportFileName.c_str()))
	{
		if(Service != NULL)
			GetMain()->Services.FreeService(ServiceName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "LOG", NV_LOG_CANNOT_CREATE, LogName);
		return false;
	}

	if(Service != NULL)
		GetMain()->Services.FreeService(ServiceName.c_str());

	
	// Set the eventual options
	return LogSetOptions(CommandToExecute);
}


//SYSTEM LOG REMOVE
bool NvSession::LogRemove(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string LogName = CommandToExecute->Parameters.GetValue("LOG");
	CleanString(LogName);

	if(!LogName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LOG:REMOVE:LOG");
		return false;
	}

	// By default, we have an application log
	
	bool bServiceLog = false;
	bool bSystemLog = false;
	bool bApplicationLog = true;

	NvService *Service = NULL;
	
	
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);
	if(ServiceName.length())
	{
		bApplicationLog = false;
		if(ServiceName == "SYSTEM")
			bSystemLog = true;
		else
			bServiceLog = true;
	}
		
	if(bSystemLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_LOG_ADMIN"))
			return false;
	if(bServiceLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_LOG_ADMIN"))
			return false;
	if(bApplicationLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_LOG_ADMIN"))
			return false;

	if(bServiceLog && ServiceName.length())
	{
		Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
		if(Service == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
			return false;
		}
	}

	NvLogList *LogList = &(Application->Logs);
	if(bSystemLog)
		LogList = &(GetMain()->Logs);
	if(bServiceLog)
		LogList = &(Service->Logs);

	string sErase = CommandToExecute->Parameters.GetValue("ERASE");
	CleanString(sErase);
	MakeUpper(sErase);
	bool bErase = false;
	if(sErase == "YES")
		bErase = true;

	bool bProtectSystemLog = false;
	if(bSystemLog)
		bProtectSystemLog = true;

	LogList->RemoveLog(LogName.c_str(), bErase, bProtectSystemLog);

	if(Service != NULL)
		GetMain()->Services.FreeService(ServiceName.c_str());

	return true;
}


//SYSTEM LOG INFO
bool NvSession::LogInfo(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get variable name that should be in command parameters
	string LogName = CommandToExecute->Parameters.GetValue("LOG");
	CleanString(LogName);

	// The log name is not mandatory for this command
	// If it's provided, the command returns information only about this log

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	OutContainer->RemoveObject("LOG_INFO");
	if(!OutContainer->CreateObject("LOG_INFO", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "LOG_INFO");
		return false;
	}

	NvObjTable *LogInfo = (NvObjTable *)(OutContainer->GetObject("LOG_INFO"));

	// By default, we have an application log
	
	bool bServiceLog = false;
	bool bSystemLog = false;
	bool bApplicationLog = true;

	NvService *Service = NULL;
	
	
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);
	if(ServiceName.length())
	{
		bApplicationLog = false;
		if(ServiceName == "SYSTEM")
			bSystemLog = true;
		else
			bServiceLog = true;
	}

	if(bSystemLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_LOG_ADMIN"))
			return false;
	if(bServiceLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_LOG_ADMIN"))
			return false;
	if(bApplicationLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_LOG_ADMIN"))
			return false;

	if(bServiceLog && ServiceName.length())
	{
		Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
		if(Service == NULL)
		{
			OutContainer->RemoveObject("LOG_INFO");
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
			return false;
		}
	}

	NvLogList *LogList = &(Application->Logs);
	if(bSystemLog)
		LogList = &(GetMain()->Logs);
	if(bServiceLog)
		LogList = &(Service->Logs);

	

	if(!LogList->GetInfo(LogName.c_str(), LogInfo))
	{
		OutContainer->RemoveObject("LOG_INFO");
		if(Service != NULL)
			GetMain()->Services.FreeService(ServiceName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "LOG", NV_LOG_CANNOT_GET_INFO, LogName);
		return false;
	}

	if(Service != NULL)
		GetMain()->Services.FreeService(ServiceName.c_str());

	return true;
}


//SYSTEM LOG WRITE
bool NvSession::LogWrite(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string LogName = CommandToExecute->Parameters.GetValue("LOG");
	CleanString(LogName);

	if(!LogName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LOG:WRITE:LOG");
		return false;
	}

	// By default, we have an application log
	
	bool bServiceLog = false;
	bool bSystemLog = false;
	bool bApplicationLog = true;

	NvService *Service = NULL;
	
	
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);
	if(ServiceName.length())
	{
		bApplicationLog = false;
		if(ServiceName == "SYSTEM")
			bSystemLog = true;
		else
			bServiceLog = true;
	}
		
	if(bServiceLog && ServiceName.length())
	{
		Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
		if(Service == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
			return false;
		}
	}

	NvLogList *LogList = &(Application->Logs);
	if(bSystemLog)
		LogList = &(GetMain()->Logs);
	if(bServiceLog)
		LogList = &(Service->Logs);

	string sMessage = CommandToExecute->Parameters.GetValue("MESSAGE");
	string sExtra = CommandToExecute->Parameters.GetValue("EXTRA");
	string sType = CommandToExecute->Parameters.GetValue("TYPE");
	CleanString(sType);
	MakeUpper(sType);
	int iType = NV_LOG_INFO;
	if(sType == "WARNING")
		iType = NV_LOG_WARNING;
	else if(sType == "ERROR")
		iType = NV_LOG_ERROR;
	string sLevel = CommandToExecute->Parameters.GetValue("LEVEL");
	CleanString(sLevel);
	int iLevel = 1;
	if(sLevel.length())
		iLevel = atoi(sLevel.c_str());
	string sIdentifier = CommandToExecute->Parameters.GetValue("IDENT");
	if(!sIdentifier.length())
		sIdentifier = m_sIdent;	// Session identifier by default
	string sSource = CommandToExecute->Parameters.GetValue("SOURCE");
	if(!sSource.length())
	{
		if(CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT)
			sSource = "client";
		if(CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER)
			sSource = "browser";
		if(CommandToExecute->m_iCommandSource == NV_SOURCE_PROCEDURE)
			sSource = "procedure";
		if(CommandToExecute->m_iCommandSource == NV_SOURCE_SERVICE)
			sSource = "service";
		if(CommandToExecute->m_iCommandSource == NV_SOURCE_SERVER)
			sSource = "server";
	}

	LogList->Write(LogName.c_str(), sMessage.c_str(), sExtra.c_str(), iType, iLevel, sIdentifier.c_str(), sSource.c_str());

	if(Service != NULL)
		GetMain()->Services.FreeService(ServiceName.c_str());


	return true;
}


//SYSTEM LOG SET_OPTIONS
bool NvSession::LogSetOptions(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string LogName = CommandToExecute->Parameters.GetValue("LOG");
	CleanString(LogName);

	if(!LogName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LOG:SET_OPTIONS:LOG");
		return false;
	}

	// By default, we have an application log
	
	bool bServiceLog = false;
	bool bSystemLog = false;
	bool bApplicationLog = true;

	NvService *Service = NULL;
	
	
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);
	if(ServiceName.length())
	{
		bApplicationLog = false;
		if(ServiceName == "SYSTEM")
			bSystemLog = true;
		else
			bServiceLog = true;
	}
		
	if(bSystemLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_LOG_ADMIN"))
			return false;
	if(bServiceLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_LOG_ADMIN"))
			return false;
	if(bApplicationLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_LOG_ADMIN"))
			return false;

	if(bServiceLog && ServiceName.length())
	{
		Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
		if(Service == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
			return false;
		}
	}

	NvLogList *LogList = &(Application->Logs);
	if(bSystemLog)
		LogList = &(GetMain()->Logs);
	if(bServiceLog)
		LogList = &(Service->Logs);

	if(CommandToExecute->Parameters.KeyExist("DESCRIPTION"))
	{
		string Description = CommandToExecute->Parameters.GetValue("DESCRIPTION");
		CleanString(Description);
		LogList->SetDescription(LogName.c_str(), Description.c_str());
	}

	if(CommandToExecute->Parameters.KeyExist("LEVEL"))
	{
		string sLevel = CommandToExecute->Parameters.GetValue("LEVEL");
		CleanString(sLevel);
		int iLevel = atoi(sLevel.c_str());
		LogList->SetLevel(LogName.c_str(), iLevel);
	}

	if(CommandToExecute->Parameters.KeyExist("MAXTIME"))
	{
		string sMaxTime = CommandToExecute->Parameters.GetValue("MAXTIME");
		CleanString(sMaxTime);
		int iMaxTime = atoi(sMaxTime.c_str());
		LogList->SetMaxTime(LogName.c_str(), iMaxTime);
	}

	if(CommandToExecute->Parameters.KeyExist("SAVE"))
	{
		string sSave = CommandToExecute->Parameters.GetValue("SAVE");
		CleanString(sSave);
		MakeUpper(sSave);
		bool SaveIt = true;
		if(sSave == "NO")
			SaveIt = false;
		LogList->SetSave(LogName.c_str(), SaveIt);
	}

	if(CommandToExecute->Parameters.KeyExist("MAXSAVEDAYS"))
	{
		string sMaxSaveDays = CommandToExecute->Parameters.GetValue("MAXSAVEDAYS");
		CleanString(sMaxSaveDays);
		int iMaxSaveDays = atoi(sMaxSaveDays.c_str());
		LogList->SetMaxSaveDays(LogName.c_str(), iMaxSaveDays);
	}

	if(Service != NULL)
		GetMain()->Services.FreeService(ServiceName.c_str());

	return true;
}


//SYSTEM LOG SEARCH
bool NvSession::LogSearch(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get variable name that should be in command parameters
	string LogName = CommandToExecute->Parameters.GetValue("LOG");
	CleanString(LogName);

	if(!LogName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LOG:SEARCH:LOG");
		return false;
	}

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get object result table that should be in command parameters
	string ObjResult = CommandToExecute->Parameters.GetValue("RESULT");
	CleanString(ObjResult);
	if(!ObjResult.length())
		ObjResult = "LOG_VIEW";

	NvObjTable* LogView = (NvObjTable *)(OutContainer->GetObject(ObjResult.c_str()));

	if(LogView != NULL)
	{
		// The object exists so we remove it
		OutContainer->RemoveObject(ObjResult.c_str());
		LogView = NULL;
	}
		
	if(!OutContainer->CreateObject(ObjResult.c_str(), NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjResult);
		return false;
	}
	
	LogView = (NvObjTable *)(OutContainer->GetObject(ObjResult.c_str()));
	if(LogView == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjResult);
		return false;
	}

	// By default, we have an application log
	bool bServiceLog = false;
	bool bSystemLog = false;
	bool bApplicationLog = true;

	NvService *Service = NULL;
	
	
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);
	if(ServiceName.length())
	{
		bApplicationLog = false;
		if(ServiceName == "SYSTEM")
			bSystemLog = true;
		else
			bServiceLog = true;
	}
		
	if(bSystemLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_LOG_READ"))
			return false;
	if(bServiceLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_LOG_READ"))
			return false;
	if(bApplicationLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_LOG_READ"))
			return false;

	if(bServiceLog && ServiceName.length())
	{
		Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
		if(Service == NULL)
		{
			OutContainer->RemoveObject("LOG_VIEW");
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
			return false;
		}
	}

	NvLogList *LogList = &(Application->Logs);
	if(bSystemLog)
		LogList = &(GetMain()->Logs);
	if(bServiceLog)
		LogList = &(Service->Logs);

	
	string Identifier = CommandToExecute->Parameters.GetValue("IDENTIFIER");
	CleanString(Identifier);

	string Message = CommandToExecute->Parameters.GetValue("MESSAGE");
	CleanString(Message);

	string Extra = CommandToExecute->Parameters.GetValue("EXTRA");
	CleanString(Extra);

	string Source = CommandToExecute->Parameters.GetValue("SOURCE");
	CleanString(Identifier);

	string FromDate = CommandToExecute->Parameters.GetValue("DATE");
	CleanString(FromDate);

	string ToDate = CommandToExecute->Parameters.GetValue("TO_DATE");
	CleanString(ToDate);
	
	string FromHour = CommandToExecute->Parameters.GetValue("TIME");
	CleanString(FromHour);

	string ToHour = CommandToExecute->Parameters.GetValue("TO_TIME");
	CleanString(ToHour);
	
	string sLevel = CommandToExecute->Parameters.GetValue("LEVEL");
	CleanString(sLevel);
	int iLevel = 0;
	if(sLevel.length())
	{
		iLevel = atoi(sLevel.c_str());
		if(iLevel < 0)
			iLevel = 0;
	}
	
	string sType = CommandToExecute->Parameters.GetValue("TYPE");
	CleanString(sType);
	int iType = NV_LOG_ALL;
	if(sType.length())
	{
		iType = atoi(sType.c_str());
		if(iType < 0)
			iType = NV_LOG_ALL;
		if(iType == 0)
			iType = NV_LOG_INFO;
		if(iType == 1)
			iType = NV_LOG_WARNING;
		if(iType == 2)
			iType = NV_LOG_ERROR;
		if(iType > 2)
			iType = NV_LOG_ALL;
	}

	
	string sMaxSize = CommandToExecute->Parameters.GetValue("MAXSIZE");
	CleanString(sMaxSize);
	int iMaxSize = 1024;
	if(sMaxSize.length())
	{
		iMaxSize = atoi(sMaxSize.c_str());
		if(iMaxSize < 0)
			iMaxSize = 128;
	}

	string ExportFileName = "";
	if(CommandToExecute->Parameters.KeyExist("FILE"))
	{
		// Get variable name that should be in command parameters
		string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
		CleanString(FileObjectName);
		MakeUpper(FileObjectName);

		if(!FileObjectName.length())
			FileObjectName = "LOG_FILE";

		NvObjFile *FileObject = (NvObjFile *)OutContainer->GetObject(FileObjectName.c_str());
		if(FileObject == NULL)
		{
			if(!OutContainer->CreateObject(FileObjectName.c_str(), NV_OBJTYPE_FILE))
			{
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, FileObjectName);
				return false;
			}
			FileObject = (NvObjFile *)OutContainer->GetObject(FileObjectName.c_str());
			FileObject->SetAutoFileName(Application->GetWorkDir(), "nvl");
		}
		if(FileObject->GetType() != NV_OBJTYPE_FILE)
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, FileObjectName);
			return false;
		}

		ExportFileName = FileObject->GetFilePathName();
	}

	if(!LogList->Search(LogName.c_str(), LogView, FromDate.c_str(), ToDate.c_str(), FromHour.c_str(), ToHour.c_str(), Identifier.c_str(), iType, Message.c_str(), Extra.c_str(), Source.c_str(), iLevel, ExportFileName.c_str(), iMaxSize))
	{
		OutContainer->RemoveObject("LOG_VIEW");
		if(Service != NULL)
			GetMain()->Services.FreeService(ServiceName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "LOG", NV_LOG_SEARCH_ERROR, LogName);
		return false;
	}

	if(Service != NULL)
		GetMain()->Services.FreeService(ServiceName.c_str());

	return true;
}


//SYSTEM LOG ERASE
bool NvSession::LogErase(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get variable name that should be in command parameters
	string LogName = CommandToExecute->Parameters.GetValue("LOG");
	CleanString(LogName);

	if(!LogName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LOG:ERASE:LOG");
		return false;
	}


	// By default, we have an application log
	
	bool bServiceLog = false;
	bool bSystemLog = false;
	bool bApplicationLog = true;

	NvService *Service = NULL;
	
	
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);
	if(ServiceName.length())
	{
		bApplicationLog = false;
		if(ServiceName == "SYSTEM")
			bSystemLog = true;
		else
			bServiceLog = true;
	}
		
	if(bSystemLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_LOG_ADMIN"))
			return false;
	if(bServiceLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SERVICE_LOG_ADMIN"))
			return false;
	if(bApplicationLog)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_LOG_ADMIN"))
			return false;

		
	if(bServiceLog && ServiceName.length())
	{
		Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
		if(Service == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
			return false;
		}
	}

	NvLogList *LogList = &(Application->Logs);
	if(bSystemLog)
		LogList = &(GetMain()->Logs);
	if(bServiceLog)
		LogList = &(Service->Logs);


	string FromDate = CommandToExecute->Parameters.GetValue("DATE");
	CleanString(FromDate);

	string ToDate = CommandToExecute->Parameters.GetValue("TO_DATE");
	CleanString(ToDate);
	
	string sClear = CommandToExecute->Parameters.GetValue("CLEAR");
	CleanString(sClear);
	MakeUpper(sClear);
	bool bClear = false;
	if(sClear == "YES")
		bClear = true;

	if(!LogList->RemoveLogData(LogName.c_str(), FromDate.c_str(), ToDate.c_str(), bClear))
	{
		if(Service != NULL)
			GetMain()->Services.FreeService(ServiceName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "LOG", NV_LOG_ERASE_ERROR, LogName);
		return false;
	}

	if(Service != NULL)
		GetMain()->Services.FreeService(ServiceName.c_str());

	return true;
}



//SYSTEM SECURITY INFO
bool NvSession::SecurityInfo(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
	{
		FreeSecurity(SecuApp);
		return false;
	}
	
	OutContainer->RemoveObject("INFO");
	if(!OutContainer->CreateObject("INFO", NV_OBJTYPE_INDSTRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "INFO");
		FreeSecurity(SecuApp);
		return false;
	}

	NvObjIndStringList *SecurityInfo = (NvObjIndStringList *)(OutContainer->GetObject("INFO"));
	int iPasswordExpire = CurrentSecurity->GetPasswordExpire();
	bool bPasswordChangeFirst = CurrentSecurity->GetPasswordChangeNext();
	char Buf[50];
	sprintf(Buf, "%d", iPasswordExpire);
	SecurityInfo->SetValue("PASSWORD_EXPIRE", Buf);
	if(bPasswordChangeFirst)
		SecurityInfo->SetValue("PASSWORD_CHANGE_NEXT", "YES");
	else
		SecurityInfo->SetValue("PASSWORD_CHANGE_NEXT", "NO");

	int iUserTimeOut = CurrentSecurity->GetUserTimeOut();
	sprintf(Buf, "%d", iUserTimeOut);
	SecurityInfo->SetValue("USER_TIME_OUT", Buf);

	SecurityInfo->SetValue("NEW_PASSWORD_CHECK_PROC", CurrentSecurity->GetProcCheckPassword());


	int iLockingAttempts = CurrentSecurity->GetLockingAttempts();
	sprintf(Buf, "%d", iLockingAttempts);
	SecurityInfo->SetValue("LOCKING_ATTEMPTS", Buf);

	FreeSecurity(SecuApp);
	
	return true;
}



//SYSTEM SECURITY SET_PARAM
bool NvSession::SecuritySetParam(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string sApplyToAll = CommandToExecute->Parameters.GetValue("APPLY_TO_ALL");
	CleanString(sApplyToAll, true);
	MakeUpper(sApplyToAll);
	bool bApplyToAll = false;
	if(sApplyToAll == "YES")
		bApplyToAll = true;

	if(CommandToExecute->Parameters.KeyExist("PASSWORD_EXPIRE"))
	{
		string Value = CommandToExecute->Parameters.GetValue("PASSWORD_EXPIRE");
		CleanString(Value, true);
		MakeUpper(Value);
		if(Value.length())
		{
			int iPasswordExpire = atoi(Value.c_str());
			if(iPasswordExpire < -1)
				iPasswordExpire = -1;
			CurrentSecurity->SetPasswordExpire(iPasswordExpire, bApplyToAll);
		}
	}

	if(CommandToExecute->Parameters.KeyExist("PASSWORD_CHANGE_NEXT"))
	{
		string Value = CommandToExecute->Parameters.GetValue("PASSWORD_CHANGE_NEXT");
		CleanString(Value, true);
		MakeUpper(Value);
		if(Value == "YES")
			CurrentSecurity->SetPasswordChangeNext(true, bApplyToAll);
		if(Value == "NO")
			CurrentSecurity->SetPasswordChangeNext(false, bApplyToAll);
	}

	if(CommandToExecute->Parameters.KeyExist("USER_TIME_OUT"))
	{
		string Value = CommandToExecute->Parameters.GetValue("USER_TIME_OUT");
		CleanString(Value, true);
		MakeUpper(Value);
		if(Value.length())
		{
			int iUserTimeOut = atoi(Value.c_str());
			if(iUserTimeOut < -1)
				iUserTimeOut = -1;
			CurrentSecurity->SetUserTimeOut(iUserTimeOut);
		}
	}

	if(CommandToExecute->Parameters.KeyExist("NEW_PASSWORD_CHECK_PROC"))
	{
		string Value = CommandToExecute->Parameters.GetValue("NEW_PASSWORD_CHECK_PROC");
		CleanString(Value, true);
		CurrentSecurity->SetProcCheckPassword(Value);
	}

	if(CommandToExecute->Parameters.KeyExist("LOCKING_ATTEMPTS"))
	{
		string Value = CommandToExecute->Parameters.GetValue("LOCKING_ATTEMPTS");
		CleanString(Value, true);
		MakeUpper(Value);
		if(Value.length())
		{
			int iLockingAttempts = atoi(Value.c_str());
			if(iLockingAttempts < -1)
				iLockingAttempts = -1;
			CurrentSecurity->SetLockingAttempts(iLockingAttempts);
		}
	}

	FreeSecurity(SecuApp);
	
	return true;
}



//SYSTEM SECURITY CHECK
bool NvSession::SecurityCheck(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	string Permission = CommandToExecute->Parameters.GetValue("PERMISSION");
	string Procedure = CommandToExecute->Parameters.GetValue("PROCEDURE");
	string ResultObject = CommandToExecute->Parameters.GetValue("AUTHORIZED");
	MakeUpper(ResultObject);
	if(CommandToExecute->Parameters.KeyExist("AUTHORIZED"))
		if(!ResultObject.length())
			ResultObject = "AUTHORIZED";
	if(ResultObject.length())
	{
		// The function uses the output container
		if(!CheckOutputContainer(CommandToExecute))
			return false;
		OutContainer->RemoveObject(ResultObject.c_str());
		if(!OutContainer->CreateObject(ResultObject.c_str(), NV_OBJTYPE_BOOLEAN))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ResultObject);
			return false;
		}
		NvObj *Object = OutContainer->GetObject(ResultObject.c_str());
		((NvObjBoolean *)Object)->SetValue(false);
	}

	bool RetVal = CheckPermission(CommandToExecute, ServiceName.c_str(), Permission.c_str(), Procedure.c_str());
	if(!ResultObject.length())
		return RetVal;

	NvObj *Object = OutContainer->GetObject(ResultObject.c_str());
	((NvObjBoolean *)Object)->SetValue(RetVal);

	CommandToExecute->Error.SetError();

	return true;
}


//SYSTEM SECURITY ADD_USER
bool NvSession::SecurityUserAdd(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string UserName = CommandToExecute->Parameters.GetValue("USER");
	string Description = CommandToExecute->Parameters.GetValue("DESCRIPTION");
	string FullName = CommandToExecute->Parameters.GetValue("FULLNAME");
	string Password = CommandToExecute->Parameters.GetValue("PASSWORD");
	CleanString(Password, true);
	string RealNewPassword = Password;
	if(Password.length())
	{
		Password = MD5_string(Password);
		MakeUpper(Password);
	}

	bool bNewPasswordProcResult = true;
	string ProcCheckPassword = CurrentSecurity->GetProcCheckPassword();
	if(ProcCheckPassword.length())
	{
		string OldPasswordHere = "";
		string UserNameHere = UserName;
		CleanString(UserNameHere);

		// Call here the application check password procedure
		// From a special administrator session
		// We do not need any login here
		NvSession Session;
		Session.Init();
		Session.SetAppName(Application->GetName().c_str());
		Session.Application = Application;
		Session.m_bAdministrator = true;

		NvCommand Command;
		Session.OriginalCommand = &Command;
		Command.m_pSession = &Session;
		Command.m_iCommandSource = NV_SOURCE_SERVER;
		Command.m_sService = "SYSTEM";
		Command.m_sClass = "MISC";
		Command.m_sCommand = "NOP";
		Command.Parameters.SetValue("OLD_PASSWORD", OldPasswordHere);
		Command.Parameters.SetValue("NEW_PASSWORD", RealNewPassword);
		Command.Parameters.SetValue("USER", UserNameHere);
		if(!Session.ExecuteProcedure(&Command, ProcCheckPassword, false))
			bNewPasswordProcResult = false;
		Session.Application = NULL;
	}


	if(CurrentSecurity->UserExist(UserName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_USER_EXIST, UserName);
		FreeSecurity(SecuApp);
		return false;
	}

	if(!bNewPasswordProcResult)
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_BAD_PASSWORD_SYNTAX, UserName);
		FreeSecurity(SecuApp);
		return false;
	}

	string PasswordExpire = CommandToExecute->Parameters.GetValue("PASSWORD_EXPIRE");
	string PasswordChangeFirst = CommandToExecute->Parameters.GetValue("PASSWORD_CHANGE_FIRST");
	if(!CurrentSecurity->AddUser(UserName.c_str(), Description.c_str(), FullName.c_str(), Password.c_str(), PasswordExpire.c_str(), PasswordChangeFirst.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_ADD_USER, UserName);
		FreeSecurity(SecuApp);
		return false;
	}

	FreeSecurity(SecuApp);
	
	return true;
}



//SYSTEM SECURITY REMOVE_USER
bool NvSession::SecurityUserRemove(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	string UserName = CommandToExecute->Parameters.GetValue("USER");
	CleanString(UserName, true);
	MakeLower(UserName);

	if(UserName == "nvdef")
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_REMOVE_DEFAULT, UserName);
		FreeSecurity(SecuApp);
		return false;
	}

	CurrentSecurity->RemoveUser(UserName.c_str());

	FreeSecurity(SecuApp);
	
	return true;
}



//SYSTEM SECURITY ENABLE_USER
bool NvSession::SecurityUserEnable(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	string UserName = CommandToExecute->Parameters.GetValue("USER");
	CleanString(UserName, true);
	MakeLower(UserName);

	string sEnable = CommandToExecute->Parameters.GetValue("ENABLE");
	CleanString(sEnable, true);
	MakeUpper(sEnable);
	bool bEnable = true;
	if(sEnable=="NO")
		bEnable = false;

	if(!bEnable)
	{
		if(UserName == "nvadmin")
		{
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_DISABLE_USER, UserName);
			FreeSecurity(SecuApp);
			return false;
		}
	}

	if(UserName != "nvadmin")
	{
		if(!CurrentSecurity->EnableUser(UserName.c_str(), bEnable))
		{
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_DISABLE_USER, UserName);
			FreeSecurity(SecuApp);
			return false;
		}
	}

	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY USER_LIST
bool NvSession::SecurityUserList(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("USER_LIST");
	if(!OutContainer->CreateObject("USER_LIST", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "USER_LIST");
		return false;
	}

	NvObjTable *Users = (NvObjTable *)(OutContainer->GetObject("USER_LIST"));

	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
	{
		OutContainer->RemoveObject("USER_LIST");
		return false;
	}
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			OutContainer->RemoveObject("USER_LIST");
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			OutContainer->RemoveObject("USER_LIST");
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string UserName = CommandToExecute->Parameters.GetValue("USER");
	if(!CurrentSecurity->UserList(Users, UserName.c_str()))
	{
		OutContainer->RemoveObject("USER_LIST");
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_LIST_USERS, UserName);
		FreeSecurity(SecuApp);
		return false;
	}

	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY USER_EXIST
bool NvSession::SecurityUserExist(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string UserName = CommandToExecute->Parameters.GetValue("USER");

	char Buf[20];
	strcpy(Buf, "NO");
	if(CurrentSecurity->UserExist(UserName.c_str()))
		strcpy(Buf, "YES");

	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);
	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY SET_USER_DESCRIPTION
bool NvSession::SecurityUserSetDescription(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string UserName = CommandToExecute->Parameters.GetValue("USER");

	if(!CurrentSecurity->UserExist(UserName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_INVALID_USER, UserName);
		FreeSecurity(SecuApp);
		return false;
	}

	string Description = CommandToExecute->Parameters.GetValue("DESCRIPTION");
	CurrentSecurity->SetUserDescription(UserName.c_str(), Description.c_str());

	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY SET_USER_FULLNAME
bool NvSession::SecurityUserSetFullname(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string UserName = CommandToExecute->Parameters.GetValue("USER");

	if(!CurrentSecurity->UserExist(UserName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_INVALID_USER, UserName);
		FreeSecurity(SecuApp);
		return false;
	}

	string FullName = CommandToExecute->Parameters.GetValue("FULLNAME");
	CurrentSecurity->SetUserFullName(UserName.c_str(), FullName.c_str());

	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY SET_USER_PARAM
bool NvSession::SecurityUserSetParam(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string UserName = CommandToExecute->Parameters.GetValue("USER");

	if(!CurrentSecurity->UserExist(UserName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_INVALID_USER, UserName);
		FreeSecurity(SecuApp);
		return false;
	}

	if(CommandToExecute->Parameters.KeyExist("DESCRIPTION"))
	{
		string Value = CommandToExecute->Parameters.GetValue("DESCRIPTION");
		CurrentSecurity->SetUserDescription(UserName.c_str(), Value.c_str());
	}

	if(CommandToExecute->Parameters.KeyExist("FULLNAME"))
	{
		string Value = CommandToExecute->Parameters.GetValue("FULLNAME");
		CurrentSecurity->SetUserFullName(UserName.c_str(), Value.c_str());
	}

	if(CommandToExecute->Parameters.KeyExist("PASSWORD_EXPIRE"))
	{
		string Value = CommandToExecute->Parameters.GetValue("PASSWORD_EXPIRE");
		CleanString(Value, true);
		MakeUpper(Value);
		if(Value.length())
		{
			int iPasswordExpire = atoi(Value.c_str());
			if(iPasswordExpire < -1)
				iPasswordExpire = -1;
			CurrentSecurity->SetUserPasswordExpire(UserName.c_str(), iPasswordExpire);
		}
	}

	if(CommandToExecute->Parameters.KeyExist("PASSWORD_CHANGE_NEXT"))
	{
		string Value = CommandToExecute->Parameters.GetValue("PASSWORD_CHANGE_NEXT");
		CleanString(Value, true);
		MakeUpper(Value);
		if(Value == "YES")
			CurrentSecurity->SetUserPasswordChangeNext(UserName.c_str(), true);
		if(Value == "NO")
			CurrentSecurity->SetUserPasswordChangeNext(UserName.c_str(), false);
	}

	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY SET_USER_PASSWORD
bool NvSession::SecurityUserSetPassword(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string ActualUser = GetUserName();
	CleanString(ActualUser, true);
	MakeLower(ActualUser);
	
	string ActualSecuApp = Application->GetSecuApp();
	bool bExternal = false;
	if(ActualSecuApp.length())
	{
		if((ActualSecuApp != "(SYSTEM)") && (ActualSecuApp.find('[') != string::npos) && (ActualSecuApp.find(']') != string::npos))
			bExternal = true;
		if(bExternal && (ActualUser == "nvadmin"))
			bExternal = false;	// nvadmin user is always local
	}

	if(bExternal)
	{
		// Security on another server
		// We must send the order to the distant computer
		string::size_type index = ActualSecuApp.find('[');
		string ServerAdress = ActualSecuApp.substr(index+1);
		ActualSecuApp = ActualSecuApp.substr(0, index);
		CleanString(ActualSecuApp);
		index = ServerAdress.find(']');
		ServerAdress = ServerAdress.substr(0, index);
		CleanString(ServerAdress);
		bool bUseSsl = false;
		if(ServerAdress.length() >= 5)
		{
			string sUseSsl = ServerAdress.substr(ServerAdress.length()-5);
			MakeUpper(sUseSsl);
			if(sUseSsl == "(SSL)")
			{
				bUseSsl = true;
				ServerAdress = ServerAdress.substr(0, ServerAdress.length()-5);
			}
		}
		CleanString(ServerAdress);

		string OldPassword = CommandToExecute->Parameters.GetValue("PASSWORD_OLD");
		string NewPassword = CommandToExecute->Parameters.GetValue("PASSWORD_NEW");


		// Create a new temporary session just for login
		NvSession *SpecialSession = NVNEW NvSession;
		SpecialSession->Init();
		SpecialSession->m_bAdministrator = true;
		
		// We use request to connect to peer
		NvRequest SpecialRequest;
		SpecialRequest.Session = SpecialSession;
		SpecialRequest.b_UseSSL = bUseSsl;
		SpecialRequest.b_ClientUnicode = GetMain()->IsUnicode();

		SpecialRequest.m_sServer = ServerAdress;
		SpecialRequest.m_sAppli = ActualSecuApp;
		SpecialRequest.m_sOpen = "NV_SESSION_OPEN_NONE";
		SpecialRequest.m_sClose = "NV_SESSION_CLOSE_NONE";
		SpecialRequest.m_sUser = ActualUser;
		SpecialRequest.m_sPassword = OldPassword;

		CleanString(ServerAdress);
		CleanString(SpecialRequest.m_sAppli);
		CleanString(SpecialRequest.m_sSessionId);
		CleanString(SpecialRequest.m_sUser);
		CleanString(SpecialRequest.m_sPassword);
		SpecialRequest.m_sHash = SpecialRequest.m_sAppli + SpecialRequest.m_sUser + SpecialRequest.m_sPassword;
		SpecialRequest.m_sHash = MD5_string(SpecialRequest.m_sHash);
		MakeUpper(SpecialRequest.m_sHash);

		if(!ServerAdress.length())
			ServerAdress = "127.0.0.1";

		bool bVirtualModeLoadBalancing = false;
		int iVirtualNumAddresses = 0;
		if((ServerAdress[0] == '(') && (ServerAdress[ServerAdress.length()-1] == ')'))
		{
			// This is a virtual host address
			ServerAdress = ServerAdress.substr(1); 
			ServerAdress = ServerAdress.substr(0, ServerAdress.length()-1); 
			iVirtualNumAddresses = GetMain()->VirtualHosts.GetNumAddresses(ServerAdress.c_str());
			if(iVirtualNumAddresses != 0)
			{
				if(GetMain()->VirtualHosts.GetMode(ServerAdress.c_str()) == "LOAD_BALANCING")
					bVirtualModeLoadBalancing = true;
			}
		}
		if(!iVirtualNumAddresses)
			SpecialRequest.m_sServer = ServerAdress;

		SpecialRequest.m_iAutoClose = NV_REQUEST_CLOSE_SESSION;
		SpecialRequest.m_iSessionTimeOut = 60;
		SpecialRequest.m_iServerConnectionTimeOut = 10;
		SpecialRequest.b_Hidden = true;



		if(iVirtualNumAddresses)
		{
			// We must get the real address and test the server
			NvCommand TestCommand;
			TestCommand.m_pSession = this;
			TestCommand.m_iCommandSource = NV_SOURCE_CLIENT;
			TestCommand.m_sService = "SYSTEM";
			TestCommand.m_sClass = "MISC";
			TestCommand.m_sCommand = "NOP";
			bool bConnected = false;
			if(!bVirtualModeLoadBalancing)
			{
				// In failover mode, we first try to connect the active server
				SpecialRequest.m_sServer = GetMain()->VirtualHosts.GetActiveAddress(ServerAdress.c_str());
				if(!SpecialRequest.m_sServer.length())
					SpecialRequest.m_sServer = "127.0.0.1";
				if(SpecialRequest.RequestCommand(&TestCommand))
					bConnected = true;
				else
				{
					// If the error is not a network error
					// the peer server is running so we must return the error
					if((TestCommand.Error.m_sErrorService != "SYSTEM") ||
						(TestCommand.Error.m_sErrorClass != "REQUEST") ||
						(TestCommand.Error.m_iErrorCode != NV_REQUEST_NETWORK_ERROR))
					{
						CommandToExecute->Error.SetError(TestCommand.Error.m_sErrorService, TestCommand.Error.m_sErrorClass, TestCommand.Error.m_iErrorCode, TestCommand.Error.m_sErrorInfo);
						SpecialRequest.Clear();
						delete SpecialSession; 
						m_oVariables.Remove("PASSWORD_OLD");
						m_oVariables.Remove("PASSWORD_NEW");
						m_oVariables.Remove("PASSWORD_NEW_CONFIRM");
						return false;
					}
				}
			}
			while(!bConnected)
			{
				// We must try a connection to next address but no more than the number of addresses defined
				if(!iVirtualNumAddresses)
					break;
				iVirtualNumAddresses--;
				SpecialRequest.m_sServer = GetMain()->VirtualHosts.GetNextAddress(ServerAdress.c_str());
				if(!SpecialRequest.m_sServer.length())
					SpecialRequest.m_sServer = "127.0.0.1";
				if(SpecialRequest.RequestCommand(&TestCommand))
					bConnected = true;
				else
				{
					// If the error is not a network error
					// the peer server is running so we must return the error
					if((TestCommand.Error.m_sErrorService != "SYSTEM") ||
						(TestCommand.Error.m_sErrorClass != "REQUEST") ||
						(TestCommand.Error.m_iErrorCode != NV_REQUEST_NETWORK_ERROR))
					{
						CommandToExecute->Error.SetError(TestCommand.Error.m_sErrorService, TestCommand.Error.m_sErrorClass, TestCommand.Error.m_iErrorCode, TestCommand.Error.m_sErrorInfo);
						SpecialRequest.Clear();
						delete SpecialSession; 
						m_oVariables.Remove("PASSWORD_OLD");
						m_oVariables.Remove("PASSWORD_NEW");
						m_oVariables.Remove("PASSWORD_NEW_CONFIRM");
						return false;
					}
				}
			}
			if(!bConnected)
			{
				CommandToExecute->Error.SetError(TestCommand.Error.m_sErrorService, TestCommand.Error.m_sErrorClass, TestCommand.Error.m_iErrorCode, TestCommand.Error.m_sErrorInfo);
				SpecialRequest.Clear();
				delete SpecialSession; 
				m_oVariables.Remove("PASSWORD_OLD");
				m_oVariables.Remove("PASSWORD_NEW");
				m_oVariables.Remove("PASSWORD_NEW_CONFIRM");
				return false;
			}
		}

		NvCommand Command;
		SpecialSession->OriginalCommand = CommandToExecute;
		Command.m_pSession = SpecialSession;
		Command.m_iCommandSource = NV_SOURCE_CLIENT;
		Command.m_sService = "SYSTEM";
		Command.m_sClass = "SECURITY";
		Command.m_sCommand = "SET_USER_PASSWORD";
		Command.Parameters.SetValue("PASSWORD_OLD", OldPassword);
		Command.Parameters.SetValue("PASSWORD_NEW", NewPassword);
		if(CommandToExecute->Parameters.KeyExist("PASSWORD_NEW_CONFIRM"))
		{
			string NewPasswordConfirm = CommandToExecute->Parameters.GetValue("PASSWORD_NEW_CONFIRM");
			Command.Parameters.SetValue("PASSWORD_NEW_CONFIRM", NewPasswordConfirm);
		}
		SpecialSession->Application = NULL;

		if(!SpecialRequest.RequestCommand(&Command))
		{
			CommandToExecute->Error.SetError(Command.Error.m_sErrorService, Command.Error.m_sErrorClass, Command.Error.m_iErrorCode, Command.Error.m_sErrorInfo);
			SpecialRequest.Clear();
			delete SpecialSession; 
			m_oVariables.Remove("PASSWORD_OLD");
			m_oVariables.Remove("PASSWORD_NEW");
			m_oVariables.Remove("PASSWORD_NEW_CONFIRM");
			return false;
		}

		// Do cleanup now
		SpecialRequest.Clear();
		delete SpecialSession; 
		m_sPassword = NewPassword;
	}
	else
	{
		string SecurityService = Application->GetServiceSecurity();
		if(SecurityService.length())
		{
			// Security is defined on a dedicated service
			string OldPassword = CommandToExecute->Parameters.GetValue("PASSWORD_OLD");
			string NewPassword = CommandToExecute->Parameters.GetValue("PASSWORD_NEW");
			if(CommandToExecute->Parameters.KeyExist("PASSWORD_NEW_CONFIRM"))
			{
				string NewPasswordConfirm = CommandToExecute->Parameters.GetValue("PASSWORD_NEW_CONFIRM");
				if(NewPasswordConfirm != NewPassword)
				{
					// Bad password
					CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_BAD_NEW_PASSWORD);
					m_oVariables.Remove("PASSWORD_OLD");
					m_oVariables.Remove("PASSWORD_NEW");
					m_oVariables.Remove("PASSWORD_NEW_CONFIRM");
					return false;
				}
			}
			m_oVariables.Remove("PASSWORD_OLD");
			m_oVariables.Remove("PASSWORD_NEW");
			m_oVariables.Remove("PASSWORD_NEW_CONFIRM");


			// Create a new temporary session just for setting password
			NvSession *SpecialSession = NVNEW NvSession;
			SpecialSession->Init();
			SpecialSession->m_bAdministrator = true;
			SpecialSession->bHidden = true;
			SpecialSession->Application = Application;

			NvCommand Command;
			Command.m_pSession = SpecialSession;
			Command.m_iCommandSource = NV_SOURCE_SERVER;
			Command.m_sService = SecurityService;
			Command.m_sClass = "SECURITY";
			Command.m_sCommand = "CHANGE_PASSWORD";
			Command.Parameters.SetValue("PASSWORD", OldPassword);
			Command.Parameters.SetValue("NEW_PASSWORD", NewPassword);
			Command.Parameters.SetValue("USER", this->m_sUser);
			Command.Parameters.SetValue("APPLICATION", Application->GetName());

			bool SessionRetVal = true;
			try
			{
				SessionRetVal = SpecialSession->Command(&Command);
			}
			catch(...)
			{
				string Message = Command.m_sService;
				Message += ":";
				Message += Command.m_sClass;
				Message += ":";
				Message += Command.m_sCommand;
				Command.Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_EXCEPTION, Message);
				SessionRetVal = false;
			}
			if(!SessionRetVal)
			{
				delete SpecialSession; 
				CommandToExecute->Error.SetError(Command.Error.m_sErrorService, Command.Error.m_sErrorClass, Command.Error.m_iErrorCode, Command.Error.m_sErrorInfo);

				string ErrorClass = CommandToExecute->Error.m_sErrorClass;
				bool bModifyService = false;
				if(ErrorClass == "SECURITY")
				{
					if(Command.Error.m_iErrorCode == NV_SECURITY_CANNOT_GET_CONTEXT)
						bModifyService = true;
					if(Command.Error.m_iErrorCode == NV_SECURITY_BAD_PASSWORD)
						bModifyService = true;
					if(Command.Error.m_iErrorCode == NV_SECURITY_BAD_PASSWORD_SYNTAX)
						bModifyService = true;
				}
				if(bModifyService)
					CommandToExecute->Error.SetError("SYSTEM", Command.Error.m_sErrorClass, Command.Error.m_iErrorCode, Command.Error.m_sErrorInfo);
				return false;
			}
			// Do cleanup now
			delete SpecialSession; 
	
			m_sPassword = NewPassword;
		}
		else
		{
			// Normal local security
			string SecuApp;
			bool SystemSecurity;
			NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
			if(CurrentSecurity == NULL)
			{
				m_oVariables.Remove("PASSWORD_OLD");
				m_oVariables.Remove("PASSWORD_NEW");
				m_oVariables.Remove("PASSWORD_NEW_CONFIRM");
				return false;
			}

			string OldPassword = CommandToExecute->Parameters.GetValue("PASSWORD_OLD");
			CleanString(OldPassword, true);
			string RealOldPassword = OldPassword;
			if(OldPassword.length())
				OldPassword = MD5_string(OldPassword);
			MakeUpper(OldPassword);
			
			string NewPassword = CommandToExecute->Parameters.GetValue("PASSWORD_NEW");
			CleanString(NewPassword, true);
			string RealNewPassword = NewPassword;
			if(NewPassword.length())
				NewPassword = MD5_string(NewPassword);
			MakeUpper(NewPassword);

			if(CommandToExecute->Parameters.KeyExist("PASSWORD_NEW_CONFIRM"))
			{
				string NewPasswordConfirm = CommandToExecute->Parameters.GetValue("PASSWORD_NEW_CONFIRM");
				CleanString(NewPasswordConfirm, true);
				string RealNewPasswordConfirm = NewPasswordConfirm;
				if(NewPasswordConfirm.length())
					NewPasswordConfirm = MD5_string(NewPasswordConfirm);
				MakeUpper(NewPasswordConfirm);
				if(NewPasswordConfirm != NewPassword)
				{
					// Bad password
					CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_BAD_NEW_PASSWORD);
					FreeSecurity(SecuApp);
					m_oVariables.Remove("PASSWORD_OLD");
					m_oVariables.Remove("PASSWORD_NEW");
					m_oVariables.Remove("PASSWORD_NEW_CONFIRM");
					return false;
				}
			}

			string ActualPassword = CurrentSecurity->GetUserPassword(ActualUser.c_str());
			if(ActualUser == "nvadmin")
				ActualPassword = GetMain()->GetAdminPassword();

			
			if(!CheckPermission(CommandToExecute, "SYSTEM", "SECURITY_CHANGE_OWN_PASSWORD"))
			{
				FreeSecurity(SecuApp);
				m_oVariables.Remove("PASSWORD_OLD");
				m_oVariables.Remove("PASSWORD_NEW");
				m_oVariables.Remove("PASSWORD_NEW_CONFIRM");
				return false;
			}
			
			if(OldPassword.length() && (ActualPassword != OldPassword))
			{
				// Bad password
				CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_BAD_PASSWORD);
				FreeSecurity(SecuApp);
				m_oVariables.Remove("PASSWORD_OLD");
				m_oVariables.Remove("PASSWORD_NEW");
				m_oVariables.Remove("PASSWORD_NEW_CONFIRM");
				return false;
			}

			if(ActualUser == "nvadmin")
			{
				GetMain()->SetAdminPassword(NewPassword.c_str());
			}
			else
			{
				bool bNewPasswordProcResult = true;
				string ProcCheckPassword = CurrentSecurity->GetProcCheckPassword();
				if(ProcCheckPassword.length())
				{
					// Call here the application check password procedure
					// From a special administrator session
					// We do not need any login here
					NvSession Session;
					Session.Init();
					Session.SetAppName(Application->GetName().c_str());
					Session.Application = Application;
					Session.m_bAdministrator = true;

					NvCommand Command;
					Session.OriginalCommand = &Command;
					Command.m_pSession = &Session;
					Command.m_iCommandSource = NV_SOURCE_SERVER;
					Command.m_sService = "SYSTEM";
					Command.m_sClass = "MISC";
					Command.m_sCommand = "NOP";
					Command.Parameters.SetValue("OLD_PASSWORD", RealOldPassword);
					Command.Parameters.SetValue("NEW_PASSWORD", RealNewPassword);
					Command.Parameters.SetValue("USER", ActualUser);
					if(!Session.ExecuteProcedure(&Command, ProcCheckPassword, false))
						bNewPasswordProcResult = false;
					Session.Application = NULL;
				}
				
				if(!CurrentSecurity->UserExist(ActualUser.c_str()))
				{
					CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_INVALID_USER, ActualUser);
					FreeSecurity(SecuApp);
					m_oVariables.Remove("PASSWORD_OLD");
					m_oVariables.Remove("PASSWORD_NEW");
					m_oVariables.Remove("PASSWORD_NEW_CONFIRM");
					return false;
				}
				if(!bNewPasswordProcResult)
				{
					CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_BAD_PASSWORD_SYNTAX, ActualUser);
					FreeSecurity(SecuApp);
					m_oVariables.Remove("PASSWORD_OLD");
					m_oVariables.Remove("PASSWORD_NEW");
					m_oVariables.Remove("PASSWORD_NEW_CONFIRM");
					return false;
				}
				if(!CurrentSecurity->SetUserPassword(ActualUser.c_str(), NewPassword.c_str()))
				{
					CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_SET_PASSWORD, ActualUser);
					FreeSecurity(SecuApp);
					m_oVariables.Remove("PASSWORD_OLD");
					m_oVariables.Remove("PASSWORD_NEW");
					m_oVariables.Remove("PASSWORD_NEW_CONFIRM");
					return false;
				}
			}

			m_sPassword = RealNewPassword;

			FreeSecurity(SecuApp);
		}
	}
	m_oVariables.Remove("PASSWORD_OLD");
	m_oVariables.Remove("PASSWORD_NEW");
	m_oVariables.Remove("PASSWORD_NEW_CONFIRM");
	
	return true;

}




//SYSTEM SECURITY SET_OTHER_USER_PASSWORD
bool NvSession::SecurityUserSetOtherPassword(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string ActualUser = GetUserName();
	CleanString(ActualUser, true);
	MakeLower(ActualUser);

	string UserName = CommandToExecute->Parameters.GetValue("USER");
	CleanString(UserName, true);
	MakeLower(UserName);
	if(!UserName.length())
		UserName = ActualUser;
	
	string NewPassword = CommandToExecute->Parameters.GetValue("PASSWORD_NEW");
	CleanString(NewPassword, true);
	string RealNewPassword = NewPassword;
	if(NewPassword.length())
		 NewPassword = MD5_string(NewPassword);
	MakeUpper(NewPassword);

	if(!CheckPermission(CommandToExecute, "SYSTEM", "SECURITY_CHANGE_OTHER_PASSWORD"))
	{
		FreeSecurity(SecuApp);
		return false;
	}

	if(UserName == "nvadmin")
	{
		// Super user
		if(ActualUser != UserName)
		{
			// Only the nvadmin user can change its password
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_NOT_ALLOWED, "Only the nirva main administrator can change its password");
			FreeSecurity(SecuApp);
			return false;
		}
		GetMain()->SetAdminPassword(NewPassword.c_str());
	}
	else
	{
		bool bNewPasswordProcResult = true;
		string ProcCheckPassword = CurrentSecurity->GetProcCheckPassword();
		if(ProcCheckPassword.length())
		{
			// Call here the application check password procedure
			// From a special administrator session
			// We do not need any login here
			NvSession Session;
			Session.Init();
			Session.SetAppName(Application->GetName().c_str());
			Session.Application = Application;
			Session.m_bAdministrator = true;

			NvCommand Command;
			Session.OriginalCommand = &Command;
			Command.m_pSession = &Session;
			Command.m_iCommandSource = NV_SOURCE_SERVER;
			Command.m_sService = "SYSTEM";
			Command.m_sClass = "MISC";
			Command.m_sCommand = "NOP";
			Command.Parameters.SetValue("OLD_PASSWORD", "");
			Command.Parameters.SetValue("NEW_PASSWORD", RealNewPassword);
			Command.Parameters.SetValue("USER", UserName);
			if(!Session.ExecuteProcedure(&Command, ProcCheckPassword, false))
				bNewPasswordProcResult = false;
			Session.Application = NULL;
		}
		if(!CurrentSecurity->UserExist(UserName.c_str()))
		{
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_INVALID_USER, UserName);
			FreeSecurity(SecuApp);
			return false;
		}
		if(!bNewPasswordProcResult)
		{
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_BAD_PASSWORD_SYNTAX, UserName);
			FreeSecurity(SecuApp);
			return false;
		}
		if(!CurrentSecurity->SetUserPassword(UserName.c_str(), NewPassword.c_str()))
		{
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_SET_PASSWORD, UserName);
			FreeSecurity(SecuApp);
			return false;
		}
	}
	
	if(ActualUser == UserName)
		m_sPassword = RealNewPassword;

	FreeSecurity(SecuApp);
	return true;
}



//SYSTEM SECURITY SET_USER_ROLES
bool NvSession::SecurityUserSetRoles(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string UserName = CommandToExecute->Parameters.GetValue("USER");

	if(!CurrentSecurity->UserExist(UserName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_INVALID_USER, UserName);
		FreeSecurity(SecuApp);
		return false;
	}

	string Roles = CommandToExecute->Parameters.GetValue("ROLES");
	string sAppend = CommandToExecute->Parameters.GetValue("APPEND");
	CleanString(sAppend, true);
	MakeUpper(sAppend);
	bool bAppend = false;
	if(sAppend == "YES")
		bAppend = true;
	CurrentSecurity->SetUserRoles(UserName.c_str(), Roles.c_str(), bAppend);

	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY REMOVE_USER_ROLES
bool NvSession::SecurityUserRemoveRoles(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string UserName = CommandToExecute->Parameters.GetValue("USER");

	if(!CurrentSecurity->UserExist(UserName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_INVALID_USER, UserName);
		FreeSecurity(SecuApp);
		return false;
	}

	string Roles = CommandToExecute->Parameters.GetValue("ROLES");
	string sAll = CommandToExecute->Parameters.GetValue("ALL");
	CleanString(sAll, true);
	MakeUpper(sAll);
	bool bAll = false;
	if(sAll == "YES")
		bAll = true;
	CurrentSecurity->RemoveUserRoles(UserName.c_str(), Roles.c_str(), bAll);

	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY USER_PERMISSIONS
bool NvSession::SecurityUserPermissions(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	string UserName = CommandToExecute->Parameters.GetValue("USER");
	CleanString(UserName);

	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}


	if(!CurrentSecurity->UserExist(UserName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_INVALID_USER, UserName);
		FreeSecurity(SecuApp);
		return false;
	}
	
	OutContainer->RemoveObject("PERMISSION_LIST");
	if(!OutContainer->CreateObject("PERMISSION_LIST", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "PERMISSION_LIST");
		FreeSecurity(SecuApp);
		return false;
	}

	NvObjTable *Permissions = (NvObjTable *)(OutContainer->GetObject("PERMISSION_LIST"));
	Permissions->InsertColumn("NAME", "Permission name");
	Permissions->InsertColumn("DESCRIPTION", "Permission description");
	Permissions->InsertColumn("SELECTED");

	// First get application permissions if its not a system security
	if(!SystemSecurity)
	{
		if(!Application->GetPermissions(Permissions))
		{
			OutContainer->RemoveObject("PERMISSION_LIST");
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_GET_PERMISSIONS, UserName);
			FreeSecurity(SecuApp);
			return false;
		}
	}

	// Then get system permissions
	if(!GetMain()->GetPermissions(Permissions))
	{
		OutContainer->RemoveObject("PERMISSION_LIST");
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_GET_PERMISSIONS, UserName);
		FreeSecurity(SecuApp);
		return false;
	}

	// Then get service permissions
	if(!GetMain()->Services.GetPermissions(Permissions))
	{
		OutContainer->RemoveObject("PERMISSION_LIST");
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_GET_PERMISSIONS, UserName);
		FreeSecurity(SecuApp);
		return false;
	}

	// And finally get web service permissions
	if(!GetMain()->Webservices.GetPermissions(Permissions))
	{
		OutContainer->RemoveObject("PERMISSION_LIST");
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_GET_PERMISSIONS, UserName);
		FreeSecurity(SecuApp);
		return false;
	}

	NvObjIndStringList UserPermissions;
	if(!CurrentSecurity->GetUserPermissions(UserName.c_str(), &UserPermissions))
	{
		OutContainer->RemoveObject("PERMISSION_LIST");
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_GET_PERMISSIONS, UserName);
		FreeSecurity(SecuApp);
		return false;
	}

	FreeSecurity(SecuApp);

	int NumRows = Permissions->GetNumRow();
	int i;
	for(i=0; i<NumRows; i++)
	{
		string NameInTable = Permissions->GetCellLine(i, 0);
		MakeUpper(NameInTable);
		CleanString(NameInTable, true);
		if(!NameInTable.length())
			continue;
		if(UserPermissions.KeyExist(NameInTable))
			Permissions->InsertCellLine(i, 2, "YES");
		else
			Permissions->InsertCellLine(i, 2, "NO");
	}

	Permissions->RemoveRows("SELECTED=NO");
	Permissions->RemoveColumn(2);
	
	return true;
}



//SYSTEM SECURITY SESSION_PERMISSIONS
bool NvSession::SecuritySessionPermissions(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	string sAppOnly = CommandToExecute->Parameters.GetValue("APPONLY");
	CleanString(sAppOnly, true);
	MakeUpper(sAppOnly);
	bool bAppOnly = true;
	if(sAppOnly == "NO")
		bAppOnly = false;
	
	OutContainer->RemoveObject("PERMISSION_LIST");
	if(!OutContainer->CreateObject("PERMISSION_LIST", NV_OBJTYPE_STRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "PERMISSION_LIST");
		return false;
	}
	NvObjStringList *Permissions = (NvObjStringList *)(OutContainer->GetObject("PERMISSION_LIST"));

	map<string,string>::iterator pos;
	for(pos = m_oPermissions.m_mList.begin(); pos != m_oPermissions.m_mList.end(); ++pos)
	{
		string PermissionName = pos->first.c_str();
		MakeUpper(PermissionName);
		CleanString(PermissionName, true);
		if(!PermissionName.length())
			continue;
		if(bAppOnly)
			if(PermissionName.find(':') != string::npos)
				continue;
		Permissions->Add(PermissionName);
	}
	return true;
}



//SYSTEM SECURITY ADD_ROLE
bool NvSession::SecurityRoleAdd(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string RoleName = CommandToExecute->Parameters.GetValue("ROLE");
	string Description = CommandToExecute->Parameters.GetValue("DESCRIPTION");

	if(CurrentSecurity->RoleExist(RoleName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_ROLE_EXIST, RoleName);
		FreeSecurity(SecuApp);
		return false;
	}

	if(!CurrentSecurity->AddRole(RoleName.c_str(), Description.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_ADD_ROLE, RoleName);
		FreeSecurity(SecuApp);
		return false;
	}

	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY REMOVE_ROLE
bool NvSession::SecurityRoleRemove(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string RoleName = CommandToExecute->Parameters.GetValue("ROLE");

	CurrentSecurity->RemoveRole(RoleName.c_str());

	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY ROLE_LIST
bool NvSession::SecurityRoleList(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("ROLE_LIST");
	if(!OutContainer->CreateObject("ROLE_LIST", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "ROLE_LIST");
		return false;
	}

	NvObjTable *Roles = (NvObjTable *)(OutContainer->GetObject("ROLE_LIST"));

	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
	{
		OutContainer->RemoveObject("ROLE_LIST");
		return false;
	}
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			OutContainer->RemoveObject("ROLE_LIST");
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			OutContainer->RemoveObject("ROLE_LIST");
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string RoleName = CommandToExecute->Parameters.GetValue("ROLE");
	if(!CurrentSecurity->RoleList(Roles, RoleName.c_str()))
	{
		OutContainer->RemoveObject("ROLE_LIST");
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_LIST_ROLES, RoleName);
		FreeSecurity(SecuApp);
		return false;
	}

	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY ROLE_EXIST
bool NvSession::SecurityRoleExist(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string RoleName = CommandToExecute->Parameters.GetValue("ROLE");

	char Buf[20];
	strcpy(Buf, "NO");
	if(CurrentSecurity->RoleExist(RoleName.c_str()))
		strcpy(Buf, "YES");

	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);
	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY SET_ROLE_DESCRIPTION
bool NvSession::SecurityRoleSetDescription(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;

	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string RoleName = CommandToExecute->Parameters.GetValue("ROLE");

	if(!CurrentSecurity->RoleExist(RoleName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_INVALID_ROLE, RoleName);
		FreeSecurity(SecuApp);
		return false;
	}

	string Description = CommandToExecute->Parameters.GetValue("DESCRIPTION");
	CurrentSecurity->SetRoleDescription(RoleName.c_str(), Description.c_str());

	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY SET_ROLE_INHERITS
bool NvSession::SecurityRoleSetInherits(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string RoleName = CommandToExecute->Parameters.GetValue("ROLE");

	if(!CurrentSecurity->RoleExist(RoleName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_INVALID_ROLE, RoleName);
		FreeSecurity(SecuApp);
		return false;
	}

	string Inherits = CommandToExecute->Parameters.GetValue("INHERITS");
	string sAppend = CommandToExecute->Parameters.GetValue("APPEND");
	CleanString(sAppend, true);
	MakeUpper(sAppend);
	bool bAppend = false;
	if(sAppend == "YES")
		bAppend = true;
	CurrentSecurity->SetRoleInherits(RoleName.c_str(), Inherits.c_str(), bAppend);

	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY REMOVE_ROLE_INHERITS
bool NvSession::SecurityRoleRemoveInherits(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string RoleName = CommandToExecute->Parameters.GetValue("ROLE");

	if(!CurrentSecurity->RoleExist(RoleName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_INVALID_ROLE, RoleName);
		FreeSecurity(SecuApp);
		return false;
	}

	string Inherits = CommandToExecute->Parameters.GetValue("INHERITS");
	string sAll = CommandToExecute->Parameters.GetValue("ALL");
	CleanString(sAll, true);
	MakeUpper(sAll);
	bool bAll = false;
	if(sAll == "YES")
		bAll = true;
	CurrentSecurity->RemoveRoleInherits(RoleName.c_str(), Inherits.c_str(), bAll);

	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY SET_ROLE_PERMISSIONS
bool NvSession::SecurityRoleSetPermissions(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string RoleName = CommandToExecute->Parameters.GetValue("ROLE");

	if(!CurrentSecurity->RoleExist(RoleName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_INVALID_ROLE, RoleName);
		FreeSecurity(SecuApp);
		return false;
	}

	string Permissions = CommandToExecute->Parameters.GetValue("PERMISSIONS");
	string sAppend = CommandToExecute->Parameters.GetValue("APPEND");
	CleanString(sAppend, true);
	MakeUpper(sAppend);
	bool bAppend = false;
	if(sAppend == "YES")
		bAppend = true;
	CurrentSecurity->SetRolePermissions(RoleName.c_str(), Permissions.c_str(), bAppend);

	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY REMOVE_ROLE_PERMISSIONS
bool NvSession::SecurityRoleRemovePermissions(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string SecuApp;
	bool SystemSecurity;
	NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
	if(CurrentSecurity == NULL)
		return false;
	if(SystemSecurity)
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SYSTEM_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}
	else
	{
		if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SECURITY_ADMIN"))
		{
			FreeSecurity(SecuApp);
			return false;
		}
	}

	string RoleName = CommandToExecute->Parameters.GetValue("ROLE");

	if(!CurrentSecurity->RoleExist(RoleName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_INVALID_ROLE, RoleName);
		FreeSecurity(SecuApp);
		return false;
	}

	string Permissions = CommandToExecute->Parameters.GetValue("PERMISSIONS");
	string sAll = CommandToExecute->Parameters.GetValue("ALL");
	CleanString(sAll, true);
	MakeUpper(sAll);
	bool bAll = false;
	if(sAll == "YES")
		bAll = true;
	CurrentSecurity->RemoveRolePermissions(RoleName.c_str(), Permissions.c_str(), bAll);

	FreeSecurity(SecuApp);
	
	return true;
}




//SYSTEM SECURITY PERMISSION_LIST
bool NvSession::SecurityPermissionList(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

#ifdef PB_PERMISSION_LIST
	NV_LOCKLOCK(&ProtectConsoleLog);
	printf("NvSession::SecurityPermissionList 1\n");
	NV_UNLOCKLOCK(&ProtectConsoleLog);
#endif
	OutContainer->RemoveObject("PERMISSION_LIST");
	if(!OutContainer->CreateObject("PERMISSION_LIST", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "PERMISSION_LIST");
		return false;
	}
#ifdef PB_PERMISSION_LIST
	NV_LOCKLOCK(&ProtectConsoleLog);
	printf("NvSession::SecurityPermissionList 2\n");
	NV_UNLOCKLOCK(&ProtectConsoleLog);
#endif

	NvObjTable *Permissions = (NvObjTable *)(OutContainer->GetObject("PERMISSION_LIST"));
	Permissions->InsertColumn("NAME", "Permission name");
	Permissions->InsertColumn("DESCRIPTION", "Permission description");

	// First get application permissions if its not a system security
	string System = CommandToExecute->Parameters.GetValue("SYSTEM");
	CleanString(System, true);
	MakeUpper(System);
	if(System != "YES")
	{
#ifdef PB_PERMISSION_LIST
	NV_LOCKLOCK(&ProtectConsoleLog);
	printf("NvSession::SecurityPermissionList 3\n");
	NV_UNLOCKLOCK(&ProtectConsoleLog);
#endif
		if(!Application->GetPermissions(Permissions))
		{
			OutContainer->RemoveObject("PERMISSION_LIST");
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_GET_PERMISSIONS);
			return false;
		}
	}

#ifdef PB_PERMISSION_LIST
	NV_LOCKLOCK(&ProtectConsoleLog);
	printf("NvSession::SecurityPermissionList 4\n");
	NV_UNLOCKLOCK(&ProtectConsoleLog);
#endif
	// Then get system permissions
	if(!GetMain()->GetPermissions(Permissions))
	{
		OutContainer->RemoveObject("PERMISSION_LIST");
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_GET_PERMISSIONS);
		return false;
	}

#ifdef PB_PERMISSION_LIST
	NV_LOCKLOCK(&ProtectConsoleLog);
	printf("NvSession::SecurityPermissionList 5\n");
	NV_UNLOCKLOCK(&ProtectConsoleLog);
#endif
	// Then get service permissions
	if(!GetMain()->Services.GetPermissions(Permissions))
	{
		OutContainer->RemoveObject("PERMISSION_LIST");
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_GET_PERMISSIONS);
		return false;
	}

#ifdef PB_PERMISSION_LIST
	NV_LOCKLOCK(&ProtectConsoleLog);
	printf("NvSession::SecurityPermissionList 6\n");
	NV_UNLOCKLOCK(&ProtectConsoleLog);
#endif
	// And finally get web service permissions
	if(!GetMain()->Webservices.GetPermissions(Permissions))
	{
		OutContainer->RemoveObject("PERMISSION_LIST");
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_GET_PERMISSIONS);
		return false;
	}
#ifdef PB_PERMISSION_LIST
	NV_LOCKLOCK(&ProtectConsoleLog);
	printf("NvSession::SecurityPermissionList 7\n");
	NV_UNLOCKLOCK(&ProtectConsoleLog);
#endif

	return true;
}

//SYSTEM SECURITY ADD_SESSION_PERMISSION
bool NvSession::SecurityAddSessionPermission(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// This command is available only in the open session procedure (or a call from it)
	if(!bInOpenProc)
		return true;
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	string Permission = CommandToExecute->Parameters.GetValue("PERMISSION");
	CleanString(ServiceName);
	MakeUpper(ServiceName);
	CleanString(Permission);
	MakeUpper(Permission);
	string NewPermission = ServiceName;
	if(NewPermission.length())
		NewPermission += ":";
	NewPermission += Permission;
	if(NewPermission.length())
		m_oPermissions.SetValue(NewPermission,"");

	// Check if a role name has been given
	string RoleName = CommandToExecute->Parameters.GetValue("ROLE");
	CleanString(RoleName);
	MakeUpper(RoleName);
	if(RoleName.length())
	{
		// Yes there is also a role name so we must add all the permissions associated to this role
		NvObjTable Roles;

		string SecuApp;
		bool SystemSecurity;
		NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
		if(CurrentSecurity == NULL)
			return false;

		if(!CurrentSecurity->RoleList(&Roles, RoleName.c_str()))
		{
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_LIST_ROLES, RoleName);
			FreeSecurity(SecuApp);
			return false;
		}

		FreeSecurity(SecuApp);
		if(Roles.GetNumRow())
		{
			int PermissionColumnIndex = Roles.GetColumnIndex("PERMISSIONS");
			if(PermissionColumnIndex >=0 )
			{
				int NumPermissions = Roles.GetNumCellLines(0, PermissionColumnIndex);
				int i;
				for(i=0; i<NumPermissions; i++)
				{
					string Permission = Roles.GetCellLine(0, PermissionColumnIndex, i);
					CleanString(Permission);
					MakeUpper(Permission);
					if(Permission.length())
						m_oPermissions.SetValue(Permission,"");
				}
			}
		}
	}

	return true;
}


//SYSTEM SECURITY REMOVE_SESSION_PERMISSION
bool NvSession::SecurityRemoveSessionPermission(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// This command is available only in the open session procedure (or a call from it)
	if(!bInOpenProc)
		return true;
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	string Permission = CommandToExecute->Parameters.GetValue("PERMISSION");
	CleanString(ServiceName);
	MakeUpper(ServiceName);
	CleanString(Permission);
	MakeUpper(Permission);
	string PermissionToRemove = ServiceName;
	if(PermissionToRemove.length())
		PermissionToRemove += ":";
	PermissionToRemove += Permission;
	if(PermissionToRemove.length())
		m_oPermissions.Remove(PermissionToRemove);
	return true;
}

//SYSTEM SECURITY LOAD_USER_CONTEXT
bool NvSession::SecurityLoadUserContext(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string sUser = GetUserName();
	CleanString(sUser, true);
	MakeLower(sUser);
	if(!sUser.length())
		sUser = NV_SESSION_DEFAULT_USER;
	
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	string ActualSecuApp = Application->GetSecuApp();
	bool bExternal = false;
	if(ActualSecuApp.length())
	{
		if((ActualSecuApp != "(SYSTEM)") && (ActualSecuApp.find('[') != string::npos) && (ActualSecuApp.find(']') != string::npos))
			bExternal = true;
		if(bExternal && (sUser == "nvadmin"))
			bExternal = false;	// nvadmin user is always local
	}

	if(bExternal)
	{
		// Security on another server
		// We must send the order to the distant computer
		// Security on another server
		// We must send the order to the distant computer
		string::size_type index = ActualSecuApp.find('[');
		string ServerAdress = ActualSecuApp.substr(index+1);
		ActualSecuApp = ActualSecuApp.substr(0, index);
		CleanString(ActualSecuApp);
		index = ServerAdress.find(']');
		ServerAdress = ServerAdress.substr(0, index);
		CleanString(ServerAdress);
		bool bUseSsl = false;
		if(ServerAdress.length() >= 5)
		{
			string sUseSsl = ServerAdress.substr(ServerAdress.length()-5);
			MakeUpper(sUseSsl);
			if(sUseSsl == "(SSL)")
			{
				bUseSsl = true;
				ServerAdress = ServerAdress.substr(0, ServerAdress.length()-5);
			}
		}
		CleanString(ServerAdress);


		// Create a new temporary session just for login
		NvSession *SpecialSession = NVNEW NvSession;
		SpecialSession->Init();
		SpecialSession->m_bAdministrator = true;
		SpecialSession->Application = Application;
		
		// We use request to connect to peer
		NvRequest SpecialRequest;
		SpecialRequest.Session = SpecialSession;
		SpecialRequest.b_UseSSL = bUseSsl;
		SpecialRequest.b_ClientUnicode = GetMain()->IsUnicode();

		SpecialRequest.m_sServer = ServerAdress;
		SpecialRequest.m_sAppli = ActualSecuApp;
		SpecialRequest.m_sOpen = "NV_SESSION_OPEN_NONE";
		SpecialRequest.m_sClose = "NV_SESSION_CLOSE_NONE";
		SpecialRequest.m_sUser = sUser;
		SpecialRequest.m_sPassword = m_sPassword;

		CleanString(ServerAdress);
		CleanString(SpecialRequest.m_sAppli);
		CleanString(SpecialRequest.m_sSessionId);
		CleanString(SpecialRequest.m_sUser);
		CleanString(SpecialRequest.m_sPassword);
		SpecialRequest.m_sHash = SpecialRequest.m_sAppli + SpecialRequest.m_sUser + SpecialRequest.m_sPassword;
		SpecialRequest.m_sHash = MD5_string(SpecialRequest.m_sHash);
		MakeUpper(SpecialRequest.m_sHash);

		if(!ServerAdress.length())
			ServerAdress = "127.0.0.1";

		bool bVirtualModeLoadBalancing = false;
		int iVirtualNumAddresses = 0;
		if((ServerAdress[0] == '(') && (ServerAdress[ServerAdress.length()-1] == ')'))
		{
			// This is a virtual host address
			ServerAdress = ServerAdress.substr(1); 
			ServerAdress = ServerAdress.substr(0, ServerAdress.length()-1); 
			iVirtualNumAddresses = GetMain()->VirtualHosts.GetNumAddresses(ServerAdress.c_str());
			if(iVirtualNumAddresses != 0)
			{
				if(GetMain()->VirtualHosts.GetMode(ServerAdress.c_str()) == "LOAD_BALANCING")
					bVirtualModeLoadBalancing = true;
			}
		}
		if(!iVirtualNumAddresses)
			SpecialRequest.m_sServer = ServerAdress;

		SpecialRequest.m_iAutoClose = NV_REQUEST_CLOSE_SESSION;
		SpecialRequest.m_iSessionTimeOut = 60;
		SpecialRequest.m_iServerConnectionTimeOut = 10;
		SpecialRequest.b_Hidden = true;


		if(iVirtualNumAddresses)
		{
			// We must get the real address and test the server
			NvCommand TestCommand;
			TestCommand.m_pSession = this;
			TestCommand.m_iCommandSource = NV_SOURCE_CLIENT;
			TestCommand.m_sService = "SYSTEM";
			TestCommand.m_sClass = "MISC";
			TestCommand.m_sCommand = "NOP";
			bool bConnected = false;
			if(!bVirtualModeLoadBalancing)
			{
				// In failover mode, we first try to connect the active server
				SpecialRequest.m_sServer = GetMain()->VirtualHosts.GetActiveAddress(ServerAdress.c_str());
				if(!SpecialRequest.m_sServer.length())
					SpecialRequest.m_sServer = "127.0.0.1";
				if(SpecialRequest.RequestCommand(&TestCommand))
					bConnected = true;
				else
				{
					// If the error is not a network error
					// the peer server is running so we must return the error
					if((TestCommand.Error.m_sErrorService != "SYSTEM") ||
						(TestCommand.Error.m_sErrorClass != "REQUEST") ||
						(TestCommand.Error.m_iErrorCode != NV_REQUEST_NETWORK_ERROR))
					{
						CommandToExecute->Error.SetError(TestCommand.Error.m_sErrorService, TestCommand.Error.m_sErrorClass, TestCommand.Error.m_iErrorCode, TestCommand.Error.m_sErrorInfo);
						SpecialRequest.Clear();
						SpecialSession->Application = NULL;
						delete SpecialSession; 
						return false;
					}
				}
			}
			while(!bConnected)
			{
				// We must try a connection to next address but no more than the number of addresses defined
				if(!iVirtualNumAddresses)
					break;
				iVirtualNumAddresses--;
				SpecialRequest.m_sServer = GetMain()->VirtualHosts.GetNextAddress(ServerAdress.c_str());
				if(!SpecialRequest.m_sServer.length())
					SpecialRequest.m_sServer = "127.0.0.1";
				if(SpecialRequest.RequestCommand(&TestCommand))
					bConnected = true;
				else
				{
					// If the error is not a network error
					// the peer server is running so we must return the error
					if((TestCommand.Error.m_sErrorService != "SYSTEM") ||
						(TestCommand.Error.m_sErrorClass != "REQUEST") ||
						(TestCommand.Error.m_iErrorCode != NV_REQUEST_NETWORK_ERROR))
					{
						CommandToExecute->Error.SetError(TestCommand.Error.m_sErrorService, TestCommand.Error.m_sErrorClass, TestCommand.Error.m_iErrorCode, TestCommand.Error.m_sErrorInfo);
						SpecialRequest.Clear();
						SpecialSession->Application = NULL;
						delete SpecialSession; 
						return false;
					}
				}
			}
			if(!bConnected)
			{
				CommandToExecute->Error.SetError(TestCommand.Error.m_sErrorService, TestCommand.Error.m_sErrorClass, TestCommand.Error.m_iErrorCode, TestCommand.Error.m_sErrorInfo);
				SpecialRequest.Clear();
				SpecialSession->Application = NULL;
				delete SpecialSession; 
				return false;
			}
		}

		NvCommand Command;
		SpecialSession->OriginalCommand = CommandToExecute;
		Command.m_pSession = SpecialSession;
		Command.m_iCommandSource = NV_SOURCE_CLIENT;
		Command.m_sService = "SYSTEM";
		Command.m_sClass = "SECURITY";
		Command.m_sCommand = "LOAD_USER_CONTEXT";
		Command.Parameters.SetValue("NV_OUT_CONTAINER", "TMP");

		// Send the LOAD_USER_CONTEXT command to security server
		if(!SpecialRequest.RequestCommand(&Command))
		{
			CommandToExecute->Error.SetError(Command.Error.m_sErrorService, Command.Error.m_sErrorClass, Command.Error.m_iErrorCode, Command.Error.m_sErrorInfo);
			SpecialRequest.Clear();
			SpecialSession->Application = NULL;
			delete SpecialSession; 
			return false;
		}

		// Export the output container to a file object
		Command.m_pSession = SpecialSession;
		Command.m_iCommandSource = NV_SOURCE_CLIENT;
		Command.m_sService = "SYSTEM";
		Command.m_sClass = "OBJECT";
		Command.m_sCommand = "CREATE";
		Command.Parameters.RemoveAll();
		Command.Parameters.SetValue("NAME", "FILE");
		Command.Parameters.SetValue("TYPE", "FILE");
		if(!SpecialRequest.RequestCommand(&Command))
		{
			CommandToExecute->Error.SetError(Command.Error.m_sErrorService, Command.Error.m_sErrorClass, Command.Error.m_iErrorCode, Command.Error.m_sErrorInfo);
			SpecialRequest.Clear();
			SpecialSession->Application = NULL;
			delete SpecialSession; 
			return false;
		}

		Command.m_pSession = SpecialSession;
		Command.m_iCommandSource = NV_SOURCE_CLIENT;
		Command.m_sService = "SYSTEM";
		Command.m_sClass = "OBJECT";
		Command.m_sCommand = "FILE_GET_PATHNAME";
		Command.Parameters.RemoveAll();
		Command.Parameters.SetValue("NAME", "FILE");
		Command.Parameters.SetValue("NV_VAR", "FILENAME");
		if(!SpecialRequest.RequestCommand(&Command))
		{
			CommandToExecute->Error.SetError(Command.Error.m_sErrorService, Command.Error.m_sErrorClass, Command.Error.m_iErrorCode, Command.Error.m_sErrorInfo);
			SpecialRequest.Clear();
			SpecialSession->Application = NULL;
			delete SpecialSession; 
			return false;
		}

		Command.m_pSession = SpecialSession;
		Command.m_iCommandSource = NV_SOURCE_CLIENT;
		Command.m_sService = "SYSTEM";
		Command.m_sClass = "CONTAINER";
		Command.m_sCommand = "EXPORT";
		Command.Parameters.RemoveAll();
		Command.Parameters.SetValue("FILENAME", "#FILENAME");
		Command.Parameters.SetValue("SUBCONTAINERS", "YES");
		Command.Parameters.SetValue("NV_IN_CONTAINER", "TMP");
		if(!SpecialRequest.RequestCommand(&Command))
		{
			CommandToExecute->Error.SetError(Command.Error.m_sErrorService, Command.Error.m_sErrorClass, Command.Error.m_iErrorCode, Command.Error.m_sErrorInfo);
			SpecialRequest.Clear();
			SpecialSession->Application = NULL;
			delete SpecialSession; 
			return false;
		}

		NvContainer *LocalContainer = SpecialSession->Containers.GetCreateContainer("TTMMPP_JUST_FOR_THIS_COMMAND");
		if(LocalContainer != NULL)
		{
			SpecialSession->InContainer = LocalContainer;
			Command.m_pSession = SpecialSession;
			Command.m_iCommandSource = NV_SOURCE_CLIENT;
			Command.m_sService = "SYSTEM";
			Command.m_sClass = "OBJECT";
			Command.m_sCommand = "GET";
			Command.Parameters.RemoveAll();
			Command.Parameters.SetValue("NAME", "FILE");
			Command.Parameters.SetValue("NV_IN_CONTAINER", "TTMMPP_JUST_FOR_THIS_COMMAND");
			if(!SpecialRequest.RequestCommand(&Command))
			{
				CommandToExecute->Error.SetError(Command.Error.m_sErrorService, Command.Error.m_sErrorClass, Command.Error.m_iErrorCode, Command.Error.m_sErrorInfo);
				SpecialRequest.Clear();
				SpecialSession->Application = NULL;
				delete SpecialSession; 
				return false;
			}
			SpecialSession->InContainer = NULL;
		}

		// Do cleanup now
		SpecialRequest.Clear();

		// Now we have the exported container in local object named FILE in the container named TTMMPP_JUST_FOR_THIS_COMMAND
		if(LocalContainer != NULL)
		{
			// Should be always the case
			NvObjFile *ObjFile = (NvObjFile *)(LocalContainer->GetObject("FILE"));
			string FileName = "";
			if(ObjFile != NULL)
				FileName = ObjFile->GetFilePathName();
			if(FileName.c_str())
				OutContainer->Import(FileName.c_str(), NV_CONTAINER_COPY_NO_APPEND);
		}

		SpecialSession->Application = NULL;
		delete SpecialSession; 
		
		// Remove the local temporary container
		//Containers.RemoveContainer("TTMMPP_JUST_FOR_THIS_COMMAND");
	}
	else
	{
		string SecurityService = Application->GetServiceSecurity();
		if(SecurityService.length())
		{
			// Security is defined on a dedicated service
			// Create a new temporary session just for this command
			NvSession *SpecialSession = NVNEW NvSession;
			SpecialSession->Init();
			SpecialSession->m_bAdministrator = true;
			SpecialSession->bHidden = true;
			SpecialSession->Application = Application;

			NvCommand Command;
			Command.m_pSession = SpecialSession;
			Command.m_iCommandSource = NV_SOURCE_SERVER;
			Command.m_sService = SecurityService;
			Command.m_sClass = "SECURITY";
			Command.m_sCommand = "LOAD_USER_CONTEXT";
			Command.Parameters.SetValue("USER", this->m_sUser);
			Command.Parameters.SetValue("APPLICATION", Application->GetName());

			bool SessionRetVal = true;
			try
			{
				SessionRetVal = SpecialSession->Command(&Command);
			}
			catch(...)
			{
				string Message = Command.m_sService;
				Message += ":";
				Message += Command.m_sClass;
				Message += ":";
				Message += Command.m_sCommand;
				Command.Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_EXCEPTION, Message);
				SessionRetVal = false;
			}
			if(!SessionRetVal)
			{
				SpecialSession->Application = NULL;
				delete SpecialSession; 
				CommandToExecute->Error.SetError(Command.Error.m_sErrorService, Command.Error.m_sErrorClass, Command.Error.m_iErrorCode, Command.Error.m_sErrorInfo);

				string ErrorClass = CommandToExecute->Error.m_sErrorClass;
				bool bModifyService = false;
				if(ErrorClass == "SECURITY")
				{
					if(Command.Error.m_iErrorCode == NV_SECURITY_CANNOT_GET_CONTEXT)
						bModifyService = true;
					if(Command.Error.m_iErrorCode == NV_SECURITY_CANNOT_LOAD_USER_CONTEXT)
						bModifyService = true;
				}
				if(bModifyService)
					CommandToExecute->Error.SetError("SYSTEM", Command.Error.m_sErrorClass, Command.Error.m_iErrorCode, Command.Error.m_sErrorInfo);
				return false;
			}
			// If command was successful, we must copy its output container (nvdef) to the ouptut container of this command
			NvContainer *SpecialSessionContainer = SpecialSession->Containers.GetCreateContainer(NV_CONTAINER_DEFAULT_NAME);
			OutContainer->CopyFrom(SpecialSessionContainer, NV_CONTAINER_COPY_NO_APPEND, NULL, true);
			
			SpecialSession->Application = NULL;
			delete SpecialSession; 


		}
		else
		{
			// Normal local security
			string SecuApp;
			bool SystemSecurity;
			NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
			if(CurrentSecurity == NULL)
				return false;
			
			if(!CurrentSecurity->LoadUserContext(sUser.c_str(), InContainer))
			{
				CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_LOAD_USER_CONTEXT, sUser);
				FreeSecurity(SecuApp);
				return false;
			}
			FreeSecurity(SecuApp);
		}
	}
	
	return true;
}

//SYSTEM SECURITY SAVE_USER_CONTEXT
bool NvSession::SecuritySaveUserContext(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string sUser = GetUserName();
	CleanString(sUser, true);
	MakeLower(sUser);
	if(!sUser.length())
		sUser = NV_SESSION_DEFAULT_USER;
	
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	string ActualSecuApp = Application->GetSecuApp();
	bool bExternal = false;
	if(ActualSecuApp.length())
	{
		if((ActualSecuApp != "(SYSTEM)") && (ActualSecuApp.find('[') != string::npos) && (ActualSecuApp.find(']') != string::npos))
			bExternal = true;
		if(bExternal && (sUser == "nvadmin"))
			bExternal = false;	// nvadmin user is always local
	}

	if(bExternal)
	{
		// Security on another server
		// We must send the order to the distant computer


		string::size_type index = ActualSecuApp.find('[');
		string ServerAdress = ActualSecuApp.substr(index+1);
		ActualSecuApp = ActualSecuApp.substr(0, index);
		CleanString(ActualSecuApp);
		index = ServerAdress.find(']');
		ServerAdress = ServerAdress.substr(0, index);
		CleanString(ServerAdress);
		bool bUseSsl = false;
		if(ServerAdress.length() >= 5)
		{
			string sUseSsl = ServerAdress.substr(ServerAdress.length()-5);
			MakeUpper(sUseSsl);
			if(sUseSsl == "(SSL)")
			{
				bUseSsl = true;
				ServerAdress = ServerAdress.substr(0, ServerAdress.length()-5);
			}
		}
		CleanString(ServerAdress);


		// Create a new temporary session just for login
		NvSession *SpecialSession = NVNEW NvSession;
		SpecialSession->Init();
		SpecialSession->m_bAdministrator = true;
		SpecialSession->Application = Application;
		
		// We use request to connect to peer
		NvRequest SpecialRequest;
		SpecialRequest.Session = SpecialSession;
		SpecialRequest.b_UseSSL = bUseSsl;
		SpecialRequest.b_ClientUnicode = GetMain()->IsUnicode();

		SpecialRequest.m_sServer = ServerAdress;
		SpecialRequest.m_sAppli = ActualSecuApp;
		SpecialRequest.m_sOpen = "NV_SESSION_OPEN_NONE";
		SpecialRequest.m_sClose = "NV_SESSION_CLOSE_NONE";
		SpecialRequest.m_sUser = sUser;
		SpecialRequest.m_sPassword = m_sPassword;

		CleanString(ServerAdress);
		CleanString(SpecialRequest.m_sAppli);
		CleanString(SpecialRequest.m_sSessionId);
		CleanString(SpecialRequest.m_sUser);
		CleanString(SpecialRequest.m_sPassword);
		SpecialRequest.m_sHash = SpecialRequest.m_sAppli + SpecialRequest.m_sUser + SpecialRequest.m_sPassword;
		SpecialRequest.m_sHash = MD5_string(SpecialRequest.m_sHash);
		MakeUpper(SpecialRequest.m_sHash);

		if(!ServerAdress.length())
			ServerAdress = "127.0.0.1";

		bool bVirtualModeLoadBalancing = false;
		int iVirtualNumAddresses = 0;
		if((ServerAdress[0] == '(') && (ServerAdress[ServerAdress.length()-1] == ')'))
		{
			// This is a virtual host address
			ServerAdress = ServerAdress.substr(1); 
			ServerAdress = ServerAdress.substr(0, ServerAdress.length()-1); 
			iVirtualNumAddresses = GetMain()->VirtualHosts.GetNumAddresses(ServerAdress.c_str());
			if(iVirtualNumAddresses != 0)
			{
				if(GetMain()->VirtualHosts.GetMode(ServerAdress.c_str()) == "LOAD_BALANCING")
					bVirtualModeLoadBalancing = true;
			}
		}
		if(!iVirtualNumAddresses)
			SpecialRequest.m_sServer = ServerAdress;

		SpecialRequest.m_iAutoClose = NV_REQUEST_CLOSE_SESSION;
		SpecialRequest.m_iSessionTimeOut = 60;
		SpecialRequest.m_iServerConnectionTimeOut = 10;
		SpecialRequest.b_Hidden = true;


		if(iVirtualNumAddresses)
		{
			// We must get the real address and test the server
			NvCommand TestCommand;
			TestCommand.m_pSession = this;
			TestCommand.m_iCommandSource = NV_SOURCE_CLIENT;
			TestCommand.m_sService = "SYSTEM";
			TestCommand.m_sClass = "MISC";
			TestCommand.m_sCommand = "NOP";
			bool bConnected = false;
			if(!bVirtualModeLoadBalancing)
			{
				// In failover mode, we first try to connect the active server
				SpecialRequest.m_sServer = GetMain()->VirtualHosts.GetActiveAddress(ServerAdress.c_str());
				if(!SpecialRequest.m_sServer.length())
					SpecialRequest.m_sServer = "127.0.0.1";
				if(SpecialRequest.RequestCommand(&TestCommand))
					bConnected = true;
				else
				{
					// If the error is not a network error
					// the peer server is running so we must return the error
					if((TestCommand.Error.m_sErrorService != "SYSTEM") ||
						(TestCommand.Error.m_sErrorClass != "REQUEST") ||
						(TestCommand.Error.m_iErrorCode != NV_REQUEST_NETWORK_ERROR))
					{
						CommandToExecute->Error.SetError(TestCommand.Error.m_sErrorService, TestCommand.Error.m_sErrorClass, TestCommand.Error.m_iErrorCode, TestCommand.Error.m_sErrorInfo);
						SpecialRequest.Clear();
						SpecialSession->Application = NULL;
						delete SpecialSession; 
						return false;
					}
				}
			}
			while(!bConnected)
			{
				// We must try a connection to next address but no more than the number of addresses defined
				if(!iVirtualNumAddresses)
					break;
				iVirtualNumAddresses--;
				SpecialRequest.m_sServer = GetMain()->VirtualHosts.GetNextAddress(ServerAdress.c_str());
				if(!SpecialRequest.m_sServer.length())
					SpecialRequest.m_sServer = "127.0.0.1";
				if(SpecialRequest.RequestCommand(&TestCommand))
					bConnected = true;
				else
				{
					// If the error is not a network error
					// the peer server is running so we must return the error
					if((TestCommand.Error.m_sErrorService != "SYSTEM") ||
						(TestCommand.Error.m_sErrorClass != "REQUEST") ||
						(TestCommand.Error.m_iErrorCode != NV_REQUEST_NETWORK_ERROR))
					{
						CommandToExecute->Error.SetError(TestCommand.Error.m_sErrorService, TestCommand.Error.m_sErrorClass, TestCommand.Error.m_iErrorCode, TestCommand.Error.m_sErrorInfo);
						SpecialRequest.Clear();
						SpecialSession->Application = NULL;
						delete SpecialSession; 
						return false;
					}
				}
			}
			if(!bConnected)
			{
				CommandToExecute->Error.SetError(TestCommand.Error.m_sErrorService, TestCommand.Error.m_sErrorClass, TestCommand.Error.m_iErrorCode, TestCommand.Error.m_sErrorInfo);
				SpecialRequest.Clear();
				SpecialSession->Application = NULL;
				delete SpecialSession; 
				return false;
			}
		}

		// First export the input container to a file object named FILE in a temporary container named TTMMPP_JUST_FOR_THIS_COMMAND
		NvContainer *LocalContainer = SpecialSession->Containers.GetCreateContainer("TTMMPP_JUST_FOR_THIS_COMMAND");
		if(LocalContainer != NULL)
		{
			// Should be always the case

			// We create an object named FILE in the temp container
			LocalContainer->CreateObject("FILE", NV_OBJTYPE_FILE);
			string FileName = "";
			NvObjFile *ObjFile = (NvObjFile *)(LocalContainer->GetObject("FILE"));
			if(ObjFile != NULL)
			{
				ObjFile->SetAutoFileName(Application->GetWorkDir());
				FileName = ObjFile->GetFilePathName();
			}
			if(FileName.c_str())
				InContainer->Export(FileName.c_str());
		}

		NvCommand Command;
		SpecialSession->OriginalCommand = CommandToExecute;
		SpecialSession->OutContainer = LocalContainer;
		
		Command.m_pSession = SpecialSession;
		Command.m_iCommandSource = NV_SOURCE_CLIENT;
		Command.m_sService = "SYSTEM";
		Command.m_sClass = "OBJECT";
		Command.m_sCommand = "SEND";
		Command.Parameters.RemoveAll();
		Command.Parameters.SetValue("NAME", "FILE");
		Command.Parameters.SetValue("NV_OUT_CONTAINER", "TTMMPP_JUST_FOR_THIS_COMMAND");
		if(!SpecialRequest.RequestCommand(&Command))
		{
			CommandToExecute->Error.SetError(Command.Error.m_sErrorService, Command.Error.m_sErrorClass, Command.Error.m_iErrorCode, Command.Error.m_sErrorInfo);
			SpecialRequest.Clear();
			SpecialSession->Application = NULL;
			delete SpecialSession; 
			return false;
		}
		SpecialSession->OutContainer = NULL;
		
		// Now the file object is the distant server
		// we must import it into a TMP container

		Command.m_pSession = SpecialSession;
		Command.m_iCommandSource = NV_SOURCE_CLIENT;
		Command.m_sService = "SYSTEM";
		Command.m_sClass = "OBJECT";
		Command.m_sCommand = "FILE_GET_PATHNAME";
		Command.Parameters.RemoveAll();
		Command.Parameters.SetValue("NAME", "FILE");
		Command.Parameters.SetValue("NV_VAR", "FILENAME");
		if(!SpecialRequest.RequestCommand(&Command))
		{
			CommandToExecute->Error.SetError(Command.Error.m_sErrorService, Command.Error.m_sErrorClass, Command.Error.m_iErrorCode, Command.Error.m_sErrorInfo);
			SpecialRequest.Clear();
			SpecialSession->Application = NULL;
			delete SpecialSession; 
			return false;
		}

		Command.m_pSession = SpecialSession;
		Command.m_iCommandSource = NV_SOURCE_CLIENT;
		Command.m_sService = "SYSTEM";
		Command.m_sClass = "CONTAINER";
		Command.m_sCommand = "IMPORT";
		Command.Parameters.RemoveAll();
		Command.Parameters.SetValue("FILENAME", "#FILENAME");
		Command.Parameters.SetValue("NV_OUT_CONTAINER", "TMP");
		if(!SpecialRequest.RequestCommand(&Command))
		{
			CommandToExecute->Error.SetError(Command.Error.m_sErrorService, Command.Error.m_sErrorClass, Command.Error.m_iErrorCode, Command.Error.m_sErrorInfo);
			SpecialRequest.Clear();
			SpecialSession->Application = NULL;
			delete SpecialSession; 
			return false;
		}


		// Send the SAVE_USER_CONTEXT command to security server
		Command.m_pSession = SpecialSession;
		Command.m_iCommandSource = NV_SOURCE_CLIENT;
		Command.m_sService = "SYSTEM";
		Command.m_sClass = "SECURITY";
		Command.m_sCommand = "SAVE_USER_CONTEXT";
		Command.Parameters.SetValue("NV_IN_CONTAINER", "TMP");
		SpecialSession->Application = NULL;
		if(!SpecialRequest.RequestCommand(&Command))
		{
			CommandToExecute->Error.SetError(Command.Error.m_sErrorService, Command.Error.m_sErrorClass, Command.Error.m_iErrorCode, Command.Error.m_sErrorInfo);
			SpecialRequest.Clear();
			SpecialSession->Application = NULL;
			delete SpecialSession; 
			return false;
		}

		// Do cleanup now
		SpecialRequest.Clear();
		SpecialSession->Application = NULL;
		delete SpecialSession; 

		// Remove the local temporary container
		//Containers.RemoveContainer("TTMMPP_JUST_FOR_THIS_COMMAND");
	}
	else
	{
		string SecurityService = Application->GetServiceSecurity();
		if(SecurityService.length())
		{
			// Security is defined on a dedicated service
			// Create a new temporary session just for this command
			NvSession *SpecialSession = NVNEW NvSession;
			SpecialSession->Init();
			SpecialSession->m_bAdministrator = true;
			SpecialSession->bHidden = true;
			SpecialSession->Application = Application;

			// We must copy the input container the special session input container (nvdef)
			NvContainer *SpecialSessionContainer = SpecialSession->Containers.GetCreateContainer(NV_CONTAINER_DEFAULT_NAME);
			SpecialSessionContainer->CopyFrom(InContainer, NV_CONTAINER_COPY_NO_APPEND, NULL, true);

			NvCommand Command;
			Command.m_pSession = SpecialSession;
			Command.m_iCommandSource = NV_SOURCE_SERVER;
			Command.m_sService = SecurityService;
			Command.m_sClass = "SECURITY";
			Command.m_sCommand = "SAVE_USER_CONTEXT";
			Command.Parameters.SetValue("USER", this->m_sUser);
			Command.Parameters.SetValue("APPLICATION", Application->GetName());

			bool SessionRetVal = true;
			try
			{
				SessionRetVal = SpecialSession->Command(&Command);
			}
			catch(...)
			{
				string Message = Command.m_sService;
				Message += ":";
				Message += Command.m_sClass;
				Message += ":";
				Message += Command.m_sCommand;
				Command.Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_EXCEPTION, Message);
				SessionRetVal = false;
			}
			SpecialSession->Application = NULL;
			delete SpecialSession; 
			if(!SessionRetVal)
			{
				CommandToExecute->Error.SetError(Command.Error.m_sErrorService, Command.Error.m_sErrorClass, Command.Error.m_iErrorCode, Command.Error.m_sErrorInfo);

				string ErrorClass = CommandToExecute->Error.m_sErrorClass;
				bool bModifyService = false;
				if(ErrorClass == "SECURITY")
				{
					if(Command.Error.m_iErrorCode == NV_SECURITY_CANNOT_GET_CONTEXT)
						bModifyService = true;
					if(Command.Error.m_iErrorCode == NV_SECURITY_CANNOT_SAVE_USER_CONTEXT)
						bModifyService = true;
				}
				if(bModifyService)
					CommandToExecute->Error.SetError("SYSTEM", Command.Error.m_sErrorClass, Command.Error.m_iErrorCode, Command.Error.m_sErrorInfo);
				return false;
			}
		}
		else
		{
			// Normal local security
			string SecuApp;
			bool SystemSecurity;
			NvSecurity *CurrentSecurity = GetSecurity(CommandToExecute, SecuApp, SystemSecurity);
			if(CurrentSecurity == NULL)
				return false;
			
			if(!CurrentSecurity->SaveUserContext(sUser.c_str(), InContainer))
			{
				CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_CANNOT_SAVE_USER_CONTEXT, sUser);
				FreeSecurity(SecuApp);
				return false;
			}

			FreeSecurity(SecuApp);
		}
	}
	
	return true;
}


//SYSTEM MQ ISAVAILABLE
bool NvSession::MqIsAvailable(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	char Buf[20];
	strcpy(Buf, "NO");
	if(GetMain()->IsMqAvailable())
		strcpy(Buf, "YES");

	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);
	
	return true;
}



//SYSTEM MQ CREATE
bool NvSession::MqCreate(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "MQ_ADMIN"))
		return false;

	// Get variable name that should be in command parameters
	string Name = CommandToExecute->Parameters.GetValue("QUEUE");
	CleanString(Name);

	if(!Name.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "MQ:CREATE:QUEUE");
		return false;
	}

	if(GetMain()->Queues.QueueExist(Name.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "MQ", NV_MQ_EVER_EXIST, Name);
		return false;
	}

	string Description = CommandToExecute->Parameters.GetValue("DESCRIPTION");
	string QueueManager = CommandToExecute->Parameters.GetValue("MQ_MANAGER");
	CleanString(QueueManager);
	if(!QueueManager.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "MQ:CREATE:MQ_MANAGER");
		return false;
	}
	string QueueName = CommandToExecute->Parameters.GetValue("MQ_QUEUE");
	if(!QueueName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "MQ:CREATE:MQ_QUEUE");
		return false;
	}
	CleanString(QueueName);
	string Filter = CommandToExecute->Parameters.GetValue("FILTER");
	CleanString(Filter);
	if(Filter.length() > 23)
		Filter = Filter.substr(0, 23);
	string Channel = CommandToExecute->Parameters.GetValue("CHANNEL");
	CleanString(Channel);
	string Server = CommandToExecute->Parameters.GetValue("SERVER");
	CleanString(Server);
	string NumThreads = CommandToExecute->Parameters.GetValue("NUM_THREADS");
	CleanString(NumThreads, true);
	int iNumThreads = 1;
	if(NumThreads.length())
		iNumThreads = atoi(NumThreads.c_str());
	if(iNumThreads < 1)
		iNumThreads = 1;
	if(iNumThreads > 100)
		iNumThreads = 100;
	string AutoStart = CommandToExecute->Parameters.GetValue("AUTO_START");
	CleanString(AutoStart, true);
	MakeUpper(AutoStart);
	int iAutoStart = 1;
	if(AutoStart == "NO")
		iAutoStart = 0;

	if(!GetMain()->Queues.CreateQueue(Name.c_str(), Description.c_str(), QueueManager.c_str(), QueueName.c_str(), Filter.c_str(), iNumThreads, iAutoStart, Channel.c_str(), Server.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "MQ", NV_MQ_CANNOT_CREATE, Name);
		return false;
	}

	return true;
}



//SYSTEM MQ SET_PARAM
bool NvSession::MqSetParam(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "MQ_ADMIN"))
		return false;

	// Get variable name that should be in command parameters
	string Name = CommandToExecute->Parameters.GetValue("QUEUE");
	CleanString(Name);

	if(!Name.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "MQ:CREATE:QUEUE");
		return false;
	}

	if(!GetMain()->Queues.QueueExist(Name.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "MQ", NV_MQ_INVALID_QUEUE, Name);
		return false;
	}

	if(GetMain()->Queues.IsQueueRunning(Name.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "MQ", NV_MQ_QUEUE_RUNNING, Name);
		return false;
	}

	const char *pDescription = NULL;
	string Description;
	if(CommandToExecute->Parameters.KeyExist("DESCRIPTION"))
	{
		Description = CommandToExecute->Parameters.GetValue("DESCRIPTION");
		pDescription = Description.c_str();
	}
	const char *pQueueManager = NULL;
	string QueueManager;
	if(CommandToExecute->Parameters.KeyExist("MQ_MANAGER"))
	{
		QueueManager = CommandToExecute->Parameters.GetValue("MQ_MANAGER");
		CleanString(QueueManager);
		pQueueManager = QueueManager.c_str();
	}
	const char *pQueueName = NULL;
	string QueueName;
	if(CommandToExecute->Parameters.KeyExist("MQ_QUEUE"))
	{
		QueueName = CommandToExecute->Parameters.GetValue("MQ_QUEUE");
		CleanString(QueueName);
		pQueueName = QueueName.c_str();
	}
	const char *pFilter = NULL;
	string Filter;
	if(CommandToExecute->Parameters.KeyExist("FILTER"))
	{
		Filter = CommandToExecute->Parameters.GetValue("FILTER");
		CleanString(Filter);
		if(Filter.length() > 23)
			Filter = Filter.substr(0, 23);
		pFilter = Filter.c_str();
	}
	const char *pChannel = NULL;
	string Channel;
	if(CommandToExecute->Parameters.KeyExist("CHANNEL"))
	{
		Channel = CommandToExecute->Parameters.GetValue("CHANNEL");
		CleanString(Channel);
		pChannel = Channel.c_str();
	}
	const char *pServer = NULL;
	string Server;
	if(CommandToExecute->Parameters.KeyExist("SERVER"))
	{
		Server = CommandToExecute->Parameters.GetValue("SERVER");
		CleanString(Server);
		pServer = Server.c_str();
	}
	int iNumThreads = -1;
	if(CommandToExecute->Parameters.KeyExist("NUM_THREADS"))
	{
		string NumThreads = CommandToExecute->Parameters.GetValue("NUM_THREADS");
		CleanString(NumThreads, true);
		if(NumThreads.length())
			iNumThreads = atoi(NumThreads.c_str());
		if(iNumThreads < 1)
			iNumThreads = 1;
		if(iNumThreads > 100)
			iNumThreads = 100;
	}
	int iAutoStart = -1;
	if(CommandToExecute->Parameters.KeyExist("AUTO_START"))
	{
		string AutoStart = CommandToExecute->Parameters.GetValue("AUTO_START");
		CleanString(AutoStart, true);
		MakeUpper(AutoStart);
		iAutoStart = 1;
		if(AutoStart == "NO")
			iAutoStart = 0;
	}

	if(!GetMain()->Queues.SetQueueParam(Name.c_str(), pDescription, pQueueManager, pQueueName, pFilter, iNumThreads, iAutoStart, pChannel, pServer))
	{
		CommandToExecute->Error.SetError("SYSTEM", "MQ", NV_MQ_CANNOT_SET_PARAM, Name);
		return false;
	}

	return true;
}



//SYSTEM MQ REMOVE
bool NvSession::MqRemove(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "MQ_ADMIN"))
		return false;

	// Get variable name that should be in command parameters
	string Name = CommandToExecute->Parameters.GetValue("QUEUE");
	CleanString(Name);

	if(!Name.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "MQ:CREATE:QUEUE");
		return false;
	}

	if(!GetMain()->Queues.QueueExist(Name.c_str()))
		return true;	// Nothing to do in this case


	if(GetMain()->Queues.IsQueueRunning(Name.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "MQ", NV_MQ_QUEUE_RUNNING, Name);
		return false;
	}

	if(!GetMain()->Queues.RemoveQueue(Name.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "MQ", NV_MQ_CANNOT_REMOVE, Name);
		return false;
	}

	return true;
}



//SYSTEM MQ START
bool NvSession::MqStart(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "MQ_ADMIN"))
		return false;

	// Get variable name that should be in command parameters
	string Name = CommandToExecute->Parameters.GetValue("QUEUE");
	CleanString(Name);

	if(!Name.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "MQ:CREATE:QUEUE");
		return false;
	}

	if(!GetMain()->Queues.QueueExist(Name.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "MQ", NV_MQ_INVALID_QUEUE, Name);
		return false;
	}

	if(!GetMain()->IsMqAvailable())
	{
		CommandToExecute->Error.SetError("SYSTEM", "MQ", NV_MQ_NOT_RUNNING, Name);
		return false;
	}

	if(!GetMain()->Queues.StartQueue(Name.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "MQ", NV_MQ_CANNOT_START, Name);
		return false;
	}

	return true;
}



//SYSTEM MQ STOP
bool NvSession::MqStop(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "MQ_ADMIN"))
		return false;

	// Get variable name that should be in command parameters
	string Name = CommandToExecute->Parameters.GetValue("QUEUE");
	CleanString(Name);

	if(!Name.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "MQ:CREATE:QUEUE");
		return false;
	}

	if(!GetMain()->Queues.QueueExist(Name.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "MQ", NV_MQ_INVALID_QUEUE, Name);
		return false;
	}

	if(!GetMain()->IsMqAvailable())
	{
		CommandToExecute->Error.SetError("SYSTEM", "MQ", NV_MQ_NOT_RUNNING, Name);
		return false;
	}

	int iWaitTime = 10;
	string sWaitTime = CommandToExecute->Parameters.GetValue("WAIT");
	CleanString(sWaitTime);
	if(sWaitTime.length())
		iWaitTime = atoi(sWaitTime.c_str());
	if(iWaitTime < 0)
		iWaitTime = 0;

	bool bForce = false;
	string sForce = CommandToExecute->Parameters.GetValue("FORCE");
	CleanString(sForce);
	MakeUpper(sForce);
	if(sForce == "YES")
		bForce = true;


	if(!GetMain()->Queues.StopQueue(Name.c_str(), bForce, iWaitTime))
	{
		CommandToExecute->Error.SetError("SYSTEM", "MQ", NV_MQ_CANNOT_STOP, Name);
		return false;
	}

	return true;
}


//SYSTEM MQ LIST
bool NvSession::MqList(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string Name = CommandToExecute->Parameters.GetValue("QUEUE");
	CleanString(Name);

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("QUEUE_LIST");
	if(!OutContainer->CreateObject("QUEUE_LIST", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "QUEUE_LIST");
		return false;
	}

	NvObjTable *QueueInfo = (NvObjTable *)(OutContainer->GetObject("QUEUE_LIST"));



	if(!GetMain()->Queues.QueueList(Name.c_str(), QueueInfo))
	{
		CommandToExecute->Error.SetError("SYSTEM", "MQ", NV_MQ_CANNOT_LIST, Name);
		return false;
	}

	return true;
}




//SYSTEM MQ EXIST
bool NvSession::MqExist(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	string Name = CommandToExecute->Parameters.GetValue("QUEUE");
	CleanString(Name);

	char Buf[20];
	strcpy(Buf, "NO");
	if(Name.length() && GetMain()->Queues.QueueExist(Name.c_str()))
		strcpy(Buf, "YES");

	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}



//SYSTEM TRANSACTION START
bool NvSession::TransactionStart(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(CurrentTransaction != NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_TRANSACTION_CANNOT_START_FROM_TRANS, "");
		return false;
	}


	// Get variable name that should be in command parameters
	string TransactionProc = CommandToExecute->Parameters.GetValue("PROC_TRANS");
	CleanString(TransactionProc);

	if(!TransactionProc.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "TRANSACTION:START:PROC_TRANS");
		return false;
	}

	string TransactionName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(TransactionProc);
	string TransactionDesc = CommandToExecute->Parameters.GetValue("DESCRIPTION");
	CleanString(TransactionProc);

	string StartDate = "";
	string StartTime = "";
	string StartMode = CommandToExecute->Parameters.GetValue("START_MODE");
	CleanString(StartMode);
	MakeUpper(StartMode);
	if(!StartMode.length())
		StartMode = "AUTO";
	time_t iStartDate = 0; 
	if(StartMode == "SCHEDULED")
	{
		// In auto mode, we must check the START_DATE and START_TIME
		StartDate = CommandToExecute->Parameters.GetValue("START_DATE");
		CleanString(StartDate);
		MakeUpper(StartDate);
		if(StartDate.length())
		{
			if(!ArrangeDate(StartDate))
			{
				CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_DATE_FORMAT, StartDate);
				return false;
			}
		}
		else
		{
			// If start date is not given, we use the actual day
			time_t tTime = time(NULL);
			struct tm *tCurrentTime = localtime(&tTime);
			char Buf[50];
			sprintf(Buf, "%02d.%02d.%04d", tCurrentTime->tm_mday, tCurrentTime->tm_mon + 1, tCurrentTime->tm_year + 1900);
			StartDate = Buf;
		}

		StartTime = CommandToExecute->Parameters.GetValue("START_TIME");
		CleanString(StartTime);
		MakeUpper(StartTime);
		if(StartTime.length())
		{
			if(!ArrangeHour(StartTime))
			{
				CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_DATE_FORMAT, StartTime);
				return false;
			}
		}
		else
		{
			// If start time is not given, we use the actual time
			time_t tTime = time(NULL);
			struct tm *tCurrentTime = localtime(&tTime);
			char Buf[50];
			sprintf(Buf, "%02d%02d%02d", tCurrentTime->tm_hour, tCurrentTime->tm_min, tCurrentTime->tm_sec);
			StartTime = Buf;
		}

		int iStartDay;
		int iStartMonth;
		int iStartYear;
		string Day = "";
		string Month = "";
		string Year = "";
		string::size_type index;
		index = StartDate.find('.');
		if(index != string::npos)
		{
			Day = StartDate.substr(0, index);
			StartDate = StartDate.substr(index+1);
		}
		index = StartDate.find('.');
		if(index != string::npos)
		{
			Month = StartDate.substr(0, index);
			Year = StartDate.substr(index+1);
		}

		iStartDay = atoi(Day.c_str());
		iStartMonth = atoi(Month.c_str());
		iStartMonth--;
		iStartYear = atoi(Year.c_str());
		iStartYear -= 1900;
		struct tm ftime;
		ftime.tm_isdst = 0;
		ftime.tm_sec = 0;
		ftime.tm_min = 0;
		ftime.tm_hour = 0;
		ftime.tm_mday = iStartDay;
		ftime.tm_mon = iStartMonth;
		ftime.tm_year = iStartYear;
		ftime.tm_isdst = -1;
		if(StartTime.length() == 6)
		{
			string Hour = StartTime.substr(0, 2);
			string Min = StartTime.substr(2, 2);
			string Sec = StartTime.substr(4, 2);
			ftime.tm_hour = atoi(Hour.c_str());
			ftime.tm_min = atoi(Min.c_str());
			ftime.tm_sec = atoi(Sec.c_str());
		}
		iStartDate = mktime(&ftime); 
		if(iStartDate == -1)
		{
			CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_DATE_FORMAT);
			return false;
		}
	}

	string TransactionId = Application->Transactions.CreateTransaction(TransactionName.c_str(), TransactionDesc.c_str(), TransactionProc.c_str());

	if(!TransactionId.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_CANNOT_CREATE, TransactionName);
		return false;
	}

	NvTransaction *Transaction = Application->Transactions.GetTransaction(TransactionId.c_str(), this);

	if(Transaction == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_CANNOT_GET, TransactionName);
		return false;
	}
	CurrentTransaction = Transaction;

	GetStringValue(CommandToExecute, TransactionId.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	string ValidationProc = CommandToExecute->Parameters.GetValue("PROC_VALI");
	CleanString(ValidationProc);
	Transaction->SetProcValidation(ValidationProc.c_str());
	
	string RollbackProcC = CommandToExecute->Parameters.GetValue("PROC_ROLLC");
	CleanString(RollbackProcC);
	Transaction->SetProcRollbackC(RollbackProcC.c_str());
	
	string RollbackProcF = CommandToExecute->Parameters.GetValue("PROC_ROLLF");
	CleanString(RollbackProcF);
	Transaction->SetProcRollbackF(RollbackProcF.c_str());

	string FailedProc = CommandToExecute->Parameters.GetValue("PROC_FAIL");
	CleanString(FailedProc);
	Transaction->SetProcFailed(FailedProc.c_str());

	string CompletedProc = CommandToExecute->Parameters.GetValue("PROC_COMP");
	CleanString(CompletedProc);
	Transaction->SetProcCompleted(CompletedProc.c_str());

	string ValidateOkProc = CommandToExecute->Parameters.GetValue("PROC_VOK");
	CleanString(ValidateOkProc);
	Transaction->SetProcValidatedOk(ValidateOkProc.c_str());

	string ValidateKoProc = CommandToExecute->Parameters.GetValue("PROC_VKO");
	CleanString(ValidateKoProc);
	Transaction->SetProcValidatedKo(ValidateKoProc.c_str());

	if(StartMode == "AUTO")
		Transaction->SetStartMode(NV_TRANSACTION_START_AUTO);
	if(StartMode == "MANUAL")
		Transaction->SetStartMode(NV_TRANSACTION_START_MANUAL);
	if(StartMode == "SCHEDULED")
	{
		Transaction->SetStartMode(NV_TRANSACTION_START_SCHEDULED);
		Transaction->SetStartTime(iStartDate);
	}

	string ValidationMode = CommandToExecute->Parameters.GetValue("VALIDATION_MODE");
	CleanString(ValidationMode);
	MakeUpper(ValidationMode);
	if(!ValidationMode.length())
		ValidationMode = "AUTO";
	if(ValidationMode == "AUTO")
		Transaction->SetValidateMode(NV_TRANSACTION_VALIDATION_AUTO);
	if(ValidationMode == "MANUAL")
		Transaction->SetValidateMode(NV_TRANSACTION_VALIDATION_MANUAL);

	string RemoveMode = CommandToExecute->Parameters.GetValue("REMOVE_MODE");
	CleanString(RemoveMode);
	MakeUpper(RemoveMode);
	if(!RemoveMode.length())
		RemoveMode = "AUTO";
	if(RemoveMode == "AUTO")
		Transaction->SetRemoveMode(NV_TRANSACTION_REMOVE_AUTO);
	if(RemoveMode == "MANUAL")
		Transaction->SetRemoveMode(NV_TRANSACTION_REMOVE_MANUAL);

	bool bSaveInitial = true;
	string sSaveInitial = CommandToExecute->Parameters.GetValue("SAVE_INITIAL");
	CleanString(sSaveInitial);
	MakeUpper(sSaveInitial);
	if(sSaveInitial == "NO")
		bSaveInitial = false;
	Transaction->SetSaveInitial(bSaveInitial);

	bool bSaveCurrent = true;
	string sSaveCurrent = CommandToExecute->Parameters.GetValue("SAVE_CURRENT");
	CleanString(sSaveCurrent);
	MakeUpper(sSaveCurrent);
	if(sSaveCurrent == "NO")
		bSaveCurrent = false;
	Transaction->SetSaveCurrent(bSaveCurrent);

	bool bAutoRollBack = false;
	string sAutoRollBack = CommandToExecute->Parameters.GetValue("AUTO_ROLLBACK");
	CleanString(sAutoRollBack);
	MakeUpper(sAutoRollBack);
	if(sAutoRollBack == "YES")
		bAutoRollBack = true;


	if(!Transaction->Start(CommandToExecute, bAutoRollBack))
	{
		CurrentTransaction = NULL;
		Application->Transactions.FreeTransaction(TransactionId.c_str());
		return false;
	}
	
	// We remove the transaction if allowed
	bool ToRemove = false;
	if(Transaction->GetRemoveMode() == NV_TRANSACTION_REMOVE_AUTO)
		if(Transaction->GetStatus() == NV_TRANSACTION_STATUS_VALIDATED_OK)
			ToRemove = true;

	CurrentTransaction = NULL;
	Application->Transactions.FreeTransaction(TransactionId.c_str());

	if(ToRemove)
	{
		if(!Application->Transactions.RemoveTransaction(TransactionId.c_str()))
		{
			CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_CANNOT_REMOVE, TransactionId);
			return false;
		}
	}

	return true;
}


//SYSTEM TRANSACTION RESTART
bool NvSession::TransactionReStart(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string TransactionId = CommandToExecute->Parameters.GetValue("TID");
	CleanString(TransactionId);
	MakeUpper(TransactionId);

	if(!TransactionId.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "TRANSACTION:RESTART:TID");
		return false;
	}

	NvTransaction *Transaction = Application->Transactions.GetTransaction(TransactionId.c_str());

	if(Transaction == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_CANNOT_GET, TransactionId);
		return false;
	}
	CurrentTransaction = Transaction;


	bool bAutoRollBack = false;
	string sAutoRollBack = CommandToExecute->Parameters.GetValue("AUTO_ROLLBACK");
	CleanString(sAutoRollBack);
	MakeUpper(sAutoRollBack);
	if(sAutoRollBack == "YES")
		bAutoRollBack = true;


	if(!Transaction->Restart(CommandToExecute, bAutoRollBack))
	{
		CurrentTransaction = NULL;
		Application->Transactions.FreeTransaction(TransactionId.c_str());
		return false;
	}
	

	bool ToRemove = false;
	if(Transaction->GetRemoveMode() == NV_TRANSACTION_REMOVE_AUTO)
		if(Transaction->GetStatus() == NV_TRANSACTION_STATUS_VALIDATED_OK)
			ToRemove = true;

	CurrentTransaction = NULL;
	Application->Transactions.FreeTransaction(TransactionId.c_str());

	string sNoRemove = CommandToExecute->Parameters.GetValue("NO_REMOVE");
	CleanString(sNoRemove);
	MakeUpper(sNoRemove);
	if(sNoRemove == "YES")
		ToRemove = false;

	// We remove the transaction if allowed
	if(ToRemove)
	{
		if(!Application->Transactions.RemoveTransaction(TransactionId.c_str()))
		{
			CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_CANNOT_REMOVE, TransactionId);
			return false;
		}
	}

	return true;
}


//SYSTEM TRANSACTION ROLLBACK
bool NvSession::TransactionRollback(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string TransactionId = CommandToExecute->Parameters.GetValue("TID");
	CleanString(TransactionId);
	MakeUpper(TransactionId);

	if(!TransactionId.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "TRANSACTION:RESTART:TID");
		return false;
	}

	NvTransaction *Transaction = Application->Transactions.GetTransaction(TransactionId.c_str());

	if(Transaction == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_CANNOT_GET, TransactionId);
		return false;
	}
	CurrentTransaction = Transaction;

	if(!Transaction->Rollback(CommandToExecute))
	{
		CurrentTransaction = NULL;
		Application->Transactions.FreeTransaction(TransactionId.c_str());
		return false;
	}
	

	CurrentTransaction = NULL;
	Application->Transactions.FreeTransaction(TransactionId.c_str());

	return true;
}


//SYSTEM TRANSACTION VALIDATE
bool NvSession::TransactionValidate(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string TransactionId = CommandToExecute->Parameters.GetValue("TID");
	CleanString(TransactionId);
	MakeUpper(TransactionId);

	if(!TransactionId.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "TRANSACTION:VALIDATE:TID");
		return false;
	}

	NvTransaction *Transaction = Application->Transactions.GetTransaction(TransactionId.c_str());

	if(Transaction == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_CANNOT_GET, TransactionId);
		return false;
	}
	CurrentTransaction = Transaction;

	if(!Transaction->Validate(CommandToExecute))
	{
		CurrentTransaction = NULL;
		Application->Transactions.FreeTransaction(TransactionId.c_str());
		return false;
	}
	
	bool ToRemove = false;
	if(Transaction->GetRemoveMode() == NV_TRANSACTION_REMOVE_AUTO)
		if(Transaction->GetStatus() == NV_TRANSACTION_STATUS_VALIDATED_OK)
			ToRemove = true;

	CurrentTransaction = NULL;
	Application->Transactions.FreeTransaction(TransactionId.c_str());

	string sNoRemove = CommandToExecute->Parameters.GetValue("NO_REMOVE");
	CleanString(sNoRemove);
	MakeUpper(sNoRemove);
	if(sNoRemove == "YES")
		ToRemove = false;

	// We remove the transaction if allowed
	if(ToRemove)
	{
		if(!Application->Transactions.RemoveTransaction(TransactionId.c_str()))
		{
			CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_CANNOT_REMOVE, TransactionId);
			return false;
		}
	}

	return true;
}


//SYSTEM TRANSACTION REMOVE
bool NvSession::TransactionRemove(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string TransactionId = CommandToExecute->Parameters.GetValue("TID");
	CleanString(TransactionId);
	MakeUpper(TransactionId);

	if(!TransactionId.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "TRANSACTION:REMOVE:TID");
		return false;
	}

	if((CurrentTransaction != NULL) && (CurrentTransaction->GetIdent() == TransactionId))
	{
		CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_CANNOT_REMOVE, TransactionId);
		return false;
	}

	int iWaitTime = 10;
	string sWaitTime = CommandToExecute->Parameters.GetValue("WAIT");
	CleanString(sWaitTime);
	if(sWaitTime.length())
		iWaitTime = atoi(sWaitTime.c_str());
	if(iWaitTime < 0)
		iWaitTime = 0;

	if(!Application->Transactions.RemoveTransaction(TransactionId.c_str(), iWaitTime))
	{
		CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_CANNOT_REMOVE, TransactionId);
		return false;
	}

	return true;
}


//SYSTEM TRANSACTION INFO
bool NvSession::TransactionInfo(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string TransactionName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(TransactionName);
	MakeUpper(TransactionName);

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("INFO");
	if(!OutContainer->CreateObject("INFO", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "INFO");
		return false;
	}

	NvObjTable *TransactionInfo = (NvObjTable *)(OutContainer->GetObject("INFO"));


	string ToDate = CommandToExecute->Parameters.GetValue("TO_DATE");
	CleanString(ToDate);
	if(ToDate.length())
	{
		if(!ArrangeDate(ToDate))
		{
			CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_DATE_FORMAT, ToDate);
			return false;
		}
	}


	string FromDate = CommandToExecute->Parameters.GetValue("DATE");
	CleanString(FromDate);
	if(FromDate.length())
	{
		if(!ArrangeDate(FromDate))
		{
			CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_DATE_FORMAT, FromDate);
			return false;
		}
	}
	else
		ToDate = "";


	if(!Application->Transactions.GetInfo(TransactionName.c_str(), FromDate.c_str(), ToDate.c_str(), TransactionInfo, CommandToExecute->m_sLanguage.c_str()))
	{
		OutContainer->RemoveObject("INFO");
		CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_CANNOT_GET_INFO, TransactionName);
		return false;
	}

	return true;
}


//SYSTEM TRANSACTION INFOEX
bool NvSession::TransactionInfoEx(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string TransactionId = CommandToExecute->Parameters.GetValue("TID");
	CleanString(TransactionId);
	MakeUpper(TransactionId);

	if(!TransactionId.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "TRANSACTION:INFOEX:TID");
		return false;
	}

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("TRANSACTION_INFO");
	if(!OutContainer->CreateObject("TRANSACTION_INFO", NV_OBJTYPE_INDSTRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "TRANSACTION_INFO");
		return false;
	}

	NvObjIndStringList *TransactionInfo = (NvObjIndStringList *)(OutContainer->GetObject("TRANSACTION_INFO"));

	if(!Application->Transactions.GetInfoEx(TransactionId.c_str(), TransactionInfo, CommandToExecute->m_sLanguage.c_str()))
	{
		OutContainer->RemoveObject("TRANSACTION_INFO");
		CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_CANNOT_GET_INFO, TransactionId);
		return false;
	}

	return true;
}


//SYSTEM TRANSACTION SET_STATUS
bool NvSession::TransactionSetStatus(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string TransactionId = CommandToExecute->Parameters.GetValue("TID");
	CleanString(TransactionId);
	MakeUpper(TransactionId);

	if(!TransactionId.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "TRANSACTION:SET_STATUS:TID");
		return false;
	}

	NvTransaction *Transaction = Application->Transactions.GetTransaction(TransactionId.c_str());

	if(Transaction == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_CANNOT_GET, TransactionId);
		return false;
	}

	string NewStatus = CommandToExecute->Parameters.GetValue("STATUS");
	CleanString(NewStatus);
	MakeUpper(NewStatus);
	int iNewStatus = NV_TRANSACTION_STATUS_NOT_STARTED;
	if(NewStatus == "NOT_STARTED")
		iNewStatus = NV_TRANSACTION_STATUS_NOT_STARTED;
	if(NewStatus == "FAILED")
		iNewStatus = NV_TRANSACTION_STATUS_FAILED;
	if(NewStatus == "COMPLETED")
		iNewStatus = NV_TRANSACTION_STATUS_COMPLETED;
	if(NewStatus == "VALIDATED")
		iNewStatus = NV_TRANSACTION_STATUS_VALIDATED_OK;
	if(NewStatus == "NOT_VALIDATED")
		iNewStatus = NV_TRANSACTION_STATUS_VALIDATED_KO;

	if(CommandToExecute->Parameters.KeyExist("STATUS"))
		if(NewStatus != "RUNNING")
			Transaction->SetStatus(CommandToExecute, iNewStatus);

	CurrentTransaction = NULL;
	Application->Transactions.FreeTransaction(TransactionId.c_str());

	return true;
}


//SYSTEM TRANSACTION SET_OPTIONS
bool NvSession::TransactionSetOptions(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string TransactionId = CommandToExecute->Parameters.GetValue("TID");
	CleanString(TransactionId);
	MakeUpper(TransactionId);

	if(!TransactionId.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "TRANSACTION:SET_OPTIONS:TID");
		return false;
	}

	// This command is authorized from inside a transaction
	NvTransaction *Transaction;
	bool IsCurrent = false;
	if((CurrentTransaction != NULL) && (CurrentTransaction->GetIdent() == TransactionId))
	{
		IsCurrent = true;
		Transaction = CurrentTransaction;
	}
	else
		Transaction = Application->Transactions.GetTransaction(TransactionId.c_str());

	if(Transaction == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_CANNOT_GET, TransactionId);
		return false;
	}
	if(Transaction->GetInQueue())
	{
		CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_IN_QUEUE, TransactionId);
		Application->Transactions.FreeTransaction(TransactionId.c_str());
		return false;
	}
	
	if(!IsCurrent)
		CurrentTransaction = Transaction;


	if(CommandToExecute->Parameters.KeyExist("NAME"))
	{
		string Name = CommandToExecute->Parameters.GetValue("NAME");
		CleanString(Name);
		Transaction->SetName(Name.c_str());
	}
	if(CommandToExecute->Parameters.KeyExist("DESCRIPTION"))
	{
		string Description = CommandToExecute->Parameters.GetValue("DESCRIPTION");
		CleanString(Description);
		Transaction->SetDesc(Description.c_str());
	}
	if(CommandToExecute->Parameters.KeyExist("PROC_VALI"))
	{
		string ValidationProc = CommandToExecute->Parameters.GetValue("PROC_VALI");
		CleanString(ValidationProc);
		Transaction->SetProcValidation(ValidationProc.c_str());
	}
	if(CommandToExecute->Parameters.KeyExist("PROC_ROLLC"))
	{
		string RollbackProcC = CommandToExecute->Parameters.GetValue("PROC_ROLLC");
		CleanString(RollbackProcC);
		Transaction->SetProcRollbackC(RollbackProcC.c_str());
	}
	if(CommandToExecute->Parameters.KeyExist("PROC_ROLLF"))
	{
		string RollbackProcF = CommandToExecute->Parameters.GetValue("PROC_ROLLF");
		CleanString(RollbackProcF);
		Transaction->SetProcRollbackF(RollbackProcF.c_str());
	}
	if(CommandToExecute->Parameters.KeyExist("PROC_FAIL"))
	{
		string FailedProc = CommandToExecute->Parameters.GetValue("PROC_FAIL");
		CleanString(FailedProc);
		Transaction->SetProcFailed(FailedProc.c_str());
	}
	if(CommandToExecute->Parameters.KeyExist("PROC_COMP"))
	{
		string CompletedProc = CommandToExecute->Parameters.GetValue("PROC_COMP");
		CleanString(CompletedProc);
		Transaction->SetProcCompleted(CompletedProc.c_str());
	}
	if(CommandToExecute->Parameters.KeyExist("PROC_VOK"))
	{
		string ValidateOkProc = CommandToExecute->Parameters.GetValue("PROC_VOK");
		CleanString(ValidateOkProc);
		Transaction->SetProcValidatedOk(ValidateOkProc.c_str());
	}
	if(CommandToExecute->Parameters.KeyExist("PROC_VKO"))
	{
		string ValidateKoProc = CommandToExecute->Parameters.GetValue("PROC_VKO");
		CleanString(ValidateKoProc);
		Transaction->SetProcValidatedKo(ValidateKoProc.c_str());
	}


	if(CommandToExecute->Parameters.KeyExist("VALIDATION_MODE"))
	{
		string ValidationMode = CommandToExecute->Parameters.GetValue("VALIDATION_MODE");
		CleanString(ValidationMode);
		MakeUpper(ValidationMode);
		if(!ValidationMode.length())
			ValidationMode = "AUTO";
		if(ValidationMode == "AUTO")
			Transaction->SetValidateMode(NV_TRANSACTION_VALIDATION_AUTO);
		if(ValidationMode == "MANUAL")
			Transaction->SetValidateMode(NV_TRANSACTION_VALIDATION_MANUAL);
	}

	if(CommandToExecute->Parameters.KeyExist("REMOVE_MODE"))
	{
		string RemoveMode = CommandToExecute->Parameters.GetValue("REMOVE_MODE");
		CleanString(RemoveMode);
		MakeUpper(RemoveMode);
		if(!RemoveMode.length())
			RemoveMode = "AUTO";
		if(RemoveMode == "AUTO")
			Transaction->SetRemoveMode(NV_TRANSACTION_REMOVE_AUTO);
		if(RemoveMode == "MANUAL")
			Transaction->SetRemoveMode(NV_TRANSACTION_REMOVE_MANUAL);
	}

	Transaction->Save();

	if(!IsCurrent)
	{
		CurrentTransaction = NULL;
		Application->Transactions.FreeTransaction(TransactionId.c_str());
	}

	return true;
}





//TRANSACTION GET_CONTAINER
bool NvSession::TransactionGetContainer(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get variable name that should be in command parameters
	string TransactionId = CommandToExecute->Parameters.GetValue("TID");
	CleanString(TransactionId);
	MakeUpper(TransactionId);

	if(!TransactionId.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "TRANSACTION:SET_STATUS:TID");
		return false;
	}

	NvTransaction *Transaction = Application->Transactions.GetTransaction(TransactionId.c_str());

	if(Transaction == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_CANNOT_GET, TransactionId);
		return false;
	}

	// Get container name that should be in command parameters
	string ContainerName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ContainerName);

	// Get context
	string Context = CommandToExecute->Parameters.GetValue("CONTEXT");
	CleanString(Context);
	MakeUpper(Context);

	// Get append option
	int AppendMode = NV_CONTAINER_COPY_NO_APPEND;
	string sAppend = CommandToExecute->Parameters.GetValue("APPEND");
	CleanString(sAppend);
	MakeUpper(sAppend);
	if(sAppend == "YES")
	{
		AppendMode = NV_CONTAINER_COPY_APPEND_NO_REPLACE;
		string sReplace = CommandToExecute->Parameters.GetValue("REPLACE");
		CleanString(sReplace);
		MakeUpper(sReplace);
		if(sReplace == "YES")
			AppendMode = NV_CONTAINER_COPY_APPEND;
	}

	// Get subcontainer option
	bool bWithSubContainers = false;
	string sWithSubContainers = CommandToExecute->Parameters.GetValue("SUBCONTAINERS");
	CleanString(sWithSubContainers);
	MakeUpper(sWithSubContainers);
	if(sWithSubContainers == "YES")
		bWithSubContainers = true;

	// Get object names that may be in parameters
	string ObjNames = CommandToExecute->Parameters.GetValue("OBJECTS");
	CleanString(ObjNames);

	// Check the persist parameter
	string PersistTimeFileInCommand = CommandToExecute->Parameters.GetValue("PERSIST");
	CleanString(PersistTimeFileInCommand);
	MakeUpper(PersistTimeFileInCommand);
	int iPersistTimeFileInCommand = 0;	// The file is not persistant by default on the server
	if(PersistTimeFileInCommand.length())
		iPersistTimeFileInCommand = atoi(PersistTimeFileInCommand.c_str());
	if(iPersistTimeFileInCommand < 0)
		iPersistTimeFileInCommand = -1;

	// Get with file option
	bool bWithFiles = true;
	string sWithFiles = CommandToExecute->Parameters.GetValue("FILES");
	CleanString(sWithFiles);
	MakeUpper(sWithFiles);
	if(sWithFiles == "NO")
		bWithFiles = false;

	string FileDir = "";
	if(bWithFiles)
	{
		if(iPersistTimeFileInCommand == -1)
			FileDir = Application->GetFileDir();
		else
			FileDir = Application->GetWorkDir();
		
		
		if((CommandToExecute->m_iCommandSource == NV_SOURCE_PROCEDURE) || (CommandToExecute->m_iCommandSource == NV_SOURCE_SERVICE))
		{
			// from internal, we can change the file directory of new file objects
			string FileDirInCommand = CommandToExecute->Parameters.GetValue("FILEDIR");
			CleanString(FileDirInCommand);
			SetDirName(FileDirInCommand);
			// eliminate last slash
			if(FileDirInCommand.size() && FileDirInCommand[FileDirInCommand.size()-1] == '/')
				FileDirInCommand.erase(FileDirInCommand.size()-1, 1);
			// eliminate last backslash
			if(FileDirInCommand.size() && FileDirInCommand[FileDirInCommand.size()-1] == '\\')
				FileDirInCommand.erase(FileDirInCommand.size()-1, 1);
			if(FileDirInCommand.length())
				FileDir = FileDirInCommand;
			if(iPersistTimeFileInCommand == -1)
			{
				// We get application work dir
				string sWorkDir = "";
				sWorkDir = Application->GetWorkDir();
				if(CompareWorkDir(FileDir, sWorkDir))
				{
					CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_SET_PERSISTANT, FileDir);
					Application->Transactions.FreeTransaction(TransactionId.c_str());
					return false;
				}
			}
		}
	}


	// Get with data option option
	bool bWithData = true;
	string sWithData = CommandToExecute->Parameters.GetValue("WITH_DATA");
	CleanString(sWithData);
	MakeUpper(sWithData);
	if(sWithData == "NO")
		bWithData = false;

	const char *ObjsForCommand = NULL;
	if(CommandToExecute->Parameters.KeyExist("OBJECTS"))
		ObjsForCommand = ObjNames.c_str();

	if(!Transaction->GetContainer(CommandToExecute, Context.c_str(), ContainerName.c_str(), OutContainer, AppendMode, ObjsForCommand, bWithSubContainers, bWithData, FileDir.c_str(), iPersistTimeFileInCommand))
	{
		// The error is directly managed into the function
		Application->Transactions.FreeTransaction(TransactionId.c_str());
		return false;
	}

	Application->Transactions.FreeTransaction(TransactionId.c_str());
	return true;
}






//TRANSACTION GET_VARIABLES
bool NvSession::TransactionGetVariables(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get variable name that should be in command parameters
	string TransactionId = CommandToExecute->Parameters.GetValue("TID");
	CleanString(TransactionId);
	MakeUpper(TransactionId);

	if(!TransactionId.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "TRANSACTION:SET_STATUS:TID");
		return false;
	}

	NvTransaction *Transaction = Application->Transactions.GetTransaction(TransactionId.c_str());

	if(Transaction == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "TRANSACTION", NV_TRANSACTION_CANNOT_GET, TransactionId);
		return false;
	}


	// Get context
	string Context = CommandToExecute->Parameters.GetValue("CONTEXT");
	CleanString(Context);
	MakeUpper(Context);

	OutContainer->RemoveObject("VARIABLES");
	if(!OutContainer->CreateObject("VARIABLES", NV_OBJTYPE_INDSTRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "VARIABLES");
		return false;
	}

	NvObjIndStringList *TransactionVariables = (NvObjIndStringList *)(OutContainer->GetObject("VARIABLES"));


	if(!Transaction->GetVariables(CommandToExecute, Context.c_str(), TransactionVariables))
	{
		// The error is directly managed into the function
		Application->Transactions.FreeTransaction(TransactionId.c_str());
		return false;
	}

	Application->Transactions.FreeTransaction(TransactionId.c_str());
	return true;
}



//SYSTEM SCHEDULER CREATE_TASK
bool NvSession::SchedulerCreateTask(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	if(!CheckPermission(CommandToExecute, "SYSTEM", "SCHEDULER_ADMIN"))
		return false;

	// Get variable name that should be in command parameters
	string TaskName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(TaskName);

	if(!TaskName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SCHEDULER:CREATE_TASK:NAME");
		return false;
	}

	// The real name is not cleaned
	TaskName = CommandToExecute->Parameters.GetValue("NAME");

	string Description = CommandToExecute->Parameters.GetValue("DESCRIPTION");

	if(Application->Tasks.TaskExist(TaskName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SCHEDULER", NV_SCHEDULER_TASK_EVER_EXIST, TaskName);
		return false;
	}

	if(!Application->Tasks.CreateTask(TaskName.c_str(), Description.c_str(), m_sUser.c_str(), CommandToExecute->m_sLanguage.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SCHEDULER", NV_SCHEDULER_CANNOT_CREATE_TASK, TaskName);
		return false;
	}
	return true;
}


//SYSTEM SCHEDULER REMOVE_TASK
bool NvSession::SchedulerRemoveTask(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	if(!CheckPermission(CommandToExecute, "SYSTEM", "SCHEDULER_ADMIN"))
		return false;

	// Get variable name that should be in command parameters
	string TaskName = CommandToExecute->Parameters.GetValue("NAME");

	int iWaitTime = 10;
	string sWaitTime = CommandToExecute->Parameters.GetValue("WAIT");
	CleanString(sWaitTime);
	if(sWaitTime.length())
		iWaitTime = atoi(sWaitTime.c_str());
	if(iWaitTime < 0)
		iWaitTime = 0;

	if(!Application->Tasks.RemoveTask(TaskName.c_str(), iWaitTime))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SCHEDULER", NV_SCHEDULER_CANNOT_REMOVE_TASK, TaskName);
		return false;
	}

	return true;
}


//SYSTEM SCHEDULER ENABLE_TASK
bool NvSession::SchedulerEnableTask(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	if(!CheckPermission(CommandToExecute, "SYSTEM", "SCHEDULER_ADMIN"))
		return false;

	// Get variable name that should be in command parameters
	string TaskName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(TaskName);

	if(!TaskName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SCHEDULER:ENABLE_TASK:NAME");
		return false;
	}

	// The real name is not cleaned
	TaskName = CommandToExecute->Parameters.GetValue("NAME");

	if(!Application->Tasks.TaskExist(TaskName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SCHEDULER", NV_SCHEDULER_INVALID_TASK, TaskName);
		return false;
	}

	string sTaskEnable = CommandToExecute->Parameters.GetValue("ENABLE");
	CleanString(sTaskEnable);
	MakeUpper(sTaskEnable);
	bool bEnableIt = true;
	if(sTaskEnable == "NO")
		bEnableIt = false;

	Application->Tasks.EnableTask(TaskName.c_str(), bEnableIt);

	return true;
}


//SYSTEM SCHEDULER RUN_TASK
bool NvSession::SchedulerRunTask(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	if(!CheckPermission(CommandToExecute, "SYSTEM", "SCHEDULER_ADMIN"))
		if(!CheckPermission(CommandToExecute, "SYSTEM", "SCHEDULER_RUN_MANUAL"))
			return false;

	// Get variable name that should be in command parameters
	string TaskName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(TaskName);

	if(!TaskName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SCHEDULER:ENABLE_TASK:NAME");
		return false;
	}

	// The real name is not cleaned
	TaskName = CommandToExecute->Parameters.GetValue("NAME");

	if(!Application->Tasks.TaskExist(TaskName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SCHEDULER", NV_SCHEDULER_INVALID_TASK, TaskName);
		return false;
	}


	Application->Tasks.RunTask(TaskName.c_str());

	return true;
}


//SYSTEM SCHEDULER SET_TASK_PARAM
bool NvSession::SchedulerSetTaskParam(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	if(!CheckPermission(CommandToExecute, "SYSTEM", "SCHEDULER_ADMIN"))
		return false;

	// We must check the user parameter here
	if(CommandToExecute->Parameters.KeyExist("USER"))
	{
		string TaskUser = CommandToExecute->Parameters.GetValue("USER");
		CleanString(TaskUser);

		// If no user given, we use actual user
		if(!TaskUser.length())	
			TaskUser = m_sUser;
		
		// If the user is not the actual one, we must verify that we have the permission
		// to run the task with another user name
		string UserForComparaison = m_sUser;
		MakeUpper(UserForComparaison);
		string TaskUserForComparaison = TaskUser;
		MakeUpper(TaskUserForComparaison);
		if(TaskUserForComparaison != UserForComparaison)
		{
			if(!CheckPermission(CommandToExecute, "SYSTEM", "SCHEDULER_RUN_OTHER_USER"))
				return false;
		}

		// We must set the real user name to the task
		CommandToExecute->Parameters.SetValue("USER", TaskUser);
	}
	
	return Application->Tasks.SetTaskParam(CommandToExecute);
}


//SYSTEM SCHEDULER GET_TASK_PARAM
bool NvSession::SchedulerGetTaskParam(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	if(!CheckPermission(CommandToExecute, "SYSTEM", "SCHEDULER_ADMIN"))
		return false;

	// Get variable name that should be in command parameters
	string TaskName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(TaskName);

	if(!TaskName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SCHEDULER:CREATE_TASK:NAME");
		return false;
	}

	// The real name is not cleaned
	TaskName = CommandToExecute->Parameters.GetValue("NAME");

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("TASK_PARAM");
	if(!OutContainer->CreateObject("TASK_PARAM", NV_OBJTYPE_INDSTRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "TASK_PARAM");
		return false;
	}

	NvObjIndStringList *TaskInfo = (NvObjIndStringList *)(OutContainer->GetObject("TASK_PARAM"));

	if(!Application->Tasks.GetInfoEx(TaskName.c_str(), TaskInfo, CommandToExecute->m_sLanguage.c_str()))
	{
		OutContainer->RemoveObject("TASK_PARAM");
		CommandToExecute->Error.SetError("SYSTEM", "SCHEDULER", NV_SCHEDULER_CANNOT_GET_TASK_PARAM, TaskName);
		return false;
	}

	return true;
}


//SYSTEM SCHEDULER TASK_LIST
bool NvSession::SchedulerTaskList(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	if(!CheckPermission(CommandToExecute, "SYSTEM", "SCHEDULER_ADMIN"))
		return false;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("TASK_LIST");
	if(!OutContainer->CreateObject("TASK_LIST", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "TASK_LIST");
		return false;
	}

	NvObjTable *TaskInfo = (NvObjTable *)(OutContainer->GetObject("TASK_LIST"));


	if(!Application->Tasks.GetInfo(TaskInfo, CommandToExecute->m_sLanguage.c_str()))
	{
		OutContainer->RemoveObject("TASK_LIST");
		CommandToExecute->Error.SetError("SYSTEM", "SCHEDULER", NV_SCHEDULER_CANNOT_GET_TASK_LIST);
		return false;
	}

	return true;
}



//SYSTEM APPLICATION GET_NAME
bool NvSession::ApplicationGetName(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	GetStringValue(CommandToExecute, GetAppName().c_str(), Buffer, BufferSize, DataSize, iEncoding);
	return true;
}

//SYSTEM APPLICATION GET_WORK_DIR
bool NvSession::ApplicationGetWorkDir(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	GetStringValue(CommandToExecute, Application->GetWorkDir().c_str(), Buffer, BufferSize, DataSize, iEncoding);
	return true;
}

//SYSTEM APPLICATION GET_FILE_DIR
bool NvSession::ApplicationGetFileDir(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	GetStringValue(CommandToExecute, Application->GetFileDir().c_str(), Buffer, BufferSize, DataSize, iEncoding);
	return true;
}

//SYSTEM APPLICATION GET_WROOT_DIR
bool NvSession::ApplicationGetWrootDir(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	GetStringValue(CommandToExecute, Application->GetWrootDir().c_str(), Buffer, BufferSize, DataSize, iEncoding);
	return true;
}

//SYSTEM APPLICATION GET_DIR
bool NvSession::ApplicationGetDir(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string AppDirName = GetMain()->GetNirvaDir();
	AddDirSuffix(AppDirName);
	AppDirName += "Applications";
	AddDirSuffix(AppDirName);
	string AppNameHere = Application->GetAppNameForFiles();
	MakeUpper(AppNameHere);
	AppDirName += AppNameHere;
	AddDirSuffix(AppDirName);
	
	GetStringValue(CommandToExecute, AppDirName.c_str(), Buffer, BufferSize, DataSize, iEncoding);
	return true;
}


//SYSTEM APPLICATION REMOVE_DIR
bool NvSession::ApplicationRemoveDir(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// This command is authorized only from procedure or service
	if((CommandToExecute->m_iCommandSource != NV_SOURCE_SERVICE) && (CommandToExecute->m_iCommandSource != NV_SOURCE_PROCEDURE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:APPLICATION:REMOVE_DIR");
		return false;
	}

	
	bool bRelative = true;
	string sRelative= CommandToExecute->Parameters.GetValue("RELATIVE");
	CleanString(sRelative);
	MakeUpper(sRelative);
	if(sRelative == "NO")
		bRelative = false;

	string DirName = CommandToExecute->Parameters.GetValue("DIR");
	SetDirName(DirName);
	RemoveDirSuffix(DirName);
	CleanString(DirName);
	if(bRelative)
	{
		while((DirName.length()) && (DirName[0] == '\\'))
			DirName.erase(0, 1);
		while((DirName.length()) && (DirName[0] == '/'))
			DirName.erase(0, 1);
	}

	if(!DirName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "APPLICATION:REMOVE_DIR:DIR");
		return false;
	}

	string RealDirName = DirName;
	RemoveDirSuffix(RealDirName);

	if(bRelative)
	{
		RealDirName = GetMain()->GetNirvaDir();
		AddDirSuffix(RealDirName);
		RealDirName += "Applications";
		AddDirSuffix(RealDirName);
		string AppNameHere = Application->GetAppNameForFiles();
		MakeUpper(AppNameHere);
		RealDirName += AppNameHere;
		AddDirSuffix(RealDirName);
		RealDirName += DirName;
	}
	
	bool bSelf = true;
	string sSelf = CommandToExecute->Parameters.GetValue("SELF");
	CleanString(sSelf);
	MakeUpper(sSelf);
	if(sSelf == "NO")
		bSelf = false;
	
	string SubDirRoot = "*";
	if(CommandToExecute->Parameters.KeyExist("SUBDIRS"))
	{
		SubDirRoot = CommandToExecute->Parameters.GetValue("SUBDIRS");
		CleanString(SubDirRoot);
	}

	
	string FileRoot = "*";
	if(CommandToExecute->Parameters.KeyExist("FILES"))
	{
		FileRoot = CommandToExecute->Parameters.GetValue("FILES");
		CleanString(FileRoot);
	}

	if(!RemoveDir(RealDirName.c_str(), bSelf, FileRoot.c_str(), SubDirRoot.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_CANNOT_REMOVE_DIR, DirName.c_str());
		return false;
	}

	return true;
}

//SYSTEM APPLICATION CREATE_DIR
bool NvSession::ApplicationCreateDir(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// This command is authorized only from procedure or service
	if((CommandToExecute->m_iCommandSource != NV_SOURCE_SERVICE) && (CommandToExecute->m_iCommandSource != NV_SOURCE_PROCEDURE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:APPLICATION:CREATE_DIR");
		return false;
	}

	
	bool bRelative = true;
	string sRelative= CommandToExecute->Parameters.GetValue("RELATIVE");
	CleanString(sRelative);
	MakeUpper(sRelative);
	if(sRelative == "NO")
		bRelative = false;

	string DirName = CommandToExecute->Parameters.GetValue("DIR");
	SetDirName(DirName);
	RemoveDirSuffix(DirName);
	CleanString(DirName);
	if(bRelative)
	{
		while((DirName.length()) && (DirName[0] == '\\'))
			DirName.erase(0, 1);
		while((DirName.length()) && (DirName[0] == '/'))
			DirName.erase(0, 1);
	}
		

	if(!DirName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "APPLICATION:CREATE_DIR:DIR");
		return false;
	}

	string RealDirName = DirName;
	RemoveDirSuffix(RealDirName);

	if(bRelative)
	{
		RealDirName = GetMain()->GetNirvaDir();
		AddDirSuffix(RealDirName);
		RealDirName += "Applications";
		AddDirSuffix(RealDirName);
		string AppNameHere = Application->GetAppNameForFiles();
		MakeUpper(AppNameHere);
		RealDirName += AppNameHere;
		AddDirSuffix(RealDirName);
		RealDirName += DirName;
	}
	
	if(!CreateDir(RealDirName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_CANNOT_CREATE_DIR, DirName.c_str());
		return false;
	}
	
	return true;
}



//SYSTEM APPLICATION DIR
bool NvSession::ApplicationDir(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// This command is authorized only from procedure or service
	if((CommandToExecute->m_iCommandSource != NV_SOURCE_SERVICE) && (CommandToExecute->m_iCommandSource != NV_SOURCE_PROCEDURE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:APPLICATION:REMOVE_DIR");
		return false;
	}

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	
	bool bRelative = true;
	string sRelative= CommandToExecute->Parameters.GetValue("RELATIVE");
	CleanString(sRelative);
	MakeUpper(sRelative);
	if(sRelative == "NO")
		bRelative = false;

	string DirName = CommandToExecute->Parameters.GetValue("DIR");
	SetDirName(DirName);
	RemoveDirSuffix(DirName);
	CleanString(DirName);
	if(bRelative)
	{
		while((DirName.length()) && (DirName[0] == '\\'))
			DirName.erase(0, 1);
		while((DirName.length()) && (DirName[0] == '/'))
			DirName.erase(0, 1);
	}

	if(!DirName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "APPLICATION:REMOVE_DIR:DIR");
		return false;
	}


	string RealDirName = DirName;
	RemoveDirSuffix(RealDirName);

	if(bRelative)
	{
		RealDirName = GetMain()->GetNirvaDir();
		AddDirSuffix(RealDirName);
		RealDirName += "Applications";
		AddDirSuffix(RealDirName);
		string AppNameHere = Application->GetAppNameForFiles();
		MakeUpper(AppNameHere);
		RealDirName += AppNameHere;
		AddDirSuffix(RealDirName);
		RealDirName += DirName;
	}

	string SubDirRoot = "*";
	if(CommandToExecute->Parameters.KeyExist("SUBDIRS"))
	{
		SubDirRoot = CommandToExecute->Parameters.GetValue("SUBDIRS");
		CleanString(SubDirRoot);
	}

	
	string FileRoot = "*";
	if(CommandToExecute->Parameters.KeyExist("FILES"))
	{
		FileRoot = CommandToExecute->Parameters.GetValue("FILES");
		CleanString(FileRoot);
	}

	// Get recurse option
	string RecurseInCommand = CommandToExecute->Parameters.GetValue("RECURSE");
	CleanString(RecurseInCommand);
	MakeUpper(RecurseInCommand);
	bool bRecurseInCommand = true;
	if(RecurseInCommand == "NO")
		bRecurseInCommand = false;

	string PersistTimeFileInCommand = CommandToExecute->Parameters.GetValue("PERSIST");
	CleanString(PersistTimeFileInCommand);
	MakeUpper(PersistTimeFileInCommand);
	int iPersistTimeFileInCommand = -1;	// The file is persistant by default for the DIR command
	if(PersistTimeFileInCommand.length())
		iPersistTimeFileInCommand = atoi(PersistTimeFileInCommand.c_str());
	if(iPersistTimeFileInCommand < 0)
		iPersistTimeFileInCommand = -1;


	// Get naming option
	string sNaming = CommandToExecute->Parameters.GetValue("NAMING");
	CleanString(sNaming);
	MakeUpper(sNaming);
	bool bNamingStandard = false;
	if(sNaming == "STANDARD")
		bNamingStandard = true;

	if(!OutContainer->Dir(RealDirName.c_str(), FileRoot.c_str(), SubDirRoot.c_str(), bRecurseInCommand, iPersistTimeFileInCommand, bNamingStandard))
	{
		CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_CANNOT_GET_DIR, DirName.c_str());
		return false;
	}

	return true;

}




//SYSTEM APPLICATION REMOVE
bool NvSession::ApplicationRemove(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_REMOVE"))
		return false;

	// Get variable name that should be in command parameters
	string ApplicationName = CommandToExecute->Parameters.GetValue("APPLICATION");
	CleanString(ApplicationName);
	MakeUpper(ApplicationName);

	if(!ApplicationName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "APPLICATION:REMOVE:APPLICATION");
		return false;
	}


	if(!GetMain()->Applications.RemoveApplication(ApplicationName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_CANNOT_REMOVE, ApplicationName);
		return false;
	}


	return true;
}


//SYSTEM APPLICATION START
bool NvSession::ApplicationStart(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_RUN"))
		return false;

	// Get variable name that should be in command parameters
	string ApplicationName = CommandToExecute->Parameters.GetValue("APPLICATION");
	CleanString(ApplicationName);
	MakeUpper(ApplicationName);

	if(!ApplicationName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "APPLICATION:START:APPLICATION");
		return false;
	}



	if(!GetMain()->Applications.StartApplication(ApplicationName.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_CANNOT_START, ApplicationName);
		return false;
	}


	return true;
}


//SYSTEM APPLICATION STOP
bool NvSession::ApplicationStop(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_RUN"))
		return false;

	// Get variable name that should be in command parameters
	string ApplicationName = CommandToExecute->Parameters.GetValue("APPLICATION");
	CleanString(ApplicationName);
	MakeUpper(ApplicationName);

	if(!ApplicationName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "APPLICATION:STOP:APPLICATION");
		return false;
	}


	if(!GetMain()->Applications.StopApplication(ApplicationName.c_str(), m_sIdent.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_CANNOT_STOP, ApplicationName);
		return false;
	}


	return true;
}


//SYSTEM APPLICATION SESSIONS
bool NvSession::ApplicationSessions(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_SESSION_LIST"))
		return false;

	// Get variable name that should be in command parameters
	string ApplicationName = CommandToExecute->Parameters.GetValue("APPLICATION");
	CleanString(ApplicationName);
	MakeUpper(ApplicationName);

	if(!ApplicationName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "APPLICATION:SESSIONS:APPLICATION");
		return false;
	}

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("SESSIONS");
	if(!OutContainer->CreateObject("SESSIONS", NV_OBJTYPE_STRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "SESSIONS");
		return false;
	}

	NvObj *Object = OutContainer->GetObject("SESSIONS");
	NvObjStringList *ApplicationSessions = (NvObjStringList *)Object;

	if(!GetMain()->Applications.GetSessions(ApplicationName.c_str(), ApplicationSessions))
	{
		OutContainer->RemoveObject("SESSIONS");
		CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_CANNOT_GET_SESSION_LIST, ApplicationName);
		return false;
	}

	return true;
}


//SYSTEM APPLICATION INFO
bool NvSession::ApplicationInfo(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_LIST"))
		return false;

	// Get variable name that should be in command parameters
	string ApplicationName = CommandToExecute->Parameters.GetValue("APPLICATION");
	CleanString(ApplicationName);
	MakeUpper(ApplicationName);

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("INFO");
	if(!OutContainer->CreateObject("INFO", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "INFO");
		return false;
	}

	NvObjTable *ApplicationInfo = (NvObjTable *)(OutContainer->GetObject("INFO"));

	if(!GetMain()->Applications.GetInfo(ApplicationName.c_str(), ApplicationInfo))
	{
		OutContainer->RemoveObject("INFO");
		CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_CANNOT_GET_INFO, ApplicationName);
		return false;
	}

	return true;
}


//SYSTEM APPLICATION INFOEX
bool NvSession::ApplicationInfoEx(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string ApplicationName = CommandToExecute->Parameters.GetValue("APPLICATION");
	CleanString(ApplicationName);
	MakeUpper(ApplicationName);

	if(!ApplicationName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "APPLICATION:INFOEX:APPLICATION");
		return false;
	}

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	OutContainer->RemoveObject("APPLICATION_INFO");
	if(!OutContainer->CreateObject("APPLICATION_INFO", NV_OBJTYPE_INDSTRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "APPLICATION_INFO");
		return false;
	}

	NvObjIndStringList *ApplicationInfo = (NvObjIndStringList *)(OutContainer->GetObject("APPLICATION_INFO"));

	if(!GetMain()->Applications.GetInfoEx(ApplicationName.c_str(), ApplicationInfo))
	{
		OutContainer->RemoveObject("APPLICATION_INFO");
		CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_CANNOT_GET_INFO, ApplicationName);
		return false;
	}

	return true;
}


//SYSTEM APPLICATION CREATE
bool NvSession::ApplicationCreate(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_CREATE"))
		return false;

	// Get variable name that should be in command parameters
	string ApplicationName = CommandToExecute->Parameters.GetValue("APPLICATION");
	CleanString(ApplicationName, true);
	if(!CheckAsciiName(ApplicationName))
	{
		CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_BAD_NAME, ApplicationName);
		return false;
	}
	MakeUpper(ApplicationName);

	if(!ApplicationName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "APPLICATION:CREATE:APPLICATION");
		return false;
	}

	string ApplicationDesc = CommandToExecute->Parameters.GetValue("DESCRIPTION");
	string ApplicationTimeOut = CommandToExecute->Parameters.GetValue("TIMEOUT");
	int iApplicationTimeOut = 300;
	if(ApplicationTimeOut.length())
		iApplicationTimeOut = atoi(ApplicationTimeOut.c_str());
	string ApplicationSecuApp = CommandToExecute->Parameters.GetValue("SECUAPP");
	string ApplicationUseScheduled = CommandToExecute->Parameters.GetValue("USE_SCHEDULED_TRANSAC");
	CleanString(ApplicationUseScheduled);
	MakeUpper(ApplicationUseScheduled);
	bool bApplicationUseScheduled = true;
	if(ApplicationUseScheduled == "NO")
		bApplicationUseScheduled = false;
	string ApplicationAllowWebXml = CommandToExecute->Parameters.GetValue("ALLOW_WEB_XML");
	CleanString(ApplicationAllowWebXml);
	MakeUpper(ApplicationAllowWebXml);
	bool bApplicationAllowWebXml = true;
	if(ApplicationAllowWebXml == "NO")
		bApplicationAllowWebXml = false;
	string ApplicationAllowNotOwnedSession = CommandToExecute->Parameters.GetValue("ALLOW_NOT_OWNED_SESSIONS");
	CleanString(ApplicationAllowNotOwnedSession);
	MakeUpper(ApplicationAllowNotOwnedSession);
	bool bApplicationAllowNotOwnedSession = true;
	if(ApplicationAllowNotOwnedSession == "NO")
		bApplicationAllowNotOwnedSession = false;
	string ApplicationAuthMode = CommandToExecute->Parameters.GetValue("AUTH_MODE");
	int iApplicationAuthMode = NV_AUTHENTICATION_NIRVA;
	if(ApplicationAuthMode == "SSO")
		iApplicationAuthMode = NV_AUTHENTICATION_SSO;
	if(ApplicationAuthMode == "NIRVA_SSO")
		iApplicationAuthMode = NV_AUTHENTICATION_NIRVA_SSO;
	string ApplicationNirvaIsSsoUser = CommandToExecute->Parameters.GetValue("NIRVA_IS_SSO_USER");
	CleanString(ApplicationNirvaIsSsoUser);
	MakeUpper(ApplicationNirvaIsSsoUser);
	bool bApplicationNirvaIsSsoUser = false;
	if(ApplicationNirvaIsSsoUser == "YES")
		bApplicationNirvaIsSsoUser = true;
	if(!GetMain()->Applications.CreateApplication(ApplicationName.c_str(), ApplicationDesc.c_str(), iApplicationTimeOut, ApplicationSecuApp.c_str(), bApplicationUseScheduled, bApplicationAllowWebXml, false, bApplicationAllowNotOwnedSession, true, iApplicationAuthMode, bApplicationNirvaIsSsoUser))
	{
		CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_CANNOT_CREATE, ApplicationName);
		return false;
	}

	return true;
}


//SYSTEM APPLICATION CONFIG
bool NvSession::ApplicationConfig(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_CONFIG"))
		return false;

	// This command is authorized only from browser
	if(CommandToExecute->m_iCommandSource != NV_SOURCE_BROWSER)
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:APPLICATION:CONFIG");
		return false;
	}
	
	// We can only configure the application we are logged in
	string ApplicationName = GetAppName();

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	

	string ApplicationProcedureName = "Config/config";


	// Try to execute the procedure but do nothing if it doesn't exist
	if(!ExecuteProcedure(CommandToExecute, ApplicationProcedureName, false))
		return false;


	string ApplicationXslName = "Config/config";
	CommandToExecute->Parameters.SetValue("NV_XML_XSL", ApplicationXslName);		// Application standard xsl file

	return true;
}


//SYSTEM APPLICATION PACKAGE
bool NvSession::ApplicationPackage(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_PACKAGE"))
		return false;

	// Get variable name that should be in command parameters
	string ApplicationName = CommandToExecute->Parameters.GetValue("APPLICATION");
	CleanString(ApplicationName);
	MakeUpper(ApplicationName);

	if(!ApplicationName.length())
		ApplicationName = GetAppName();

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get variable name that should be in command parameters
	string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
	CleanString(FileObjectName);
	MakeUpper(FileObjectName);

	if(!FileObjectName.length())
		FileObjectName = "PACKAGE_FILE";

	NvObjFile *FileObject = (NvObjFile *)OutContainer->GetObject(FileObjectName.c_str());
	if(FileObject == NULL)
	{
		if(!OutContainer->CreateObject(FileObjectName.c_str(), NV_OBJTYPE_FILE))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, FileObjectName);
			return false;
		}
		FileObject = (NvObjFile *)OutContainer->GetObject(FileObjectName.c_str());
		FileObject->SetAutoFileName(Application->GetWorkDir());
	}
	if(FileObject->GetType() != NV_OBJTYPE_FILE)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, FileObjectName);
		return false;
	}

	// Get variable name that should be in command parameters
	string DescriptionFile = CommandToExecute->Parameters.GetValue("DESC_FILE");
	CleanString(DescriptionFile);

	string Info = "";
	if(!GetMain()->Applications.Package(ApplicationName.c_str(), DescriptionFile.c_str(), FileObject->GetFilePathName().c_str(), Info))
	{
		string ErrorMessage = ApplicationName;
		ErrorMessage += " ";
		ErrorMessage += Info;
		CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_CANNOT_CREATE_PACKAGE, ErrorMessage);
		return false;
	}

	return true;
}


//SYSTEM APPLICATION INSTALL
bool NvSession::ApplicationInstall(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_INSTALL"))
		return false;

	// Get variable name that should be in command parameters
	string ApplicationName = CommandToExecute->Parameters.GetValue("APPLICATION");
	CleanString(ApplicationName);
	MakeUpper(ApplicationName);

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get variable name that should be in command parameters
	string FileObjectName = CommandToExecute->Parameters.GetValue("FILE");
	CleanString(FileObjectName);
	MakeUpper(FileObjectName);

	if(!FileObjectName.length())
		FileObjectName = "PACKAGE_FILE";

	NvObjFile *FileObject = (NvObjFile *)InContainer->GetObject(FileObjectName.c_str());
	if(FileObject == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, FileObjectName);
		return false;
	}
	if(FileObject->GetType() != NV_OBJTYPE_FILE)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, FileObjectName);
		return false;
	}

	string ApplicationNameInPackage = "";
	string PlatformInPackage = "";
	// Get application name and platform from package
	
	NvPackage Package;
	vector<std::string> HeaderLines;
	if(!Package.GetHeaderLines(FileObject->GetFilePathName().c_str(), &HeaderLines))
	{
		CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_CANNOT_INSTALL, Package.Info);
		return false;
	}
	int i;
	string::size_type index;
	for(i=0; i<(int)(HeaderLines.size()); i++)
	{
		string Line = HeaderLines[i];
		index = Line.find('=');
		if(index == string::npos)
			continue;	// Bad header entry
		string EntryName = Line.substr(0, index);
		string EntryValue = Line.substr(index+1);
		CleanString(EntryName);
		CleanString(EntryValue);
		MakeUpper(EntryName);
		MakeUpper(EntryValue);
		if(!ApplicationNameInPackage.length())
			if(EntryName == "APPLICATION")
				ApplicationNameInPackage = EntryValue;
		if(!PlatformInPackage.length())
			if(EntryName == "PLATFORM")
				PlatformInPackage = EntryValue;
	}
	MakeUpper(ApplicationNameInPackage);
	CleanString(ApplicationNameInPackage);
	MakeUpper(PlatformInPackage);
	CleanString(PlatformInPackage);

	if(!ApplicationName.length())
		ApplicationName = ApplicationNameInPackage;

	if(!CheckAsciiName(ApplicationName))
	{
		CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_BAD_NAME, ApplicationName);
		return false;
	}
	if(!ApplicationName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "APPLICATION:INSTALL:APPLICATION");
		return false;
	}

	string Platform = "WIN32";
#ifdef AIX
	Platform = "AIX";
#endif
#ifdef LINUX
	Platform = "LINUX";
#endif
#ifdef HPUX
	Platform = "HPUX";
#endif
#ifdef HPUXI
	Platform = "HPUXI";
#endif
#ifdef SOLARIS
	Platform = "SOLARIS";
#endif

	// Now check the platform
	if(PlatformInPackage.length())
	{
		if(PlatformInPackage != Platform)
		{
			CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_BAD_PLATFORM, PlatformInPackage);
			return false;
		}
	}


	string Info = "";

	if(!GetMain()->Applications.Install(ApplicationName.c_str(), FileObject->GetFilePathName().c_str(), Info))
	{
		string ErrorMessage = ApplicationName;
		ErrorMessage += " ";
		ErrorMessage += Info;
		CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_CANNOT_INSTALL, ErrorMessage);
		return false;
	}

	return true;
}



//SYSTEM APPLICATION REDIRECTION_ADD
bool NvSession::ApplicationRedirectionAdd(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_CONFIG"))
		return false;

	string Address = CommandToExecute->Parameters.GetValue("ADDRESS");

	Application->Redirections.AddRedirection(Address.c_str());

	return true;
}


//SYSTEM APPLICATION REDIRECTION_REMOVE
bool NvSession::ApplicationRedirectionRemove(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_CONFIG"))
		return false;

	string Address = CommandToExecute->Parameters.GetValue("ADDRESS");

	Application->Redirections.RemoveRedirection(Address.c_str());

	return true;
}


//SYSTEM APPLICATION REDIRECTION_ENABLE
bool NvSession::ApplicationRedirectionEnable(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "APPLICATION_CONFIG"))
		return false;

	string Address = CommandToExecute->Parameters.GetValue("ADDRESS");
	string sEnable = CommandToExecute->Parameters.GetValue("ENABLE");
	bool bEnable = false;
	CleanString(sEnable);
	MakeUpper(sEnable);
	if(sEnable == "YES")
		bEnable = true;

	Application->Redirections.EnableRedirection(Address.c_str(), bEnable);

	return true;
}


//SYSTEM APPLICATION REDIRECTION_INFO
bool NvSession::ApplicationRedirectionInfo(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	string Address = CommandToExecute->Parameters.GetValue("ADDRESS");

	OutContainer->RemoveObject("REDIRECTIONS");
	if(!OutContainer->CreateObject("REDIRECTIONS", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "REDIRECTIONS");
		return false;
	}

	NvObjTable *RedirectionInfo = (NvObjTable *)(OutContainer->GetObject("REDIRECTIONS"));

	Application->Redirections.GetRedirectionInfo(Address.c_str(), RedirectionInfo);

	return true;
}


//SYSTEM VARIABLE SET
bool NvSession::SetVariable(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	string::size_type index = Name.find("[NV_PARENT]:");
	bool bParent = false;
	if(index != string::npos)
	{
		Name = Name.substr(index+12);
		bParent = true;
	}
	NvSession* Parent = (NvSession*)(CommandToExecute->m_pParentSession);
	if(Parent == NULL)
		bParent = false;


	// Get value that should be in command parameters
	string Value = CommandToExecute->Parameters.GetValue("VALUE");

	if(bParent)
		Parent->m_oVariables.SetValue(Name.c_str(), Value.c_str());
	else
		m_oVariables.SetValue(Name.c_str(), Value.c_str());

	if((CommandToExecute->Parameters.KeyExist("SRC_ENCODING"))||(CommandToExecute->Parameters.KeyExist("DEST_ENCODING")))
		EncodeVariable(CommandToExecute);

	return true;
}


//SYSTEM VARIABLE GET
bool NvSession::GetVariable(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	string::size_type index = Name.find("[NV_PARENT]:");
	bool bParent = false;
	if(index != string::npos)
	{
		Name = Name.substr(index+12);
		bParent = true;
	}
	NvSession* Parent = (NvSession*)(CommandToExecute->m_pParentSession);
	if(Parent == NULL)
		bParent = false;

	if(bParent)
		GetStringValue(CommandToExecute, Parent->m_oVariables.GetValue(Name).c_str(), Buffer, BufferSize, DataSize, iEncoding);
	else
		GetStringValue(CommandToExecute, m_oVariables.GetValue(Name).c_str(), Buffer, BufferSize, DataSize, iEncoding);
	
	return true;
}


//SYSTEM VARIABLE GET_MULTI
bool NvSession::GetMultiVariable(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	MakeUpper(Name);
	string::size_type index = Name.find("[NV_PARENT]:");
	bool bParent = false;
	if(index != string::npos)
	{
		Name = Name.substr(index+12);
		bParent = true;
	}
	NvSession* Parent = (NvSession*)(CommandToExecute->m_pParentSession);
	if(Parent == NULL)
		bParent = false;

	// Get variable name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("VARIABLES");
	if(!ObjName.length())
		ObjName = "VARIABLES";

	OutContainer->RemoveObject(ObjName.c_str());
	if(!OutContainer->CreateObject(ObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjName.c_str());
		return false;
	}

	NvObjIndStringList *ObjVariables = (NvObjIndStringList *)(OutContainer->GetObject(ObjName.c_str()));
	if(ObjVariables == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET, ObjName.c_str());
		return false;
	}

	NvObjIndStringList *Variables = &m_oVariables;
	if(bParent)
		Variables = &(Parent->m_oVariables);
	map<string,string>::iterator pos;
	for(pos = Variables->m_mList.begin(); pos != Variables->m_mList.end(); ++pos)
	{
		string VarName = pos->first.c_str();
		MakeUpper(VarName);
		string VarValue = pos->second.c_str();
		if(Name.length())
		{
			if(VarName.find(Name) != 0)
				continue;
		}
		ObjVariables->SetValue(VarName, VarValue);
	}
	return true;
}



//SYSTEM VARIABLE REMOVE
bool NvSession::RemoveVariable(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	string::size_type index = Name.find("[NV_PARENT]:");
	bool bParent = false;
	if(index != string::npos)
	{
		Name = Name.substr(index+12);
		bParent = true;
	}
	NvSession* Parent = (NvSession*)(CommandToExecute->m_pParentSession);
	if(Parent == NULL)
		bParent = false;

	NvObjIndStringList *Variables = &m_oVariables;
	if(bParent)
		Variables = &(Parent->m_oVariables);

	if(!Name.length())
		Variables->RemoveAll();		// Remove all variables
	else
		Variables->Remove(Name);

	return true;
}


//SYSTEM VARIABLE EXIST
bool NvSession::VariableExist(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	string::size_type index = Name.find("[NV_PARENT]:");
	bool bParent = false;
	if(index != string::npos)
	{
		Name = Name.substr(index+12);
		bParent = true;
	}
	NvSession* Parent = (NvSession*)(CommandToExecute->m_pParentSession);
	if(Parent == NULL)
		bParent = false;

	NvObjIndStringList *Variables = &m_oVariables;
	if(bParent)
		Variables = &(Parent->m_oVariables);

	string result = "NO";
	if(Variables->KeyExist(Name))
		result = "YES";

	GetStringValue(CommandToExecute, result.c_str(), Buffer, BufferSize, DataSize, iEncoding);
	return true;
}




//SYSTEM VARIABLE ENCODE
bool NvSession::EncodeVariable(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	// Get variable name that should be in command parameters
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	string::size_type index = Name.find("[NV_PARENT]:");
	bool bParent = false;
	if(index != string::npos)
	{
		Name = Name.substr(index+12);
		bParent = true;
	}
	NvSession* Parent = (NvSession*)(CommandToExecute->m_pParentSession);
	if(Parent == NULL)
		bParent = false;

	NvObjIndStringList *Variables = &m_oVariables;
	if(bParent)
		Variables = &(Parent->m_oVariables);
	
	if(!Variables->KeyExist(Name))
		return true;	// nothing to do

	string sSrcEncoding = CommandToExecute->Parameters.GetValue("SRC_ENCODING");
	CleanString(sSrcEncoding, true);
	MakeUpper(sSrcEncoding);
	if(!sSrcEncoding.length())
		sSrcEncoding = "INTERNAL";
	bool bSrcUnicode = false;
	if(sSrcEncoding == "UTF-8")
		bSrcUnicode = true;
	if(sSrcEncoding == "INTERNAL")
		bSrcUnicode = GetMain()->IsUnicode();

	string sDestEncoding = CommandToExecute->Parameters.GetValue("DEST_ENCODING");
	CleanString(sDestEncoding, true);
	MakeUpper(sDestEncoding);
	if(!sDestEncoding.length())
		sDestEncoding = "INTERNAL";
	bool bDestUnicode = false;
	if(sDestEncoding == "UTF-8")
		bDestUnicode = true;
	if(sDestEncoding == "INTERNAL")
		bDestUnicode = GetMain()->IsUnicode();

	if(bSrcUnicode == bDestUnicode)
		return true;	// nothing to do

	string Value = Variables->GetValue(Name);
	EncodeString(Value, bSrcUnicode, bDestUnicode);  
	Variables->SetValue(Name.c_str(), Value.c_str());

	return true;
}


bool NvSession::CreateObject(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name and type that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);
	if(!ObjName.length())
		ObjName = NV_OBJ_DEFAULT_NAME;

	string ObjType = CommandToExecute->Parameters.GetValue("TYPE");
	CleanString(ObjType);
	MakeUpper(ObjType);
	
	if(!ObjType.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_NO_TYPE, ObjName);
		return false;
	}

	int iObjectType = NV_OBJTYPE_UNKNOWN;
	if(ObjType == "BOOLEAN")
		iObjectType = NV_OBJTYPE_BOOLEAN;
	if(ObjType == "INTEGER")
		iObjectType = NV_OBJTYPE_INTEGER;
	if(ObjType == "STRING")
		iObjectType = NV_OBJTYPE_STRING;
	if(ObjType == "STRINGLIST")
		iObjectType = NV_OBJTYPE_STRINGLIST;
	if(ObjType == "INDSTRINGLIST")
		iObjectType = NV_OBJTYPE_INDSTRINGLIST;
	if(ObjType == "TABLE")
		iObjectType = NV_OBJTYPE_TABLE;
	if(ObjType == "FILE")
		iObjectType = NV_OBJTYPE_FILE;
	if(ObjType == "BINARY")
		iObjectType = NV_OBJTYPE_BINARY;

	
	if(iObjectType == NV_OBJTYPE_UNKNOWN)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_UNKNOWN_TYPE, ObjName);
		return false;
	}

		
	NvObj *Object = InContainer->GetObject(ObjName.c_str());

	if(Object != NULL)
	{
		// Check for the no-replace mode
		string ReplaceMode = CommandToExecute->Parameters.GetValue("REPLACE");
		CleanString(ReplaceMode);
		MakeUpper(ReplaceMode);
		if(ReplaceMode != "YES")
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_EVER_EXIST, ObjName);
			return false;
		}

		// The object exists so we remove it
		InContainer->RemoveObject(ObjName.c_str());
		Object = NULL;
	}
		
	if(!InContainer->CreateObject(ObjName.c_str(), iObjectType))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjName);
		return false;
	}
	
	Object = InContainer->GetObject(ObjName.c_str());
	if(Object == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjName);
		return false;
	}

	if(iObjectType == NV_OBJTYPE_FILE)
	{
		NvObjFile *ObjectFile = (NvObjFile *)Object;
		string FileName = CommandToExecute->Parameters.GetValue("FILENAME");
		SetDirName(FileName);
	
		// If the order comes from client or browser, only auto filename is authorized
		if(CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT)
			FileName = "";
		if(CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER)
			FileName = "";
					
		string PersistTimeFileInCommand = CommandToExecute->Parameters.GetValue("PERSIST");
		CleanString(PersistTimeFileInCommand);
		MakeUpper(PersistTimeFileInCommand);
		int iPersistTimeFileInCommand = 0;	// The file is not persistant by default on the server
		if(PersistTimeFileInCommand.length())
			iPersistTimeFileInCommand = atoi(PersistTimeFileInCommand.c_str());
		if(iPersistTimeFileInCommand < 0)
			iPersistTimeFileInCommand = -1;


		if(FileName.length())
		{
			string sWorkDir = "";
			sWorkDir = Application->GetWorkDir();

			// Check if input file has a complete path or not
			string::size_type index = FileName.rfind('/');
			if(index == string::npos)
				index = FileName.rfind('\\');
			if(index == string::npos)
			{
				// There is no complete path
				// So the file will go to the application work dir
				string NewFileName = sWorkDir;
				// or to the application file dir if persistent
				if(iPersistTimeFileInCommand == -1)
					NewFileName = Application->GetFileDir();
				NewFileName += FileName;
				FileName = NewFileName;
			}
		
			if(iPersistTimeFileInCommand == -1)
			{
				string TargetDir = FileName;
				index = TargetDir.rfind('/');
				if(index == string::npos)
					index = TargetDir.rfind('\\');
				if(index != string::npos)
					TargetDir = TargetDir.substr(0, index+1);
				if(CompareWorkDir(TargetDir, sWorkDir))
				{
					CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_SET_PERSISTANT, FileName);
					InContainer->RemoveObject(ObjName.c_str());
					return false;
				}
			}
			
			bool bCreateIt = true;
			string sCreate = CommandToExecute->Parameters.GetValue("CREATE");
			MakeUpper(sCreate);
			CleanString(sCreate, true);
			if(sCreate == "NO")
				bCreateIt = false;
			
			if(bCreateIt)
			{
				if(!FileExist(FileName.c_str()))
				{
					FILE *File;
					File = Nvfopen(FileName.c_str(), "w");
					if(File != NULL)
						fclose(File);
				}
			}
			ObjectFile->SetFileName(FileName);
		}
		else
		{
		
			// If no file name given,
			// We try to create automatically a file following some information in the command
			string TargetDir = CommandToExecute->Parameters.GetValue("DIRECTORY");
			SetDirName(TargetDir);
			if(!TargetDir.length())
			{
				if(iPersistTimeFileInCommand == -1)
					TargetDir = Application->GetFileDir();
				else
					TargetDir = Application->GetWorkDir();
			}

			CleanString(TargetDir);
		
			string sWorkDir = "";
			sWorkDir = Application->GetWorkDir();

			
			string FileExtensionInCommand = CommandToExecute->Parameters.GetValue("EXTENSION");
			CleanString(FileExtensionInCommand);
			if(!FileExtensionInCommand.length())
			{
				if(iPersistTimeFileInCommand == -1)
					FileExtensionInCommand = "obj";
				else
					FileExtensionInCommand = "tmp";
			}
			string FilePrefixInCommand = CommandToExecute->Parameters.GetValue("PREFIX");
			CleanString(FilePrefixInCommand);
			if(!FilePrefixInCommand.length())
				FilePrefixInCommand = Application->GetAppNameForFiles();
			string FileSuffixInCommand = CommandToExecute->Parameters.GetValue("SUFFIX");
			
			CleanString(FileSuffixInCommand);
			if(iPersistTimeFileInCommand == -1)
			{
				if(CompareWorkDir(TargetDir, sWorkDir))
				{
					CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_SET_PERSISTANT, ObjName);
					InContainer->RemoveObject(ObjName.c_str());
					return false;
				}
			}
			if(!ObjectFile->SetAutoFileName(TargetDir, FileExtensionInCommand, FilePrefixInCommand, FileSuffixInCommand))
			{
				char *ErrorMessage = (char *)malloc(ObjName.length() + TargetDir.length() + FileExtensionInCommand.length()  + FilePrefixInCommand.length()  + FileSuffixInCommand.length() + 100);
				sprintf(ErrorMessage, "obj:%s - dir:%s - ext:%s - prefix:%s - suffix:%s", ObjName.c_str(), TargetDir.c_str(), FileExtensionInCommand.c_str(), FilePrefixInCommand.c_str(), FileSuffixInCommand.c_str());
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_CREATE, ErrorMessage);
				free(ErrorMessage);
				InContainer->RemoveObject(ObjName.c_str());
				return false;
			}
		}
		ObjectFile->SetPersistTime(iPersistTimeFileInCommand);
	}
	if(iObjectType == NV_OBJTYPE_BOOLEAN)
	{
		NvObjBoolean *ObjectBoolean = (NvObjBoolean *)Object;
		
		if(CommandToExecute->Parameters.KeyExist("VALUE"))
		{
			string ObjValue = CommandToExecute->Parameters.GetValue("VALUE");
			CleanString(ObjValue);
			MakeUpper(ObjValue);

			// The default is false
			// Any other value than "TRUE" is considered as FALSE
			bool NewValue = false;
			if(ObjValue == "TRUE")
				NewValue = true;

			ObjectBoolean->SetValue(NewValue);
		}
	}
	if(iObjectType == NV_OBJTYPE_INTEGER)
	{
		NvObjInteger *ObjectInteger = (NvObjInteger *)Object;
		if(CommandToExecute->Parameters.KeyExist("VALUE"))
		{
			string ObjValue = CommandToExecute->Parameters.GetValue("VALUE");
			CleanString(ObjValue);
			MakeUpper(ObjValue);
			int NewValue = atoi(ObjValue.c_str());
			ObjectInteger->SetValue(NewValue);
		}
	}
	if(iObjectType == NV_OBJTYPE_STRING)
	{
		NvObjString *ObjectString = (NvObjString *)Object;
		if(CommandToExecute->Parameters.KeyExist("VALUE"))
		{
			string ObjValue = CommandToExecute->Parameters.GetValue("VALUE");
			ObjectString->SetValue(ObjValue);
		}
	}
	if(iObjectType == NV_OBJTYPE_STRINGLIST)
	{
		NvObjStringList *ObjectStringList = (NvObjStringList *)Object;
		if(CommandToExecute->Parameters.KeyExist("VALUE"))
		{
			string ObjValue = CommandToExecute->Parameters.GetValue("VALUE");
			string Separator = CommandToExecute->Parameters.GetValue("SEPARATOR");
			ObjectStringList->Add(ObjValue, Separator);
		}
	}
	if(iObjectType == NV_OBJTYPE_INDSTRINGLIST)
	{
		NvObjIndStringList *ObjectIndStringList = (NvObjIndStringList *)Object;
		string CaseSensitive = CommandToExecute->Parameters.GetValue("CASE_SENSITIVE");
		CleanString(CaseSensitive);
		MakeUpper(CaseSensitive);
		if(CaseSensitive == "YES")
			ObjectIndStringList->m_bCaseSensitive = true;

		if(CommandToExecute->Parameters.KeyExist("VALUE"))
		{
			string ObjValue = CommandToExecute->Parameters.GetValue("VALUE");
			string KeySep = ";";
			string ValSep = "=";
			string KeySepHere = CommandToExecute->Parameters.GetValue("KEYSEP");
			CleanString(KeySepHere);
			if(KeySepHere.length())
				KeySep = KeySepHere;
			string ValSepHere = CommandToExecute->Parameters.GetValue("VALSEP");
			CleanString(ValSepHere);
			if(ValSepHere.length())
				ValSep = ValSepHere;
			string ObjKey = "";
			ObjectIndStringList->SetValue(ObjKey, ObjValue, KeySep, ValSep);
		}
	}
	if(iObjectType == NV_OBJTYPE_TABLE)
	{
		NvObjTable *ObjectTable = (NvObjTable *)Object;

		string ColumnsHere = CommandToExecute->Parameters.GetValue("COLUMNS");
		CleanString(ColumnsHere);
		if(ColumnsHere.length())
		{
			string::size_type index;
			while(ColumnsHere.length())
			{
				string Column = ColumnsHere;
				string ColumnDesc = "";
				bool ColumnNumeric = false;
				index = Column.find(';');
				if(index != string::npos)
				{
					Column = ColumnsHere.substr(0, index);
					ColumnsHere = ColumnsHere.substr(index+1);
				}
				else
					ColumnsHere = "";
				index = Column.find(':');
				if(index != string::npos)
				{
					ColumnDesc = Column.substr(index+1);
					Column = Column.substr(0, index);
				}
				index = ColumnDesc.find(':');
				if(index != string::npos)
				{
					string sColumnNumeric = ColumnDesc.substr(index+1);
					ColumnDesc = ColumnDesc.substr(0, index);
					CleanString(sColumnNumeric);
					MakeUpper(sColumnNumeric);
					if(sColumnNumeric == "Y")
						ColumnNumeric = true;
				}
				CleanString(Column, true);
				CleanString(ColumnDesc);
				if(!Column.length())
					continue;
				ObjectTable->InsertColumn(Column, ColumnDesc, ColumnNumeric);
			}
		}

		if(CommandToExecute->Parameters.KeyExist("VALUE"))
		{
			string ObjValue = CommandToExecute->Parameters.GetValue("VALUE");
			
			// Get eventual record separator
			string RecSep = CommandToExecute->Parameters.GetValue("ROWSEP");
			if(!RecSep.length())
				RecSep = "\n";

			// Get eventual column separator
			string ColSep = CommandToExecute->Parameters.GetValue("COLSEP");
			if(!ColSep.length())
				ColSep = ";";

			// Get eventual line separator
			string LineSep = CommandToExecute->Parameters.GetValue("LINESEP");
			if(!LineSep.length())
				LineSep = "|";

			if(!ObjectTable->AddRecords(ObjValue.c_str(), RecSep.c_str(), ColSep.c_str(), LineSep.c_str()))
			{
				InContainer->RemoveObject(ObjName.c_str());
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_ADD_RECORDS, ObjName);
				return false;
			}
		}
	}
	
	return true;
}




bool NvSession::GetNumObjects(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	char Buf[50];
	sprintf(Buf, "%d", InContainer->m_mObjects.size());
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




bool NvSession::GetObjectName(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object index that should be in command parameters
	int iObjIndex = 0;
	string ObjIndex = CommandToExecute->Parameters.GetValue("INDEX");
	CleanString(ObjIndex);
	iObjIndex = atoi(ObjIndex.c_str());
	
	if((iObjIndex < 1) || (iObjIndex > (int)(InContainer->m_mObjects.size())))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT_INDEX, ObjIndex);
		return false;
	}

	map<string,void *>::iterator pos;
	int i;
	NvObj* Object = NULL;
	for(i=0,pos = InContainer->m_mObjects.begin();i<iObjIndex;i++,++pos)
	{
		Object = (NvObj*)(pos->second);
	}
	string ObjectName = "";
	if(Object != NULL)
		ObjectName = Object->GetName();

	GetStringValue(CommandToExecute, ObjectName.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;

}




bool NvSession::GetObjectType(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObj* Object = InContainer->GetObject(ObjName.c_str());

	if(Object == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, ObjName);
		return false;
	}

	char Buf[100];
	switch(Object->GetType())
	{
		case NV_OBJTYPE_BOOLEAN:
			strcpy(Buf, "BOOLEAN");
			break;
		case NV_OBJTYPE_INTEGER:
			strcpy(Buf, "INTEGER");
			break;
		case NV_OBJTYPE_STRING:
			strcpy(Buf, "STRING");
			break;
		case NV_OBJTYPE_STRINGLIST:
			strcpy(Buf, "STRINGLIST");
			break;
		case NV_OBJTYPE_INDSTRINGLIST:
			strcpy(Buf, "INDSTRINGLIST");
			break;
		case NV_OBJTYPE_TABLE:
			strcpy(Buf, "TABLE");
			break;
		case NV_OBJTYPE_FILE:
			strcpy(Buf, "FILE");
			break;
		case NV_OBJTYPE_BINARY:
			strcpy(Buf, "BINARY");
			break;
		default:
			strcpy(Buf, "UNKNOWN");
			break;
	}

	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;

}


NvObj* NvSession::GetObjectCheckType(NvCommand *CommandToExecute, NvContainer *Container, const char *ObjName, int ObjType)
{
	
	if(ObjName == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT);
		return NULL;
	}
	NvObj* Object = Container->GetObject(ObjName);

	if(Object == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_OBJECT, ObjName);
		return NULL;
	}

	if((ObjType != -1) && (Object->GetType() != ObjType))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_BAD_TYPE, ObjName);
		return NULL;
	}

	return Object;
}


//OBJECT COPY
bool NvSession::CopyObject(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	
	// Get object source and destination names that should be in command parameters
	string SObjName = CommandToExecute->Parameters.GetValue("SNAME");
	CleanString(SObjName);
	string DObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(DObjName);

	NvObj* SourceObject = GetObjectCheckType(CommandToExecute, InContainer, SObjName.c_str());

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	NvObj* DestinationObject = GetObjectCheckType(CommandToExecute, OutContainer, DObjName.c_str());

	if(DestinationObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	if(SourceObject == DestinationObject)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_SAME_OBJECTS, SObjName);
		return false;
	}

	int iType = SourceObject->GetType();
	if(iType != DestinationObject->GetType())
	{
		char *ErrorMessage = (char *)malloc(SObjName.length() + DObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", SObjName.c_str(), DObjName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECTS_NOT_COMPATIBLE, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	if((iType <= NV_OBJTYPE_UNKNOWN) || (iType > NV_OBJTYPE_MAXTYPEVALUE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_UNKNOWN_TYPE, SObjName);
		return false;
	}

	// Check if an offset and a number of bytes has been requested
	// They are used for file and binary types
	NvInt64 Offset = 0;
	NvInt64 BytesToCopy = -1;
	if(CommandToExecute->Parameters.KeyExist("OFFSET"))
	{
		string sOffset = CommandToExecute->Parameters.GetValue("OFFSET");
		CleanString(sOffset);
		Offset = NvAtoi64(sOffset.c_str());
	}
	if(CommandToExecute->Parameters.KeyExist("NUM_BYTES"))
	{
		string sBytesToCopy = CommandToExecute->Parameters.GetValue("NUM_BYTES");
		CleanString(sBytesToCopy);
		BytesToCopy = NvAtoi64(sBytesToCopy.c_str());
	}

	bool RetVal = true;
	if(iType == NV_OBJTYPE_BOOLEAN)
	{
		NvObjBoolean *Sobj = (NvObjBoolean*)SourceObject;
		NvObjBoolean *Dobj = (NvObjBoolean*)DestinationObject;
		Dobj->CopyFrom(Sobj);
	}
	if(iType == NV_OBJTYPE_INTEGER)
	{
		NvObjInteger *Sobj = (NvObjInteger*)SourceObject;
		NvObjInteger *Dobj = (NvObjInteger*)DestinationObject;
		Dobj->CopyFrom(Sobj);
	}
	if(iType == NV_OBJTYPE_STRING)
	{
		NvObjString *Sobj = (NvObjString*)SourceObject;
		NvObjString *Dobj = (NvObjString*)DestinationObject;
		Dobj->CopyFrom(Sobj);
	}
	if(iType == NV_OBJTYPE_STRINGLIST)
	{
		NvObjStringList *Sobj = (NvObjStringList*)SourceObject;
		NvObjStringList *Dobj = (NvObjStringList*)DestinationObject;
		Dobj->CopyFrom(Sobj);
	}
	if(iType == NV_OBJTYPE_INDSTRINGLIST)
	{
		NvObjIndStringList *Sobj = (NvObjIndStringList*)SourceObject;
		NvObjIndStringList *Dobj = (NvObjIndStringList*)DestinationObject;
		Dobj->CopyFrom(Sobj);
	}
	if(iType == NV_OBJTYPE_TABLE)
	{
		NvObjTable *Sobj = (NvObjTable*)SourceObject;
		NvObjTable *Dobj = (NvObjTable*)DestinationObject;
		Dobj->CopyFrom(Sobj);
	}
	if(iType == NV_OBJTYPE_FILE)
	{
		NvObjFile *Sobj = (NvObjFile*)SourceObject;
		NvObjFile *Dobj = (NvObjFile*)DestinationObject;
		if(!(Dobj->GetFileName().length()))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_NO_FILE, DObjName);
			return false;
		}
		RetVal = Dobj->CopyFrom(Sobj, Offset, BytesToCopy);
	}
	if(iType == NV_OBJTYPE_BINARY)
	{
		NvObjBinary *Sobj = (NvObjBinary*)SourceObject;
		NvObjBinary *Dobj = (NvObjBinary*)DestinationObject;
		RetVal = Dobj->CopyFrom(Sobj, (int)Offset, (int)BytesToCopy);
	}

	if(!RetVal)
	{
		char *ErrorMessage = (char *)malloc(SObjName.length() + DObjName.length() + 100);
		sprintf(ErrorMessage, "%s -> %s", SObjName.c_str(), DObjName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_COPY, ErrorMessage);
		free(ErrorMessage);
	}
	return RetVal;
}



//OBJECT MOVE
bool NvSession::MoveObject(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	if(InContainer == OutContainer)
		return true; // moving to same container is a no-op

	
	// Get object source and destination names that should be in command parameters
	string SObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(SObjName);

	NvObj* SourceObject = GetObjectCheckType(CommandToExecute, InContainer, SObjName.c_str());

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	bool bReplace = false;
	string sReplace = CommandToExecute->Parameters.GetValue("REPLACE");
	CleanString(sReplace);
	MakeUpper(sReplace);
	if(sReplace == "YES")
		bReplace = true;
	if(!InContainer->MoveObject(OutContainer, SObjName.c_str(), bReplace))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_MOVE, SObjName);
		return false;
	}

	return true;
}



//OBJECT REMOVE
bool NvSession::SetObjectName(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);
	if(!ObjName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:SET_NAME:NAME");
		return false;
	}

	// Get object name that should be in command parameters
	string NewObjName = CommandToExecute->Parameters.GetValue("NEW_NAME");
	CleanString(NewObjName);
	if(!NewObjName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:SET_NAME:NEW_NAME");
		return false;
	}


	NvObj* Object = InContainer->GetObject(NewObjName.c_str());
	if(Object != NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_EVER_EXIST, NewObjName);
		return false;
	}


	Object = InContainer->GetObject(ObjName.c_str());
	if(Object == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET, ObjName);
		return false;
	}

	InContainer->m_mObjects.erase(Object->GetName());
	Object->SetName(NewObjName);
	InContainer->m_mObjects[Object->GetName()] = (void *)Object;

	return true;
}




//OBJECT REMOVE
bool NvSession::RemoveObject(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	InContainer->RemoveObject(ObjName.c_str());

	return true;
}




//OBJECT EXIST
bool NvSession::IsObjectExist(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	char Buf[20];
	strcpy(Buf, "NO");
	if(InContainer->GetObject(ObjName.c_str()) != NULL)
		strcpy(Buf, "YES");

	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}



//SYSTEM OBJECT ENCODE
bool NvSession::EncodeObject(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObj *Object = InContainer->GetObject(ObjName.c_str());
	if(Object == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET, ObjName);
		return false;
	}

	string sSrcEncoding = CommandToExecute->Parameters.GetValue("SRC_ENCODING");
	CleanString(sSrcEncoding, true);
	MakeUpper(sSrcEncoding);
	if(!sSrcEncoding.length())
		sSrcEncoding = "INTERNAL";
	bool bSrcUnicode = false;
	if(sSrcEncoding == "UTF-8")
		bSrcUnicode = true;
	if(sSrcEncoding == "INTERNAL")
		bSrcUnicode = GetMain()->IsUnicode();

	string sDestEncoding = CommandToExecute->Parameters.GetValue("DEST_ENCODING");
	CleanString(sDestEncoding, true);
	MakeUpper(sDestEncoding);
	if(!sDestEncoding.length())
		sDestEncoding = "INTERNAL";
	bool bDestUnicode = false;
	if(sDestEncoding == "UTF-8")
		bDestUnicode = true;
	if(sDestEncoding == "INTERNAL")
		bDestUnicode = GetMain()->IsUnicode();

	if(bSrcUnicode == bDestUnicode)
		return true;	// nothing to do

	string sWithName = CommandToExecute->Parameters.GetValue("WITH_NAME");
	CleanString(sWithName, true);
	MakeUpper(sWithName);
	bool bWithName = false;
	if(sWithName == "YES")
		bWithName = true;

	if(Object->GetType() == NV_OBJTYPE_STRING)
		((NvObjString*)Object)->Encode(bSrcUnicode, bDestUnicode);
	if(Object->GetType() == NV_OBJTYPE_STRINGLIST)
		((NvObjStringList*)Object)->Encode(bSrcUnicode, bDestUnicode);
	if(Object->GetType() == NV_OBJTYPE_INDSTRINGLIST)
		((NvObjIndStringList*)Object)->Encode(bSrcUnicode, bDestUnicode);
	if(Object->GetType() == NV_OBJTYPE_TABLE)
		((NvObjTable*)Object)->Encode(bSrcUnicode, bDestUnicode);

	if(bWithName)
		Object->EncodeName(bSrcUnicode, bDestUnicode);

	return true;
}


//OBJECT CLEAR_ALL
bool NvSession::ClearAllObjects(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	InContainer->ClearObjects();

	return true;
}


//OBJECT BOOLEAN_SET_VALUE
bool NvSession::ObjBooleanSetValue(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object value that should be in command parameters
	string ObjValue = CommandToExecute->Parameters.GetValue("VALUE");
	CleanString(ObjValue);
	MakeUpper(ObjValue);

	// The default is false
	// Any other value than "TRUE" is considered as FALSE
	bool NewValue = false;
	if(ObjValue == "TRUE")
		NewValue = true;



	NvObjBoolean* Object = (NvObjBoolean*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_BOOLEAN);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->SetValue(NewValue);

	return true;
}


//OBJECT BOOLEAN_GET_VALUE
bool NvSession::ObjBooleanGetValue(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjBoolean* Object = (NvObjBoolean*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_BOOLEAN);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	bool Value = Object->GetValue();

	char RetVal[20];
	strcpy(RetVal, "FALSE");
	if(Value)
		strcpy(RetVal, "TRUE");
	
	GetStringValue(CommandToExecute, RetVal, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}





//OBJECT INTEGER_SET_VALUE
bool NvSession::ObjIntegerSetValue(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object value that should be in command parameters
	string ObjValue = CommandToExecute->Parameters.GetValue("VALUE");
	CleanString(ObjValue);

	int NewValue = atoi(ObjValue.c_str());

	NvObjInteger* Object = (NvObjInteger*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_INTEGER);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->SetValue(NewValue);

	return true;
}


//OBJECT INTEGER_GET_VALUE
bool NvSession::ObjIntegerGetValue(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjInteger* Object = (NvObjInteger*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_INTEGER);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	int Value = Object->GetValue();

	char RetVal[50];
	sprintf(RetVal, "%d", Value);
	
	GetStringValue(CommandToExecute, RetVal, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}







//OBJECT STRING_SET_VALUE
bool NvSession::ObjStringSetValue(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object value that should be in command parameters
	string ObjValue = CommandToExecute->Parameters.GetValue("VALUE");

	NvObjString* Object = (NvObjString*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_STRING);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->SetValue(ObjValue);

	return true;
}


//OBJECT STRING_GET_VALUE
bool NvSession::ObjStringGetValue(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjString* Object = (NvObjString*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_STRING);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	GetStringValue(CommandToExecute, Object->GetValue().c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}






//OBJECT STRINGLIST_SET_VALUE
bool NvSession::ObjStringListSetValue(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object value that should be in command parameters
	string ObjValue = CommandToExecute->Parameters.GetValue("VALUE");

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	int iIndex = 1;
	bool WithIndex = false;
	if(CommandToExecute->Parameters.KeyExist("INDEX"))
	{
		string sIndex = CommandToExecute->Parameters.GetValue("INDEX");
		CleanString(sIndex);
		iIndex = atoi(sIndex.c_str());
		WithIndex = true;
	}

	if(!WithIndex)					// If no index, we just add the value to the list
	{
		// Get eventual separator
		string Separator = CommandToExecute->Parameters.GetValue("SEPARATOR");
		Object->Add(ObjValue, Separator);
	}
	else
	{
		iIndex--;	// The real index starts at 0
		if(!Object->CheckIndex(iIndex))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_STRINGLIST_INDEX, ObjName);
			return false;
		}
		Object->SetValue(iIndex, ObjValue);
	}

	return true;
}


//OBJECT STRINGLIST_GET_VALUE
bool NvSession::ObjStringListGetValue(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	int iIndex = 1;
	bool WithIndex = false;
	bool LastIndex = false;
	bool FirstIndex = false;
	if(CommandToExecute->Parameters.KeyExist("INDEX"))
	{
		string sIndex = CommandToExecute->Parameters.GetValue("INDEX");
		CleanString(sIndex);
		MakeUpper(sIndex);
		if(sIndex == "FIRST")
			FirstIndex = true;
		if(sIndex == "LAST")
			LastIndex = true;
		iIndex = atoi(sIndex.c_str());
		WithIndex = true;
	}
	if(!WithIndex)	// We take last index
		iIndex = Object->GetSize();
	if(FirstIndex)	// We take first index
		iIndex = 1;
	if(LastIndex)	// We take last index
		iIndex = Object->GetSize();

	iIndex--;	// The real index starts at 0
	if(!Object->CheckIndex(iIndex))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_STRINGLIST_INDEX, ObjName);
		return false;
	}

	GetStringValue(CommandToExecute, Object->GetValue(iIndex).c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}


//OBJECT STRINGLIST_GET_SIZE
bool NvSession::ObjStringListGetSize(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	char Buf[50];
	sprintf(Buf, "%d", Object->GetSize());
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT STRINGLIST_REMOVE
bool NvSession::ObjStringListRemove(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	int iIndex = 1;
	bool WithIndex = false;
	bool LastIndex = false;
	bool FirstIndex = false;
	bool AllIndex = false;
	if(CommandToExecute->Parameters.KeyExist("INDEX"))
	{
		string sIndex = CommandToExecute->Parameters.GetValue("INDEX");
		CleanString(sIndex);
		MakeUpper(sIndex);
		if(sIndex == "FIRST")
			FirstIndex = true;
		if(sIndex == "LAST")
			LastIndex = true;
		if(sIndex == "ALL")
			AllIndex = true;
		iIndex = atoi(sIndex.c_str());
		WithIndex = true;
	}
	if((!WithIndex) || AllIndex)	// We remove all
	{
		Object->RemoveAll();
		return true;
	}
	if(FirstIndex)	// We take first index
		iIndex = 1;
	if(LastIndex)	// We take last index
		iIndex = Object->GetSize();

	iIndex--;	// The real index starts at 0
	if(!Object->CheckIndex(iIndex))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_STRINGLIST_INDEX, ObjName);
		return false;
	}
	if(LastIndex)	// We take last index
	{
		Object->RemoveLast();
		return true;
	}

	Object->Remove(iIndex);

	return true;
}




//OBJECT STRINGLIST_INSERT
bool NvSession::ObjStringListInsert(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object value that should be in command parameters
	string ObjValue = CommandToExecute->Parameters.GetValue("VALUE");

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	int iIndex = 1;
	bool WithIndex = false;
	if(CommandToExecute->Parameters.KeyExist("INDEX"))
	{
		string sIndex = CommandToExecute->Parameters.GetValue("INDEX");
		CleanString(sIndex);
		iIndex = atoi(sIndex.c_str());
		WithIndex = true;
	}

	if(!WithIndex)					// If no index, we just add the value to the list
	{
		// Get eventual separator
		string Separator = CommandToExecute->Parameters.GetValue("SEPARATOR");
		Object->Add(ObjValue, Separator);
	}
	else
	{
		iIndex--;	// The real index starts at 0
		if((!Object->CheckIndex(iIndex)) && (iIndex < Object->GetSize()))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_STRINGLIST_INDEX, ObjName);
			return false;
		}
		Object->Insert(iIndex, ObjValue);
	}

	return true;
}



//OBJECT STRINGLIST_ADD_STRINGLIST
bool NvSession::ObjStringListAddStringList(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object source and destination names that should be in command parameters
	string SObjName = CommandToExecute->Parameters.GetValue("SNAME");
	CleanString(SObjName);
	string DObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(DObjName);
	
	bool bUseOutputContainer = false;
	string sUseOutputContainer = CommandToExecute->Parameters.GetValue("USE_OUT_CONTAINER");
	CleanString(sUseOutputContainer);
	MakeUpper(sUseOutputContainer);
	if(sUseOutputContainer == "YES")
		bUseOutputContainer = true;

	NvObjStringList* SourceObject = (NvObjStringList*)GetObjectCheckType(CommandToExecute, InContainer, SObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	NvObjStringList* DestinationObject = NULL;
	if(bUseOutputContainer)
		DestinationObject = (NvObjStringList*)GetObjectCheckType(CommandToExecute, OutContainer, DObjName.c_str(), NV_OBJTYPE_STRINGLIST);
	else
		DestinationObject = (NvObjStringList*)GetObjectCheckType(CommandToExecute, InContainer, DObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(DestinationObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	DestinationObject->Add(SourceObject);

	return true;
}






//OBJECT STRINGLIST_SWAP
bool NvSession::ObjStringListSwap(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	int iIndex1 = -1;
	if(!CommandToExecute->Parameters.KeyExist("INDEX1"))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_STRINGLIST_INDEX, ObjName);
		return false;
	}
	string sIndex = CommandToExecute->Parameters.GetValue("INDEX1");
	CleanString(sIndex);
	iIndex1 = atoi(sIndex.c_str());
	
	int iIndex2 = -1;
	if(!CommandToExecute->Parameters.KeyExist("INDEX2"))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_STRINGLIST_INDEX, ObjName);
		return false;
	}
	sIndex = CommandToExecute->Parameters.GetValue("INDEX2");
	CleanString(sIndex);
	iIndex2 = atoi(sIndex.c_str());
	
	iIndex1--;	// The real index starts at 0
	iIndex2--;	// The real index starts at 0
	
	if(!Object->CheckIndex(iIndex1))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_STRINGLIST_INDEX, ObjName);
		return false;
	}

	if(!Object->CheckIndex(iIndex1))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_STRINGLIST_INDEX, ObjName);
		return false;
	}

	Object->Swap(iIndex1, iIndex2);

	return true;
}






//OBJECT STRINGLIST_SORT
bool NvSession::ObjStringListSort(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	bool bNumeric = false;
	string sNumeric = CommandToExecute->Parameters.GetValue("NUMERIC");
	CleanString(sNumeric);
	MakeUpper(sNumeric);
	if(sNumeric == "YES")
		bNumeric = true;

	Object->Sort(bNumeric);

	return true;
}


//OBJECT STRINGLIST_GET_VALUES
bool NvSession::ObjStringListGetValues(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	string What = CommandToExecute->Parameters.GetValue("WHAT");
	CleanString(What);
	MakeUpper(What);
	if(!What.length())
		What = "NV_ALL";
	int From = 1;
	int To = Object->GetSize();
	if(What != "NV_ALL")
	{
		// range
		string sFrom = What;
		string sTo = "";
		string::size_type index = What.find('-');
		if(index == string::npos)
		{
			// Only one value telling exactly what to get
			From = atoi(What.c_str());
			To = From;
		}
		else
		{
			sFrom = What.substr(0, index);
			sTo = What.substr(index+1);
			From = atoi(sFrom.c_str());
			To = atoi(sTo.c_str());;
		}
		if(From <1)
			From = 1;
		if(To <1)
			To = 1;
		if(To > Object->GetSize())
			To = Object->GetSize();
		if(To < From)
			To = From;
	}

	// Real index starts from 0
	From--;
	To--;

	string Separator = CommandToExecute->Parameters.GetValue("SEPARATOR");
	if(!Separator.length())
		Separator = ";";


	GetStringValue(CommandToExecute, Object->GetValues(From, To, Separator).c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}


//OBJECT STRINGLIST_SEARCH
bool NvSession::ObjStringListSearch(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	string Value = CommandToExecute->Parameters.GetValue("VALUE");
	CleanString(Value);

	bool bCase = false;
	string sCase = CommandToExecute->Parameters.GetValue("CASE");
	CleanString(sCase);
	MakeUpper(sCase);
	if(sCase == "YES")
		bCase = true;

	bool bFirst = true;
	string sFirst = CommandToExecute->Parameters.GetValue("FIRST_ONLY");
	CleanString(sFirst);
	MakeUpper(sFirst);
	if(sFirst == "NO")
		bFirst = false;
	
	GetStringValue(CommandToExecute, Object->Search(Value, bCase, bFirst).c_str(), Buffer, BufferSize, DataSize, iEncoding);
	
	return true;
}







//OBJECT INDSTRINGLIST_SET_VALUE
bool NvSession::ObjIndStringListSetValue(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object value that should be in command parameters
	string ObjValue = CommandToExecute->Parameters.GetValue("VALUE");

	// Get object key that should be in command parameters
	string ObjKey = CommandToExecute->Parameters.GetValue("KEY");
	CleanString(ObjKey);
//	MakeUpper(ObjKey);

	string KeySep = ";";
	string ValSep = "=";
	if(!ObjKey.length())
	{
		string KeySepHere = CommandToExecute->Parameters.GetValue("KEYSEP");
		CleanString(KeySepHere);
		if(KeySepHere.length())
			KeySep = KeySepHere;
		string ValSepHere = CommandToExecute->Parameters.GetValue("VALSEP");
		CleanString(ValSepHere);
		if(ValSepHere.length())
			ValSep = ValSepHere;
	}


	NvObjIndStringList* Object = (NvObjIndStringList*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->SetValue(ObjKey, ObjValue, KeySep, ValSep);

	return true;
}


//OBJECT INDSTRINGLIST_GET_VALUE
bool NvSession::ObjIndStringListGetValue(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);
	
	NvObjIndStringList* Object = (NvObjIndStringList*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error
	

	// Get object key that should be in command parameters
	string ObjKey = CommandToExecute->Parameters.GetValue("KEY");
	CleanString(ObjKey);

	if(!ObjKey.length())
	{
		if(CommandToExecute->Parameters.KeyExist("INDEX"))
		{
			int Index = 0;
			string ObjKeyIndex = CommandToExecute->Parameters.GetValue("INDEX");
			CleanString(ObjKeyIndex);
			MakeUpper(ObjKeyIndex);
			if(ObjKeyIndex.length())
				Index = atoi(ObjKeyIndex.c_str());
			if(Index < 1)
				Index = 1;
			// The real index starts at 0
			Index--;
			ObjKey = Object->GetKey(Index);
		}
		if(!ObjKey.length())
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_INDSTRINGLIST_KEY, ObjName);
			return false;
		}
	}
	


	if(!Object->KeyExist(ObjKey))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_INDSTRINGLIST_KEY, ObjName);
		return false;
	}

	GetStringValue(CommandToExecute, Object->GetValue(ObjKey).c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT INDSTRINGLIST_GET_VALUES
bool NvSession::ObjIndStringListGetValues(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);
	
	NvObjIndStringList* Object = (NvObjIndStringList*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error
	
	string KeySep = CommandToExecute->Parameters.GetValue("KEYSEP");
	CleanString(KeySep);
	if(!KeySep.length())
		KeySep = ";";
	string ValSep = CommandToExecute->Parameters.GetValue("VALSEP");
	CleanString(ValSep);
	if(!ValSep.length())
		ValSep = "=";

	// Get output mode
	string Mode = CommandToExecute->Parameters.GetValue("MODE");
	CleanString(Mode, true);
	bool bModeVariables = true;
	if(Mode == "STRING")
		bModeVariables = false;

	// Get eventual variable prefix
	string Prefix = CommandToExecute->Parameters.GetValue("PREFIX");
	CleanString(Prefix, true);

	if(bModeVariables)
	{
		map<string,string>::iterator pos;
		for(pos = Object->m_mList.begin(); pos != Object->m_mList.end(); ++pos)
		{
			string VarName = Prefix;
			VarName += pos->first;
			m_oVariables.SetValue(VarName, pos->second);
		}
	}
	else
	{
		string Value = "";
		map<string,string>::iterator pos;
		int i=0;
		for(i=0,pos = Object->m_mList.begin(); pos != Object->m_mList.end(); ++pos,i++)
		{
			if(i!=0)
				Value += KeySep;
			Value += pos->first;
			Value += ValSep;
			Value += pos->second;
		}
		GetStringValue(CommandToExecute, Value.c_str(), Buffer, BufferSize, DataSize, iEncoding);
	}

	return true;
}


//OBJECT INDSTRINGLIST_GET_SIZE
bool NvSession::ObjIndStringListGetSize(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjIndStringList* Object = (NvObjIndStringList*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	char Buf[50];
	sprintf(Buf, "%d", Object->GetSize());
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT INDSTRINGLIST_REMOVE
bool NvSession::ObjIndStringListRemove(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object key that should be in command parameters
	string ObjKey = CommandToExecute->Parameters.GetValue("KEY");
	CleanString(ObjKey);
//	MakeUpper(ObjKey);
	bool AllList = false;
	if(!ObjKey.length())
		AllList = true;

	NvObjIndStringList* Object = (NvObjIndStringList*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	if(AllList)
	{
		Object->RemoveAll();
		return true;
	}
	if(!Object->KeyExist(ObjKey))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_INDSTRINGLIST_KEY, ObjName);
		return false;
	}

	Object->Remove(ObjKey);

	return true;
}




//OBJECT INDSTRINGLIST_ADD_INDSTRINGLIST
bool NvSession::ObjIndStringListAddIndStringList(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object source and destination names that should be in command parameters
	string SObjName = CommandToExecute->Parameters.GetValue("SNAME");
	CleanString(SObjName);
	string DObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(DObjName);
	
	bool bUseOutputContainer = false;
	string sUseOutputContainer = CommandToExecute->Parameters.GetValue("USE_OUT_CONTAINER");
	CleanString(sUseOutputContainer);
	MakeUpper(sUseOutputContainer);
	if(sUseOutputContainer == "YES")
		bUseOutputContainer = true;

	NvObjIndStringList* SourceObject = (NvObjIndStringList*)GetObjectCheckType(CommandToExecute, InContainer, SObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	NvObjIndStringList* DestinationObject = NULL;
	if(bUseOutputContainer)
		DestinationObject = (NvObjIndStringList*)GetObjectCheckType(CommandToExecute, OutContainer, DObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);
	else
		DestinationObject = (NvObjIndStringList*)GetObjectCheckType(CommandToExecute, InContainer, DObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(DestinationObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	DestinationObject->Add(SourceObject);

	return true;
}






//OBJECT INDSTRINGLIST_KEY_EXIST
bool NvSession::ObjIndStringListKeyExist(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object key that should be in command parameters
	string ObjKey = CommandToExecute->Parameters.GetValue("KEY");
	CleanString(ObjKey);
//	MakeUpper(ObjKey);
	if(!ObjKey.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_INVALID_INDSTRINGLIST_KEY, ObjName);
		return false;
	}

	NvObjIndStringList* Object = (NvObjIndStringList*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	
	char Buf[20];
	strcpy(Buf, "NO");
	if(Object->KeyExist(ObjKey))
		strcpy(Buf, "YES");

	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);
	
	return true;
}




//OBJECT INDSTRINGLIST_GET_KEY
bool NvSession::ObjIndStringListGetKey(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object index that should be in command parameters
	string ObjKeyIndex = CommandToExecute->Parameters.GetValue("INDEX");
	CleanString(ObjKeyIndex);
	MakeUpper(ObjKeyIndex);
	int Index = 0;
	if(ObjKeyIndex.length())
		Index = atoi(ObjKeyIndex.c_str());
	if(Index < 1)
		Index = 1;

	// The real index starts at 0
	Index--;

	NvObjIndStringList* Object = (NvObjIndStringList*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string Key = Object->GetKey(Index);

	GetStringValue(CommandToExecute, Key.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}



//OBJECT TABLE_SET_DESCRIPTION
bool NvSession::ObjTableSetDescription(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string ObjDescription = CommandToExecute->Parameters.GetValue("DESCRIPTION");

	Object->SetDescription(ObjDescription);

	return true;
}



//OBJECT TABLE_GET_DESCRIPTION
bool NvSession::ObjTableGetDescription(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	GetStringValue(CommandToExecute, Object->GetDescription().c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_CLEAR
bool NvSession::ObjTableClear(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->Clear();

	return true;
}




//OBJECT TABLE_CLEAR_DATA
bool NvSession::ObjTableClearData(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->ClearData();

	return true;
}




//OBJECT TABLE_SEARCH
bool NvSession::ObjTableSearch(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object result table that should be in command parameters
	string ObjResult = CommandToExecute->Parameters.GetValue("RESULT");
	CleanString(ObjResult);
	if(!ObjResult.length())
		ObjResult = "RESULT";

	NvObjTable* Result = (NvObjTable *)(OutContainer->GetObject(ObjResult.c_str()));

	if(Result != NULL)
	{
		// The object exists so we remove it
		OutContainer->RemoveObject(ObjResult.c_str());
		Result = NULL;
	}
		
	if(!OutContainer->CreateObject(ObjResult.c_str(), NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjResult);
		return false;
	}
	
	Result = (NvObjTable *)(OutContainer->GetObject(ObjResult.c_str()));
	if(Result == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjResult);
		return false;
	}

	string Query = CommandToExecute->Parameters.GetValue("QUERY");
	CleanString(Query);
	if(!Query.length())
		Query = "*";		// Default query is all

	string Sort = CommandToExecute->Parameters.GetValue("SORT");
	CleanString(Sort);

	int iMaxDocs = -1;
	string sMaxDocs = CommandToExecute->Parameters.GetValue("MAXDOCS");
	CleanString(sMaxDocs);
	if(sMaxDocs.length())
		iMaxDocs = atoi(sMaxDocs.c_str());

	string ColumnsHere = CommandToExecute->Parameters.GetValue("COLUMNS");
	CleanString(ColumnsHere, true);

	bool RowIndexCol = false;
	string sRowIndexCol = CommandToExecute->Parameters.GetValue("WITH_ROW_INDEX");
	CleanString(sRowIndexCol, true);
	MakeUpper(sRowIndexCol);
	if(sRowIndexCol == "YES")
		RowIndexCol = true;

	int iNumDocs = Object->Search(Query, Result, Sort, iMaxDocs, ColumnsHere, RowIndexCol);
	if(iNumDocs == -1)
	{
		char *ErrorMessage = (char *)malloc(Query.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), Query.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_SEARCH_ERROR, ErrorMessage);
		free(ErrorMessage);
	}

	char Buf[50];
	sprintf(Buf, "%d", iNumDocs);
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	if(iNumDocs == -1)
		return false;

	return true;
}




//OBJECT TABLE_SORT
bool NvSession::ObjTableSort(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string Sort = CommandToExecute->Parameters.GetValue("SORT");
	CleanString(Sort);

	Object->Sort(Sort);

	return true;
}




//OBJECT TABLE_CREATE_FROM
bool NvSession::ObjTableCreateFrom(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	bool bUseOutputContainer = false;
	string sUseOutputContainer = CommandToExecute->Parameters.GetValue("USE_OUT_CONTAINER");
	CleanString(sUseOutputContainer);
	MakeUpper(sUseOutputContainer);
	if(sUseOutputContainer == "YES")
		bUseOutputContainer = true;

	NvObjTable* Object = NULL;
	if(bUseOutputContainer)
		Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, OutContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);
	else
		Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object name that should be in command parameters
	string SourceObjName = CommandToExecute->Parameters.GetValue("SNAME");
	CleanString(SourceObjName);
	if(!SourceObjName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_CREATE_FROM:SNAME");
		return false;
	}

	NvObjTable* SourceObject = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, SourceObjName.c_str(), NV_OBJTYPE_TABLE);

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->CreateFrom(SourceObject);	
	
	return true;
}




//OBJECT TABLE_SAVE
bool NvSession::ObjTableSave(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// This command is authorized only from internal
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:OBJECT:TABLE_SAVE");
		return false;
	}

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string FileName = CommandToExecute->Parameters.GetValue("FILENAME");
	SetDirName(FileName);
	if(!FileName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SAVE:FILENAME");
		return false;
	}

	if(!Object->Save(FileName))
	{
		char *ErrorMessage = (char *)malloc(FileName.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), FileName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_SAVE, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	return true;
}




//OBJECT TABLE_LOAD
bool NvSession::ObjTableLoad(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// This command is authorized only from internal
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:OBJECT:TABLE_LOAD");
		return false;
	}

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string FileName = CommandToExecute->Parameters.GetValue("FILENAME");
	SetDirName(FileName);
	if(!FileName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_LOAD:FILENAME");
		return false;
	}

	if(!Object->Load(FileName))
	{
		char *ErrorMessage = (char *)malloc(FileName.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), FileName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_LOAD, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	return true;
}




//OBJECT TABLE_IMPORT
bool NvSession::ObjTableImport(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// This command is authorized only from internal
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:OBJECT:TABLE_IMPORT");
		return false;
	}

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string FileName = CommandToExecute->Parameters.GetValue("FILENAME");
	SetDirName(FileName);
	if(!FileName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_IMPORT:FILENAME");
		return false;
	}

	bool Append = false;
	string sAppend = CommandToExecute->Parameters.GetValue("APPEND");
	CleanString(sAppend);
	MakeUpper(sAppend);
	if(sAppend == "YES")
		Append = true;

	bool WithLf = false;
	string sWithLf = CommandToExecute->Parameters.GetValue("WITH_LF");
	CleanString(sWithLf);
	MakeUpper(sWithLf);
	if(sWithLf == "YES")
		WithLf = true;

	// Get eventual column separator
	string ColSep = CommandToExecute->Parameters.GetValue("COLSEP");
	if(!ColSep.length())
		ColSep = ";";

	// Get eventual line separator
	string LineSep = CommandToExecute->Parameters.GetValue("LINESEP");
	if(!LineSep.length())
		LineSep = "|";

	// Get eventual offset value
	string sOffset = CommandToExecute->Parameters.GetValue("OFFSET");
	int iOffset = 0;
	if(sOffset.length())
		iOffset = atoi(sOffset.c_str());

	// Get eventual skip lines value
	string sSkipLines = CommandToExecute->Parameters.GetValue("SKIPLINES");
	int iSkipLines = 0;
	if(sSkipLines.length())
		iSkipLines = atoi(sSkipLines.c_str());


	// Get eventual line length
	string sMaxLineLength = CommandToExecute->Parameters.GetValue("LINE_LENGTH");
	int iMaxLineLength = 1024;
	if(sMaxLineLength.length())
		iMaxLineLength = atoi(sMaxLineLength.c_str());
	if(iMaxLineLength < 1024)
		iMaxLineLength = 1024;

	if(!Object->Import(FileName, Append, ColSep.c_str(), LineSep.c_str(), iMaxLineLength, iOffset, iSkipLines, WithLf))
	{
		char *ErrorMessage = (char *)malloc(FileName.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), FileName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_IMPORT, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	return true;
}




//OBJECT TABLE_EXPORT
bool NvSession::ObjTableExport(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// This command is authorized only from internal
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:OBJECT:TABLE_EXPORT");
		return false;
	}

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string FileName = CommandToExecute->Parameters.GetValue("FILENAME");
	SetDirName(FileName);
	if(!FileName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_EXPORT:FILENAME");
		return false;
	}


	bool WithLf = false;
	string sWithLf = CommandToExecute->Parameters.GetValue("WITH_LF");
	CleanString(sWithLf);
	MakeUpper(sWithLf);
	if(sWithLf == "YES")
		WithLf = true;

	// Get eventual column separator
	string ColSep = CommandToExecute->Parameters.GetValue("COLSEP");
	if(!ColSep.length())
		ColSep = ";";

	// Get eventual column separator
	string LineSep = CommandToExecute->Parameters.GetValue("LINESEP");
	if(!LineSep.length())
		LineSep = "|";


	// Get eventual line length
	string sMaxLineLength = CommandToExecute->Parameters.GetValue("LINE_LENGTH");
	int iMaxLineLength = 1024;
	if(sMaxLineLength.length())
		iMaxLineLength = atoi(sMaxLineLength.c_str());
	if(iMaxLineLength < 1024)
		iMaxLineLength = 1024;

	if(!Object->Export(FileName, ColSep.c_str(), LineSep.c_str(), iMaxLineLength, WithLf))
	{
		char *ErrorMessage = (char *)malloc(FileName.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), FileName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_EXPORT, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	return true;
}




//OBJECT TABLE_SELECT_FROM
bool NvSession::ObjTableSelectFrom(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	bool bUseOutputContainer = false;
	string sUseOutputContainer = CommandToExecute->Parameters.GetValue("USE_OUT_CONTAINER");
	CleanString(sUseOutputContainer);
	MakeUpper(sUseOutputContainer);
	if(sUseOutputContainer == "YES")
		bUseOutputContainer = true;

	NvObjTable* Object = NULL;
	if(bUseOutputContainer)
		Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, OutContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);
	else
		Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object name that should be in command parameters
	string ObjSourceName = CommandToExecute->Parameters.GetValue("SOURCE");
	CleanString(ObjSourceName);

	NvObjTable* SourceObject = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjSourceName.c_str(), NV_OBJTYPE_TABLE);

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string SelectedYes = "YES";
	if(CommandToExecute->Parameters.KeyExist("SELECTED_YES"))
		SelectedYes = CommandToExecute->Parameters.GetValue("SELECTED_YES");

	string SelectedNo = "NO";
	if(CommandToExecute->Parameters.KeyExist("SELECTED_NO"))
		SelectedNo = CommandToExecute->Parameters.GetValue("SELECTED_NO");

	string sSourceCol = CommandToExecute->Parameters.GetValue("SOURCE_COL");
	CleanString(sSourceCol);
	string sSourceColName = CommandToExecute->Parameters.GetValue("SOURCE_COLNAME");
	CleanString(sSourceColName);
	if((!sSourceCol.length()) && (!sSourceColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SELECT_FROM:SOURCE_COL");
		return false;
	}
	int source_col_index = atoi(sSourceCol.c_str());
	source_col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sSourceColName.length())
		source_col_index = SourceObject->GetColumnIndex(sSourceColName);

	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SELECT_FROM:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);


	string sSelectedCol = CommandToExecute->Parameters.GetValue("SELECTED_COL");
	CleanString(sSelectedCol);
	string sSelectedColName = CommandToExecute->Parameters.GetValue("SELECTED_COLNAME");
	CleanString(sSelectedColName);
	if((!sSelectedCol.length()) && (!sSelectedColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SELECT_FROM:SELECTED_COL");
		return false;
	}
	int selected_col_index = atoi(sSelectedCol.c_str());
	selected_col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sSelectedColName.length())
		selected_col_index = Object->GetColumnIndex(sSelectedColName);
	

	bool bCaseSensitive = true;
	string sCaseSensitive = CommandToExecute->Parameters.GetValue("CASE_SENSITIVE");
	CleanString(sCaseSensitive);
	MakeUpper(sCaseSensitive);
	if(sCaseSensitive == "NO")
		bCaseSensitive = false;


	if(!Object->SelectFrom(SourceObject, source_col_index, col_index, selected_col_index, SelectedYes.c_str(), SelectedNo.c_str(), bCaseSensitive))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_SELECT_FROM, ObjName);
		return false;
	}

	return true;
}




//OBJECT TABLE_JOIN
bool NvSession::ObjTableJoin(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);
	
	bool bUseOutputContainer = false;
	string sUseOutputContainer = CommandToExecute->Parameters.GetValue("USE_OUT_CONTAINER");
	CleanString(sUseOutputContainer);
	MakeUpper(sUseOutputContainer);
	if(sUseOutputContainer == "YES")
		bUseOutputContainer = true;

	NvObjTable* Object = NULL;
	if(bUseOutputContainer)
		Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, OutContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);
	else
		Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object name that should be in command parameters
	string ObjSourceName = CommandToExecute->Parameters.GetValue("SNAME");
	CleanString(ObjSourceName);

	NvObjTable* SourceObject = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjSourceName.c_str(), NV_OBJTYPE_TABLE);

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string sForeign = CommandToExecute->Parameters.GetValue("FOREIGN");
	CleanString(sForeign);
	if(!sForeign.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_JOIN:FOREIGN");
		return false;
	}

	string sColumns = CommandToExecute->Parameters.GetValue("COLUMNS");
	CleanString(sColumns, true);
	
	bool bReplace = false;
	string sReplace = CommandToExecute->Parameters.GetValue("REPLACE");
	CleanString(sReplace);
	MakeUpper(sReplace);
	if(sReplace == "YES")
		bReplace = true;


	if(!Object->Join(SourceObject, sColumns.c_str(), sForeign.c_str(), bReplace))
	{
		string ErrorInfo = ObjName;
		ErrorInfo += ";";
		ErrorInfo += ObjSourceName;
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_JOIN, ErrorInfo);
		return false;
	}

	return true;
}






//OBJECT TABLE_ADD_TABLE
bool NvSession::ObjTableAddTable(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object source and destination names that should be in command parameters
	string SObjName = CommandToExecute->Parameters.GetValue("SNAME");
	CleanString(SObjName);
	string DObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(DObjName);
	
	bool bUseOutputContainer = false;
	string sUseOutputContainer = CommandToExecute->Parameters.GetValue("USE_OUT_CONTAINER");
	CleanString(sUseOutputContainer);
	MakeUpper(sUseOutputContainer);
	if(sUseOutputContainer == "YES")
		bUseOutputContainer = true;

	NvObjTable* SourceObject = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, SObjName.c_str(), NV_OBJTYPE_TABLE);

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	NvObjTable* DestinationObject = NULL;
	if(bUseOutputContainer)
		DestinationObject = (NvObjTable*)GetObjectCheckType(CommandToExecute, OutContainer, DObjName.c_str(), NV_OBJTYPE_TABLE);
	else
		DestinationObject = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, DObjName.c_str(), NV_OBJTYPE_TABLE);

	if(DestinationObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	
	int iMode = ADD_MODE_NORMAL;
	string sMode = CommandToExecute->Parameters.GetValue("MODE");
	CleanString(sMode);
	MakeUpper(sMode);
	if(sMode == "REPLACE")
		iMode = ADD_MODE_REPLACE;
	if(sMode == "NO_REPLACE")
		iMode = ADD_MODE_NOREPLACE;

	if(!DestinationObject->Add(SourceObject, iMode))
	{
		string ErrorInfo = SObjName;
		ErrorInfo += ";";
		ErrorInfo += DObjName;
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_ADD, ErrorInfo);
		return false;
	}

	return true;
}


//OBJECT TABLE_CLEAR_CELL
bool NvSession::ObjTableClearCell(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = CommandToExecute->Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0


	// Get object value that should be in command parameters
	if((row_index == -1) && (CommandToExecute->Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_CLEAR_CELL:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	Object->ClearCell(row_index, col_index);

	return true;
}




//OBJECT TABLE_SORT_CELL
bool NvSession::ObjTableSortCell(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = CommandToExecute->Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if((row_index == -1) && (CommandToExecute->Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SORT_CELL:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);


	bool Ascending = true;
	string sAscending = CommandToExecute->Parameters.GetValue("ASCENDING");
	CleanString(sAscending);
	MakeUpper(sAscending);
	if(sAscending == "NO")
		Ascending = false;

	Object->SortCell(row_index, col_index, Ascending);

	return true;
}




//OBJECT TABLE_GET_CELL_NUM_LINES
bool NvSession::ObjTableGetCellNumLines(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = CommandToExecute->Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if((row_index == -1) && (CommandToExecute->Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_GET_CELL_NUM_LINES:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	int NumCellLines = Object->GetNumCellLines(row_index, col_index);
	if(NumCellLines == -1)
	{
		char *ErrorMessage = (char *)malloc(sRow.length() + sCol.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - row:%s - col:%s", ObjName.c_str(), sRow.c_str(), sCol.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_INVALID_CELL, ErrorMessage);
		free(ErrorMessage);
	}

	char Buf[50];
	sprintf(Buf, "%d", NumCellLines);
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	if(NumCellLines == -1)
		return false;

	return true;
}




//OBJECT TABLE_GET_CELL_LINE
bool NvSession::ObjTableGetCellLine(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = CommandToExecute->Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0


	// Get object value that should be in command parameters
	if((row_index == -1) && (CommandToExecute->Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_GET_CELL_LINE:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	// Get object value that should be in command parameters
	int line_index = 1;
	string sLineIndex = CommandToExecute->Parameters.GetValue("LINE_INDEX");
	CleanString(sLineIndex);
	if(sLineIndex.length())
		line_index = atoi(sLineIndex.c_str());
	line_index--;	// The real index starts at 0

	// Get object name that should be in command parameters
	string sDefault = CommandToExecute->Parameters.GetValue("DEFAULT");

	GetStringValue(CommandToExecute, Object->GetCellLine(row_index, col_index, line_index, sDefault).c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_SET_CELL_LINE
bool NvSession::ObjTableSetCellLine(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = CommandToExecute->Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if((row_index == -1) && (CommandToExecute->Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SET_CELL_LINE:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	// Get object value that should be in command parameters
	int line_index = 1;
	string sLineIndex = CommandToExecute->Parameters.GetValue("LINE_INDEX");
	CleanString(sLineIndex);
	if(sLineIndex.length())
		line_index = atoi(sLineIndex.c_str());
	line_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	string Line = CommandToExecute->Parameters.GetValue("LINE");

	if(!Object->SetCellLine(row_index, col_index, Line, line_index))
	{
		char *ErrorMessage = (char *)malloc(sRow.length() + sCol.length() + sLineIndex.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - row:%s - col:%s - line:%s", ObjName.c_str(), sRow.c_str(), sCol.c_str(), sLineIndex.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_SET_CELL_LINE, ErrorMessage);
		free(ErrorMessage);
		return false;
	}
	
	return true;
}



//OBJECT TABLE_GET_CELL
bool NvSession::ObjTableGetCell(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = CommandToExecute->Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0


	// Get object value that should be in command parameters
	if((row_index == -1) && (CommandToExecute->Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_GET_CELL:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);


	// Get eventual line separator
	string LineSep = CommandToExecute->Parameters.GetValue("LINESEP");
	if(!LineSep.length())
		LineSep = ";";

	int i;
	int NumCells = Object->GetNumCellLines(row_index, col_index);
	string Value = "";
	for(i=0;i<NumCells;i++)
	{
		if(i!=0)
			Value += LineSep;
		Value += Object->GetCellLine(row_index, col_index, i);
	}

	GetStringValue(CommandToExecute, Value.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}



//OBJECT TABLE_SET_CELL
bool NvSession::ObjTableSetCell(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = CommandToExecute->Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if((row_index == -1) && (CommandToExecute->Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SET_CELL:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);


	// Get eventual line separator
	string LineSep = CommandToExecute->Parameters.GetValue("LINESEP");
	if(!LineSep.length())
		LineSep = ";";

	// Get data
	string data = CommandToExecute->Parameters.GetValue("DATA");
	
	int LineSeparatorLength = LineSep.length();
	string::size_type index;


	bool bPrimaryColumn = false;
	string PrimaryColumnName;
	bool PrimaryCaseSensitive;
	sColName = Object->GetColumnName(col_index);
	Object->GetPrimaryInfo(PrimaryColumnName, PrimaryCaseSensitive);

	if(PrimaryColumnName == sColName)
		bPrimaryColumn = true;

	string line = "";
	Object->ClearCell(row_index, col_index);
	while(1)
	{
		if((index = data.find(LineSep)) == string::npos)
		{
			if(data.length())
				Object->InsertCellLine(row_index, col_index, data);
			break;
		}

		line = data.substr(0, index);
		data = data.substr(index+LineSeparatorLength);
		if(line.length())
		{
			Object->InsertCellLine(row_index, col_index, line);
		}
		if(bPrimaryColumn)	// only one line allowed for a primary colomn
			break;
	}

	return true;
}




//OBJECT TABLE_INSERT_CELL_LINE
bool NvSession::ObjTableInsertCellLine(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = CommandToExecute->Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if((row_index == -1) && (CommandToExecute->Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_INSERT_CELL_LINE:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	// Get object value that should be in command parameters
	int line_index = 0;
	string sLineIndex = CommandToExecute->Parameters.GetValue("LINE_INDEX");
	CleanString(sLineIndex);
	if(sLineIndex.length())
		line_index = atoi(sLineIndex.c_str());
	line_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	string Line = CommandToExecute->Parameters.GetValue("LINE");

	if(!Object->InsertCellLine(row_index, col_index, Line, line_index))
	{
		char *ErrorMessage = (char *)malloc(sRow.length() + sCol.length() + sLineIndex.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - row:%s - col:%s - line:%s", ObjName.c_str(), sRow.c_str(), sCol.c_str(), sLineIndex.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_INSERT_CELL_LINE, ErrorMessage);
		free(ErrorMessage);
		return false;
	}
	
	return true;
}




//OBJECT TABLE_REMOVE_CELL_LINE
bool NvSession::ObjTableRemoveCellLine(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = CommandToExecute->Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if((row_index == -1) && (CommandToExecute->Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_REMOVE_CELL_LINE:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	// Get object value that should be in command parameters
	int line_index = 1;
	string sLineIndex = CommandToExecute->Parameters.GetValue("LINE_INDEX");
	CleanString(sLineIndex);
	if(sLineIndex.length())
		line_index = atoi(sLineIndex.c_str());
	line_index--;	// The real index starts at 0

	Object->RemoveCellLine(row_index, col_index, line_index);
	
	return true;
}




//OBJECT TABLE_SWAP_CELL_LINES
bool NvSession::ObjTableSwapCellLines(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = CommandToExecute->Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if((row_index == -1) && (CommandToExecute->Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SWAP_CELL_LINES:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	// Get object value that should be in command parameters
	string sLineIndex1 = CommandToExecute->Parameters.GetValue("LINE_INDEX1");
	CleanString(sLineIndex1);
	if(!sLineIndex1.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SWAP_CELL_LINES:LINE_INDEX1");
		return false;
	}
	int line_index1 = atoi(sLineIndex1.c_str());
	line_index1--;	// The real index starts at 0

	// Get object value that should be in command parameters
	string sLineIndex2 = CommandToExecute->Parameters.GetValue("LINE_INDEX2");
	CleanString(sLineIndex2);
	if(!sLineIndex2.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SWAP_CELL_LINES:LINE_INDEX2");
		return false;
	}
	int line_index2 = atoi(sLineIndex2.c_str());
	line_index2--;	// The real index starts at 0

	Object->SwapCellLines(row_index, col_index, line_index1, line_index2);
	
	return true;
}




//OBJECT TABLE_GET_NUM_ROWS
bool NvSession::ObjTableGetNumRows(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	int NumRows = Object->GetNumRow();

	char Buf[50];
	sprintf(Buf, "%d", NumRows);
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_REMOVE_ROW
bool NvSession::ObjTableRemoveRow(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = CommandToExecute->Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if((row_index == -1) && (CommandToExecute->Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	Object->RemoveRow(row_index);
	
	return true;
}




//OBJECT TABLE_REMOVE_ROWS
bool NvSession::ObjTableRemoveRows(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string Query = CommandToExecute->Parameters.GetValue("QUERY");
	CleanString(Query);
	if(!Query.length())
		Query = "*";		// Default query is all

	if(!Object->RemoveRows(Query))
	{
		char *ErrorMessage = (char *)malloc(Query.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), Query.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_REMOVE_ROWS_ERROR, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	return true;
}




//OBJECT TABLE_CLEAR_ROW
bool NvSession::ObjTableClearRow(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = CommandToExecute->Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if((row_index == -1) && (CommandToExecute->Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	Object->RemoveRow(row_index);
	
	return true;
}




//OBJECT TABLE_CLEAR_ROWS
bool NvSession::ObjTableClearRows(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string Query = CommandToExecute->Parameters.GetValue("QUERY");
	CleanString(Query);
	if(!Query.length())
		Query = "*";		// Default query is all

	if(!Object->ClearRows(Query))
	{
		char *ErrorMessage = (char *)malloc(Query.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), Query.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CLEAR_ROWS_ERROR, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	return true;
}




//OBJECT TABLE_INSERT_ROWS
bool NvSession::ObjTableInsertRows(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = CommandToExecute->Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if((row_index == -1) && (CommandToExecute->Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	int num_row = 1;
	string sNumRow = CommandToExecute->Parameters.GetValue("NUMROW");
	CleanString(sNumRow);
	if(sNumRow.length())
		num_row = atoi(sNumRow.c_str());

	Object->InsertRows(num_row, row_index);
	
	return true;
}




//OBJECT TABLE_SWAP_ROWS
bool NvSession::ObjTableSwapRows(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sRow1 = CommandToExecute->Parameters.GetValue("ROW1");
	CleanString(sRow1);
	if(!sRow1.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SWAP_ROWS:ROW1");
		return false;
	}
	int row_index1 = atoi(sRow1.c_str());
	row_index1--;	// The real index starts at 0

	// Get object value that should be in command parameters
	string sRow2 = CommandToExecute->Parameters.GetValue("ROW2");
	CleanString(sRow2);
	if(!sRow2.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SWAP_ROWS:ROW2");
		return false;
	}
	int row_index2 = atoi(sRow2.c_str());
	row_index2--;	// The real index starts at 0

	Object->SwapRows(row_index1, row_index2);

	return true;
}


//OBJECT TABLE_GET_ROWS
bool NvSession::ObjTableGetRows(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object result table that should be in command parameters
	string ObjResult = CommandToExecute->Parameters.GetValue("RESULT");
	CleanString(ObjResult);
	if(!ObjResult.length())
		ObjResult = "RESULT";

	NvObjTable* Result = (NvObjTable *)(OutContainer->GetObject(ObjResult.c_str()));

	if(Result != NULL)
	{
		// The object exists so we remove it
		OutContainer->RemoveObject(ObjResult.c_str());
		Result = NULL;
	}
		
	if(!OutContainer->CreateObject(ObjResult.c_str(), NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjResult);
		return false;
	}
	
	Result = (NvObjTable *)(OutContainer->GetObject(ObjResult.c_str()));
	if(Result == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjResult);
		return false;
	}

	string First = CommandToExecute->Parameters.GetValue("FIRST");
	CleanString(First);
	if(!First.length())
		First = "1";		// Default to first row
	int iFirst = atoi(First.c_str());
	iFirst--; // starts in fact at 0
	if(iFirst <= 0)
		iFirst = 0;

	string Last = CommandToExecute->Parameters.GetValue("LAST");
	CleanString(Last);
	if(!Last.length())
		Last = "0";		// Default to last row
	int iLast = atoi(Last.c_str());
	iLast--; // starts in fact at 0

	string Page = CommandToExecute->Parameters.GetValue("PAGE");
	CleanString(Page);
	if(Page.length())
	{
		int iPage = atoi(Page.c_str());
		if(iPage < 1)
			iPage = 1;
		string PageSize = CommandToExecute->Parameters.GetValue("PAGE_SIZE");
		int iPageSize = 10;
		CleanString(PageSize);
		if(PageSize.length())
			iPageSize = atoi(PageSize.c_str());
		if(iPageSize <= 0)
			First = -1;
		else
		{
			int NumRows = Object->GetNumRow();
			iFirst = (iPage-1)*iPageSize;
			iLast = iFirst + iPageSize-1;
		}
	}
	
	string ColumnsHere = CommandToExecute->Parameters.GetValue("COLUMNS");
	CleanString(ColumnsHere, true);
	int iNumDocs = Object->GetRows(iFirst, iLast, Result, ColumnsHere.c_str());

	char Buf[50];
	sprintf(Buf, "%d", iNumDocs);
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_GET_ROW
bool NvSession::ObjTableGetRow(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = CommandToExecute->Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if((row_index == -1) && (CommandToExecute->Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}


	// Get eventual column separator
	string ColSep = CommandToExecute->Parameters.GetValue("COLSEP");
	if(!ColSep.length())
		ColSep = ";";


	// Get output mode
	string Mode = CommandToExecute->Parameters.GetValue("MODE");
	CleanString(Mode, true);
	bool bModeVariables = true;
	if(Mode == "STRING")
		bModeVariables = false;

	// Get eventual line separator
	string LineSep = CommandToExecute->Parameters.GetValue("LINESEP");
	if(!LineSep.length())
	{
		if(bModeVariables)
			LineSep = ";";
		else
			LineSep = "|";
	}


	// Get eventual variable prefix
	string Prefix = CommandToExecute->Parameters.GetValue("PREFIX");
	CleanString(Prefix, true);


	int NumColumns = Object->GetNumColumn();
	int i;
	if(bModeVariables)
	{
		for(i=0;i<NumColumns;i++)
		{
			string ColumnName = Prefix;
			ColumnName += Object->GetColumnName(i);
			m_oVariables.SetValue(ColumnName, "");
		}
		int NumRows = Object->GetNumRow();
		if((row_index+1) > NumRows)
			return true;

		for(i=0;i<NumColumns;i++)
		{
			string ColumnName = Prefix;
			ColumnName += Object->GetColumnName(i);
			int NumCells = Object->GetNumCellLines(row_index, i);
			int j;
			string Value = "";
			for(j=0;j<NumCells;j++)
			{
				if(j!=0)
					Value += LineSep;
				Value += Object->GetCellLine(row_index, i, j);
			}
			m_oVariables.SetValue(ColumnName, Value);
		}
	}
	else
	{
		int NumRows = Object->GetNumRow();
		if((row_index+1) > NumRows)
			return true;

		string Value = "";
		for(i=0;i<NumColumns;i++)
		{
			if(i!=0)
				Value += ColSep;
			int NumCells = Object->GetNumCellLines(row_index, i);
			int j;
			for(j=0;j<NumCells;j++)
			{
				if(j!=0)
					Value += LineSep;
				Value += Object->GetCellLine(row_index, i, j);
			}
		}
		GetStringValue(CommandToExecute, Value.c_str(), Buffer, BufferSize, DataSize, iEncoding);
	}

	return true;
}




//OBJECT TABLE_GET_COLUMN
bool NvSession::ObjTableGetColumn(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_GET_COLUMN:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);


	// Get eventual row separator
	string RowSep = CommandToExecute->Parameters.GetValue("ROWSEP");
	// there was a bug in first release of this command using COLSEP instead of ROWSEP so we must assume ascending compatibility
	if(!RowSep.length())
		RowSep = CommandToExecute->Parameters.GetValue("COLSEP");
	if(!RowSep.length())
		RowSep = ";";

	// Get eventual line separator
	string LineSep = CommandToExecute->Parameters.GetValue("LINESEP");
	if(!LineSep.length())
		LineSep = "|";

	int i;
	int NumRows = Object->GetNumRow();
	int LengthRowSep = RowSep.length();
	int LengthLineSep = LineSep.length();


	// first count size of result
	int TotalSize = 0;
	for(i=0;i<NumRows;i++)
	{
		if(i!=0)
			TotalSize += LengthRowSep;
		int NumCells = Object->GetNumCellLines(i, col_index);
		int j;
		for(j=0;j<NumCells;j++)
		{
			if(j!=0)
				TotalSize += LengthLineSep;
			TotalSize += Object->GetCellLine(i, col_index, j).length();
		}
	}


	string Value = "";
	Value.reserve(TotalSize + 10);
	for(i=0;i<NumRows;i++)
	{
		if(i!=0)
			Value += RowSep;
		int NumCells = Object->GetNumCellLines(i, col_index);
		int j;
		for(j=0;j<NumCells;j++)
		{
			if(j!=0)
				Value += LineSep;
			Value += Object->GetCellLine(i, col_index, j);
		}
	}
	GetStringValue(CommandToExecute, Value.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_SET_COLUMN
bool NvSession::ObjTableSetColumn(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SET_COLUMN:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);


	// Get eventual row separator
	string RowSep = CommandToExecute->Parameters.GetValue("ROWSEP");
	// there was a bug in first release of this command using COLSEP instead of ROWSEP so we must assume ascending compatibility
	if(!RowSep.length())
		RowSep = CommandToExecute->Parameters.GetValue("COLSEP");
	if(!RowSep.length())
		RowSep = ";";

	// Get eventual line separator
	string LineSep = CommandToExecute->Parameters.GetValue("LINESEP");
	if(!LineSep.length())
		LineSep = "|";

	// Get data
	string data = CommandToExecute->Parameters.GetValue("DATA");
	
	bool bPrimaryColumn = false;
	string PrimaryColumnName;
	bool PrimaryCaseSensitive;
	sColName = Object->GetColumnName(col_index);
	Object->GetPrimaryInfo(PrimaryColumnName, PrimaryCaseSensitive);
	if(PrimaryColumnName == sColName)
	{
		bPrimaryColumn = true;
		Object->SetPrimaryColumn("", false);
	}

	int i;
	int RowSeparatorLength = RowSep.length();
	int LineSeparatorLength = LineSep.length();
	string::size_type index;
	int NumRows = Object->GetNumRow();
	string cell_value = "";
	string line = "";
	for(i=0;i<NumRows;i++)
	{
		Object->ClearCell(i, col_index);
		index = data.find(RowSep);
		if(index == string::npos)
		{
			cell_value = data;
			data = "";
		}
		else
		{
			cell_value = data.substr(0, index);
			data = data.substr(index+RowSeparatorLength);
		}

		while(1)
		{
			if((index = cell_value.find(LineSep)) == string::npos)
			{
				if(cell_value.length())
					Object->InsertCellLine(i, col_index, cell_value);
				break;
			}

			line = cell_value.substr(0, index);
			cell_value = cell_value.substr(index+LineSeparatorLength);
			if(line.length())
			{
				Object->InsertCellLine(i, col_index, line);
			}
			if(bPrimaryColumn)	// only one line allowed for a primary colomn
				break;
		}
	}

	if(bPrimaryColumn)
	{
		if(!Object->SetPrimaryColumn(sColName, PrimaryCaseSensitive))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_SET_PRIMARY, PrimaryColumnName);
			return false;
		}
	}

	return true;
}


//OBJECT TABLE_GET_NUM_COLUMNS
bool NvSession::ObjTableGetNumColums(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	int NumCols = Object->GetNumColumn();

	char Buf[50];
	sprintf(Buf, "%d", NumCols);
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_GET_COLUMN_NAME
bool NvSession::ObjTableGetColumnName(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	if(!sCol.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_GET_COLUMN_NAME:COL");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	GetStringValue(CommandToExecute, Object->GetColumnName(col_index).c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_GET_COLUMN_DESCRIPTION
bool NvSession::ObjTableGetColumnDescription(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_GET_COLUMN_DESCRIPTION:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	GetStringValue(CommandToExecute, Object->GetColumnDesc(col_index).c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_IS_COLUMN_NUMERIC
bool NvSession::ObjTableIsColumnNumeric(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_IS_COLUMN_NUMERIC:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	char Buf[20];
	strcpy(Buf, "NO");
	if(Object->IsNumericColumn(col_index))
		strcpy(Buf, "YES");

	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_GET_COLUMN_INDEX
bool NvSession::ObjTableGetColumnIndex(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if(!sColName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_GET_COLUMN_INDEX:COLNAME");
		return false;
	}

	int ColIndex = Object->GetColumnIndex(sColName);
	if(ColIndex == -1)
	{
		char *ErrorMessage = (char *)malloc(sColName.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), sColName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_BAD_COLUMN_NAME, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	// The real ColumnIndex starts at 0
	ColIndex++;
	char Buf[50];
	sprintf(Buf, "%d", ColIndex);
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_CLEAR_COLUMN
bool NvSession::ObjTableClearColumn(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_CLEAR_COLUMN:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);


	Object->ClearColumn(col_index);

	return true;
}




//OBJECT TABLE_REMOVE_COLUMN
bool NvSession::ObjTableRemoveColumn(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_REMOVE_COLUMN:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);


	Object->RemoveColumn(col_index);

	return true;
}





//OBJECT TABLE_REMOVE_COLUMNS
bool NvSession::ObjTableRemoveColumns(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string ColumnsHere = CommandToExecute->Parameters.GetValue("COLUMNS");
	CleanString(ColumnsHere, true);
	if(ColumnsHere.length())
	{

		bool bKeep = false;
		string sKeep = CommandToExecute->Parameters.GetValue("KEEP");
		CleanString(sKeep);
		MakeUpper(sKeep);
		if(sKeep == "YES")
			bKeep = true;

		// We must get the column name to keep (so the ones to remove)
		NvObjIndStringList ColumnsToRemove;
		string::size_type index;

		int i;
		int colindex;
		int NumColumnDst = Object->GetNumColumn();
		while(ColumnsHere.length())
		{
			string Column = ColumnsHere;
			index = Column.find(';');
			if(index != string::npos)
			{
				Column = ColumnsHere.substr(0, index);
				ColumnsHere = ColumnsHere.substr(index+1);
			}
			else
				ColumnsHere = "";
			CleanString(Column);
			MakeUpper(Column);
			if(!Column.length())
				continue;
			colindex = Object->GetColumnIndex(Column);
			if(colindex == -1)
				continue;	// the source column doesn't exist
			ColumnsToRemove.SetValue(Column, "");
		}
		vector<string> columns_to_remove;
		for(i=0; i<NumColumnDst; i++)
		{
			string ColName = Object->GetColumnName(i);
			MakeUpper(ColName);
			if(bKeep)
			{
				if(!ColumnsToRemove.KeyExist(ColName))
					columns_to_remove.push_back(ColName);
			}
			else
			{
				if(ColumnsToRemove.KeyExist(ColName))
					columns_to_remove.push_back(ColName);
			}
		}
		for(i=0; i<(int)(columns_to_remove.size()); i++)
		{
			colindex = Object->GetColumnIndex(columns_to_remove[i]);
			if(colindex != -1)
				Object->RemoveColumn(colindex);
		}
	}
	return true;
}




//OBJECT TABLE_INSERT_COLUMN
bool NvSession::ObjTableInsertColumn(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if(!sColName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_INSERT_COLUMN:COLNAME");
		return false;
	}

	// Get object value that should be in command parameters
	string sColDesc = CommandToExecute->Parameters.GetValue("DESCRIPTION");

	bool bNumeric = false;
	string sNumeric = CommandToExecute->Parameters.GetValue("NUMERIC");
	CleanString(sNumeric);
	MakeUpper(sNumeric);
	if(sNumeric == "YES")
		bNumeric = true;

	
	// Get object value that should be in command parameters
	int col_index = 0;
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	if(sCol.length())
		col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	Object->InsertColumn(sColName, sColDesc, bNumeric, col_index);


	bool bPrimary = false;
	string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
	CleanString(sPrimary);
	MakeUpper(sPrimary);
	if(sPrimary == "YES")
		bPrimary = true;

	if(bPrimary)
	{
		bool bPrimaryCs = false;
		string sPrimaryCs = CommandToExecute->Parameters.GetValue("PRIMARY_CS");
		CleanString(sPrimaryCs);
		MakeUpper(sPrimaryCs);
		if(sPrimaryCs == "YES")
			bPrimaryCs = true;

		Object->SetPrimaryColumn(sColName, bPrimaryCs);
	}


	return true;
}






//OBJECT TABLE_ADD_COLUMNS
bool NvSession::ObjTableAddColumns(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string ColumnsHere = CommandToExecute->Parameters.GetValue("COLUMNS");
	CleanString(ColumnsHere);
	if(ColumnsHere.length())
	{
		string::size_type index;
		while(ColumnsHere.length())
		{
			string Column = ColumnsHere;
			string ColumnDesc = "";
			bool ColumnNumeric = false;
			index = Column.find(';');
			if(index != string::npos)
			{
				Column = ColumnsHere.substr(0, index);
				ColumnsHere = ColumnsHere.substr(index+1);
			}
			else
				ColumnsHere = "";
			index = Column.find(':');
			if(index != string::npos)
			{
				ColumnDesc = Column.substr(index+1);
				Column = Column.substr(0, index);
			}
			index = ColumnDesc.find(':');
			if(index != string::npos)
			{
				string sColumnNumeric = ColumnDesc.substr(index+1);
				ColumnDesc = ColumnDesc.substr(0, index);
				CleanString(sColumnNumeric);
				MakeUpper(sColumnNumeric);
				if(sColumnNumeric == "Y")
					ColumnNumeric = true;
			}
			CleanString(Column, true);
			CleanString(ColumnDesc);
			if(!Column.length())
				continue;
			Object->InsertColumn(Column, ColumnDesc, ColumnNumeric);
		}
	}

	return true;
}




//OBJECT TABLE_MODIFY_COLUMN
bool NvSession::ObjTableModifyColumn(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_MODIFY_COLUMN:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	string Query = CommandToExecute->Parameters.GetValue("QUERY");
	CleanString(Query);
	if(!Query.length())
		Query = "*";		// Default query is all


	// Get object value that should be in command parameters
	string Value = CommandToExecute->Parameters.GetValue("VALUE");

	if(!Object->ModifyColumn(Query, col_index, Value))
	{
		char *ErrorMessage = (char *)malloc(Query.length() + sCol.length()  + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - col:%s - %s", ObjName.c_str(), sCol.c_str(), Query.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_MODIFY_COLUMNS_ERROR, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	return true;
}




//OBJECT TABLE_SWAP_COLUMNS
bool NvSession::ObjTableSwapColumns(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol1 = CommandToExecute->Parameters.GetValue("COL1");
	CleanString(sCol1);
	if(!sCol1.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SWAP_COLUMNS:COL1");
		return false;
	}
	int col_index1 = atoi(sCol1.c_str());
	col_index1--;	// The real index starts at 0

	// Get object value that should be in command parameters
	string sCol2 = CommandToExecute->Parameters.GetValue("COL2");
	CleanString(sCol2);
	if(!sCol2.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SWAP_COLUMNS:COL2");
		return false;
	}
	int col_index2 = atoi(sCol2.c_str());
	col_index2--;	// The real index starts at 0

	Object->SwapColumns(col_index1, col_index2);

	return true;
}





//OBJECT TABLE_FILTER_COLOMNS
bool NvSession::ObjTableFilterColumns(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string ColumnsHere = CommandToExecute->Parameters.GetValue("COLUMNS");
	CleanString(ColumnsHere, true);

	Object->FilterColumns(ColumnsHere.c_str());

	return true;
}




//OBJECT TABLE_SET_PRIMARY_COLUMN
bool NvSession::ObjTableSetPrimaryColumn(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);

	bool bPrimaryCs = false;
	string sPrimaryCs = CommandToExecute->Parameters.GetValue("PRIMARY_CS");
	CleanString(sPrimaryCs);
	MakeUpper(sPrimaryCs);
	if(sPrimaryCs == "YES")
		bPrimaryCs = true;

	if(!Object->SetPrimaryColumn(sColName, bPrimaryCs))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_SET_PRIMARY, sColName);
		return false;
	}


	return true;
}



//OBJECT TABLE_SET_COLUMN_NAME
bool NvSession::ObjTableSetColumnName(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SET_COLUMN_NAME:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	string sNewName = CommandToExecute->Parameters.GetValue("NEW_NAME");
	CleanString(sNewName);
	if(!sNewName.length())
		return true; // nothing to do

	if(col_index < 0)
		return true; // nothing to do

	if(Object->GetColumnIndex(sNewName) >= 0)
		return true; // nothing to do because the column ever exists

	Object->SetColumnName(col_index, sNewName);

	return true;
}



//OBJECT TABLE_SET_COLUMN_TYPE
bool NvSession::ObjTableSetColumnType(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SET_COLUMN_TYPE:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	if(col_index < 0)
		return true; // nothing to do

	bool bNumeric = false;
	string sNewType = CommandToExecute->Parameters.GetValue("TYPE");
	CleanString(sNewType);
	MakeUpper(sNewType);
	if(sNewType == "NUMERIC")
		bNumeric = true;

	Object->SetColumnNumeric(col_index, bNumeric);

	return true;
}



//OBJECT TABLE_SET_COLUMN_DESCRIPTION
bool NvSession::ObjTableSetColumnDescription(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	// Get object value that should be in command parameters
	string sCol = CommandToExecute->Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = CommandToExecute->Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_SET_COLUMN_DESCRIPTION:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	if(col_index < 0)
		return true; // nothing to do

	string sNewDesc = CommandToExecute->Parameters.GetValue("DESCRIPTION");

	Object->SetColumnDesc(col_index, sNewDesc);

	return true;
}


//OBJECT TABLE_GET_PRIMARY_INFO
bool NvSession::ObjTableGetPrimaryInfo(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	OutContainer->RemoveObject("PRIMARY_COLUMN");
	OutContainer->RemoveObject("PRIMARY_COLUMN_CS");
	if(!OutContainer->CreateObject("PRIMARY_COLUMN", NV_OBJTYPE_STRING))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "PRIMARY_COLUMN");
		return false;
	}
	if(!OutContainer->CreateObject("PRIMARY_COLUMN_CS", NV_OBJTYPE_BOOLEAN))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "PRIMARY_COLUMN_CS");
		OutContainer->RemoveObject("PRIMARY_COLUMN");
		return false;
	}

	string sPrimary;
	bool bPrimaryCs;
	Object->GetPrimaryInfo(sPrimary, bPrimaryCs);
	
	NvObj *ObjectResult = OutContainer->GetObject("PRIMARY_COLUMN_CS");
	((NvObjBoolean *)ObjectResult)->SetValue(bPrimaryCs);

	ObjectResult = OutContainer->GetObject("PRIMARY_COLUMN");
	((NvObjString *)ObjectResult)->SetValue(sPrimary);

	return true;
}





//OBJECT TABLE_GET_ROW_INDEX
bool NvSession::ObjTableGetRowIndex(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
	CleanString(sPrimary);
	if(!sPrimary.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:TABLE_GET_ROW_INDEX:PRIMARY");
		return false;
	}

	int RowIndex = Object->GetRowIndex(sPrimary);

	// The real RowIndex starts at 0
	RowIndex++;
	char Buf[50];
	sprintf(Buf, "%d", RowIndex);
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}



//OBJECT TABLE_SET_ROW
bool NvSession::ObjTableSetRow(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sData = CommandToExecute->Parameters.GetValue("DATA");

	// Get eventual column separator
	string ColSep = CommandToExecute->Parameters.GetValue("COLSEP");
	if(!ColSep.length())
		ColSep = ";";

	// Get eventual line separator
	string LineSep = CommandToExecute->Parameters.GetValue("LINESEP");
	if(!LineSep.length())
		LineSep = "|";


	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = CommandToExecute->Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if((row_index == -1) && (CommandToExecute->Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = CommandToExecute->Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	if(!Object->SetRowData(row_index, sData.c_str(), ColSep.c_str(), LineSep.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_SET_ROW_DATA, ObjName);
		return false;
	}

	return true;
}




//OBJECT TABLE_ADD_ROWS
bool NvSession::ObjTableAddRows(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sData = CommandToExecute->Parameters.GetValue("DATA");

	// Get eventual record separator
	string RecSep = CommandToExecute->Parameters.GetValue("ROWSEP");
	if(!RecSep.length())
		RecSep = "\n";

	// Get eventual column separator
	string ColSep = CommandToExecute->Parameters.GetValue("COLSEP");
	if(!ColSep.length())
		ColSep = ";";

	// Get eventual line separator
	string LineSep = CommandToExecute->Parameters.GetValue("LINESEP");
	if(!LineSep.length())
		LineSep = "|";

	if(!Object->AddRecords(sData.c_str(), RecSep.c_str(), ColSep.c_str(), LineSep.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_TABLE_CANNOT_ADD_RECORDS, ObjName);
		return false;
	}

	return true;
}



//OBJECT FILE_SET_FILENAME
bool NvSession::ObjFileSetFileName(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// This command is authorized only from internal
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:OBJECT:FILE_SET_FILENAME");
		return false;
	}

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string LocalFileInCommand = CommandToExecute->Parameters.GetValue("FILENAME");
	SetDirName(LocalFileInCommand);

	if(LocalFileInCommand.length())
	{
		string sWorkDir = "";
		sWorkDir = Application->GetWorkDir();
		if(Object->m_iPersistTime == -1)
		{
			if(CompareWorkDir(LocalFileInCommand, sWorkDir))
			{
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_SET_PERSISTANT, LocalFileInCommand);
				return false;
			}
		}
		Object->SetFileName(LocalFileInCommand);
	}
	else
	{
		// If no file name given
		// We try to create automatically a file following some information in the command
		string LocalFileDirInCommand = CommandToExecute->Parameters.GetValue("DIRECTORY");
		SetDirName(LocalFileDirInCommand);
		if(!LocalFileDirInCommand.length())
		{
			if(Object->m_iPersistTime == -1)
				LocalFileDirInCommand = Application->GetFileDir();
			else
				LocalFileDirInCommand = Application->GetWorkDir();
		}

		string sWorkDir = "";
		sWorkDir = Application->GetWorkDir();
		CleanString(LocalFileDirInCommand);
		string LocalFileExtensionInCommand = CommandToExecute->Parameters.GetValue("EXTENSION");
		CleanString(LocalFileExtensionInCommand);
		string LocalFilePrefixInCommand = CommandToExecute->Parameters.GetValue("PREFIX");
		CleanString(LocalFilePrefixInCommand);
		string LocalFileSuffixInCommand = CommandToExecute->Parameters.GetValue("SUFFIX");
		CleanString(LocalFileSuffixInCommand);
		if(Object->m_iPersistTime == -1)
		{
			if(CompareWorkDir(LocalFileDirInCommand, sWorkDir))
			{
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_SET_PERSISTANT, ObjName);
				return false;
			}
		}
		if(!Object->SetAutoFileName(LocalFileDirInCommand, LocalFileExtensionInCommand, LocalFilePrefixInCommand, LocalFileSuffixInCommand))
		{
			char *ErrorMessage = (char *)malloc(ObjName.length() + LocalFileDirInCommand.length() + LocalFileExtensionInCommand.length()  + LocalFilePrefixInCommand.length()  + LocalFileSuffixInCommand.length() + 100);
			sprintf(ErrorMessage, "obj:%s - dir:%s - ext:%s - prefix:%s - suffix:%s", ObjName.c_str(), LocalFileDirInCommand.c_str(), LocalFileExtensionInCommand.c_str(), LocalFilePrefixInCommand.c_str(), LocalFileSuffixInCommand.c_str());
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_CREATE, ErrorMessage);
			free(ErrorMessage);
			return false;
		}
	}
	

	return true;
}





//OBJECT FILE_SET_PERSIST
bool NvSession::ObjFileSetPersist(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string PersistTimeFileInCommand = CommandToExecute->Parameters.GetValue("PERSIST");
	CleanString(PersistTimeFileInCommand);
	MakeUpper(PersistTimeFileInCommand);
	int iPersistTimeFileInCommand = 0;	// The file is not persistant by default on the server
	if(PersistTimeFileInCommand.length())
		iPersistTimeFileInCommand = atoi(PersistTimeFileInCommand.c_str());
	if(iPersistTimeFileInCommand < 0)
		iPersistTimeFileInCommand = -1;

	// If the file has to be persistant
	// we must check that it's not in the application work dir
	if(iPersistTimeFileInCommand == -1)
	{
		// We get application work dir
		string sWorkDir = "";
		sWorkDir = Application->GetWorkDir();

		RemoveDirSuffix(sWorkDir);
		if(Object->GetFileDir() == sWorkDir)
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_SET_PERSISTANT, ObjName);
			return false;
		}
	}
	Object->SetPersistTime(iPersistTimeFileInCommand);
	
	return true;
}



//OBJECT FILE_GET_PATHNAME
bool NvSession::ObjFileGetPathName(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{


	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	GetStringValue(CommandToExecute, Object->GetFilePathName().c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT FILE_GET_DIRNAME
bool NvSession::ObjFileGetDirName(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	GetStringValue(CommandToExecute, Object->GetFileDir().c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT FILE_GET_FILENAME
bool NvSession::ObjFileGetFileName(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get with extension option
	bool bWithExt = true;
	string sWithExt = CommandToExecute->Parameters.GetValue("WITH_EXT");
	CleanString(sWithExt);
	MakeUpper(sWithExt);
	if(sWithExt == "NO")
		bWithExt = false;


	GetStringValue(CommandToExecute, Object->GetFileName(bWithExt).c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT FILE_GET_EXTENSION
bool NvSession::ObjFileGetExtension(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	GetStringValue(CommandToExecute, Object->GetFileExtension().c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT FILE_GET_SIZE
bool NvSession::ObjFileGetSize(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	NvInt64 FileSize = Object->GetSize();
	if(FileSize == -1)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_GET_SIZE, ObjName.c_str());
		return false;
	}

	char Buf[50];
	sprintf(Buf, "%llu", FileSize);
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT FILE_REMOVE
bool NvSession::ObjFileRemove(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	bool bForce = false;
	string sForce = CommandToExecute->Parameters.GetValue("FORCE");
	CleanString(sForce);
	MakeUpper(sForce);
	if(sForce == "YES")
		bForce = true;

	Object->Remove(bForce);

	return true;
}




//OBJECT FILE_CREATE
bool NvSession::ObjFileCreate(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	if(!Object->Create())
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_CREATE, ObjName.c_str());
		return false;
	}
	return true;
}




//OBJECT FILE_CLEAR
bool NvSession::ObjFileClear(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	if(!Object->Clear())
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_CLEAR, ObjName.c_str());
		return false;
	}
	return true;
}




//OBJECT FILE_APPEND
bool NvSession::ObjFileAppend(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	bool bUseOutputContainer = false;
	string sUseOutputContainer = CommandToExecute->Parameters.GetValue("USE_OUT_CONTAINER");
	CleanString(sUseOutputContainer);
	MakeUpper(sUseOutputContainer);
	if(sUseOutputContainer == "YES")
		bUseOutputContainer = true;

	NvObjFile* Object = NULL;
	if(bUseOutputContainer)
		Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, OutContainer, ObjName.c_str(), NV_OBJTYPE_FILE);
	else
		Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);


	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object source name that should be in command parameters
	string SourceObjName = CommandToExecute->Parameters.GetValue("SNAME");
	CleanString(SourceObjName);

	NvObjFile* SourceObject = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, SourceObjName.c_str(), NV_OBJTYPE_FILE);

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Check if an offset and a number of bytes has been requested
	NvInt64 Offset = 0;
	NvInt64 BytesToCopy = -1;
	if(CommandToExecute->Parameters.KeyExist("OFFSET"))
	{
		string sOffset = CommandToExecute->Parameters.GetValue("OFFSET");
		CleanString(sOffset);
		Offset = NvAtoi64(sOffset.c_str());
	}
	if(CommandToExecute->Parameters.KeyExist("NUM_BYTES"))
	{
		string sBytesToCopy = CommandToExecute->Parameters.GetValue("NUM_BYTES");
		CleanString(sBytesToCopy);
		BytesToCopy = NvAtoi64(sBytesToCopy.c_str());
	}

	if(!Object->AppendFrom(SourceObject, Offset, BytesToCopy))
	{
		char *ErrorMessage = (char *)malloc(ObjName.length() + SourceObjName.length() + 100);
		sprintf(ErrorMessage, "dest:%s - src:%s", ObjName.c_str(), SourceObjName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_APPEND, ErrorMessage);
		free(ErrorMessage);
		return false;
	}
	return true;
}



//OBJECT FILE_APPEND
bool NvSession::ObjFileSetAscii(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->SetAscii();

	return true;
}




//OBJECT FILE_EXIST
bool NvSession::ObjFileExist(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	char Buf[20];
	strcpy(Buf, "NO");
	if(Object->Exist())
		strcpy(Buf, "YES");

	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}






//OBJECT FILE_SET_ORIGIN
bool NvSession::ObjFileSetOrigin(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string Origin = CommandToExecute->Parameters.GetValue("ORIGIN");
	CleanString(Origin);

	Object->SetOrigin(Origin);

	return true;
}



//OBJECT FILE_GET_ORIGIN
bool NvSession::ObjFileGetOrigin(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	GetStringValue(CommandToExecute, Object->GetOrigin().c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}



//OBJECT FILE_COMPRESS
bool NvSession::ObjFileCompress(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// The function uses the output container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object destination name that should be in command parameters
	string DestinationObjName = CommandToExecute->Parameters.GetValue("DNAME");
	CleanString(DestinationObjName);

	NvObjFile* DestinationObject = (NvObjFile*)GetObjectCheckType(CommandToExecute, OutContainer, DestinationObjName.c_str(), NV_OBJTYPE_FILE);

	if(DestinationObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	if(!Object->Compress(DestinationObject))
	{
		char *ErrorMessage = (char *)malloc(ObjName.length() + DestinationObjName.length() + 100);
		sprintf(ErrorMessage, "dest:%s - src:%s", DestinationObjName.c_str(), ObjName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_COMPRESS, ErrorMessage);
		free(ErrorMessage);
		return false;
	}
	return true;
}



//OBJECT FILE_DECOMPRESS
bool NvSession::ObjFileDecompress(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// The function uses the output container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object destination name that should be in command parameters
	string DestinationObjName = CommandToExecute->Parameters.GetValue("DNAME");
	CleanString(DestinationObjName);

	NvObjFile* DestinationObject = (NvObjFile*)GetObjectCheckType(CommandToExecute, OutContainer, DestinationObjName.c_str(), NV_OBJTYPE_FILE);

	if(DestinationObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	if(!Object->Decompress(DestinationObject))
	{
		char *ErrorMessage = (char *)malloc(ObjName.length() + DestinationObjName.length() + 100);
		sprintf(ErrorMessage, "dest:%s - src:%s", DestinationObjName.c_str(), ObjName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_DECOMPRESS, ErrorMessage);
		free(ErrorMessage);
		return false;
	}
	return true;
}




//OBJECT BINARY_GET_SIZE
bool NvSession::ObjBinaryGetSize(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjBinary* Object = (NvObjBinary*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_BINARY);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	char Buf[50];
	sprintf(Buf, "%d", Object->GetSize());
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}


//OBJECT BINARY_CLEAR
bool NvSession::ObjBinaryClear(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjBinary* Object = (NvObjBinary*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_BINARY);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->Clear();

	return true;
}


//OBJECT BINARY_APPEND
bool NvSession::ObjBinaryAppend(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	bool bUseOutputContainer = false;
	string sUseOutputContainer = CommandToExecute->Parameters.GetValue("USE_OUT_CONTAINER");
	CleanString(sUseOutputContainer);
	MakeUpper(sUseOutputContainer);
	if(sUseOutputContainer == "YES")
		bUseOutputContainer = true;

	NvObjBinary* Object = NULL;
	if(bUseOutputContainer)
		Object = (NvObjBinary*)GetObjectCheckType(CommandToExecute, OutContainer, ObjName.c_str(), NV_OBJTYPE_BINARY);
	else
		Object = (NvObjBinary*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_BINARY);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object source name that should be in command parameters
	string SourceObjName = CommandToExecute->Parameters.GetValue("SNAME");
	CleanString(SourceObjName);

	NvObjBinary* SourceObject = (NvObjBinary*)GetObjectCheckType(CommandToExecute, InContainer, SourceObjName.c_str(), NV_OBJTYPE_BINARY);

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Check if an offset and a number of bytes has been requested
	int Offset = 0;
	int BytesToCopy = -1;
	if(CommandToExecute->Parameters.KeyExist("OFFSET"))
	{
		string sOffset = CommandToExecute->Parameters.GetValue("OFFSET");
		CleanString(sOffset);
		Offset = atoi(sOffset.c_str());
	}
	if(CommandToExecute->Parameters.KeyExist("NUM_BYTES"))
	{
		string sBytesToCopy = CommandToExecute->Parameters.GetValue("NUM_BYTES");
		CleanString(sBytesToCopy);
		BytesToCopy = atoi(sBytesToCopy.c_str());
	}

	if(!Object->AppendFrom(SourceObject, Offset, BytesToCopy))
	{
		char *ErrorMessage = (char *)malloc(ObjName.length() + SourceObjName.length() + 100);
		sprintf(ErrorMessage, "dest:%s - src:%s", ObjName.c_str(), SourceObjName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_BINARY_CANNOT_APPEND, ErrorMessage);
		free(ErrorMessage);
		return false;
	}
	return true;
}



//OBJECT BINARY_SAVE
bool NvSession::ObjBinarySave(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// This command is authorized only from internal
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:OBJECT:BINARY_SAVE");
		return false;
	}

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjBinary* Object = (NvObjBinary*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_BINARY);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object name that should be in command parameters
	string FileName = CommandToExecute->Parameters.GetValue("FILENAME");
	SetDirName(FileName);
	if(!FileName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:BINARY_SAVE:FILENAME");
		return false;
	}

	// Check if an offset and a number of bytes has been requested
	int Offset = 0;
	int BytesToCopy = -1;
	if(CommandToExecute->Parameters.KeyExist("OFFSET"))
	{
		string sOffset = CommandToExecute->Parameters.GetValue("OFFSET");
		CleanString(sOffset);
		Offset = atoi(sOffset.c_str());
	}
	if(CommandToExecute->Parameters.KeyExist("NUM_BYTES"))
	{
		string sBytesToCopy = CommandToExecute->Parameters.GetValue("NUM_BYTES");
		CleanString(sBytesToCopy);
		BytesToCopy = atoi(sBytesToCopy.c_str());
	}

	if(!Object->Save(FileName.c_str(), Offset, BytesToCopy))
	{
		char *ErrorMessage = (char *)malloc(ObjName.length() + FileName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), FileName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_BINARY_CANNOT_SAVE, ErrorMessage);
		free(ErrorMessage);
		return false;
	}
	return true;
}



//OBJECT BINARY_LOAD
bool NvSession::ObjBinaryLoad(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// This command is authorized only from internal
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:OBJECT:BINARY_LOAD");
		return false;
	}

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get object name that should be in command parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjBinary* Object = (NvObjBinary*)GetObjectCheckType(CommandToExecute, InContainer, ObjName.c_str(), NV_OBJTYPE_BINARY);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object name that should be in command parameters
	string FileName = CommandToExecute->Parameters.GetValue("FILENAME");
	SetDirName(FileName);
	if(!FileName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "OBJECT:BINARY_LOAD:FILENAME");
		return false;
	}

	// Check if an offset and a number of bytes has been requested
	int Offset = 0;
	int BytesToCopy = -1;
	if(CommandToExecute->Parameters.KeyExist("OFFSET"))
	{
		string sOffset = CommandToExecute->Parameters.GetValue("OFFSET");
		CleanString(sOffset);
		Offset = atoi(sOffset.c_str());
	}
	if(CommandToExecute->Parameters.KeyExist("NUM_BYTES"))
	{
		string sBytesToCopy = CommandToExecute->Parameters.GetValue("NUM_BYTES");
		CleanString(sBytesToCopy);
		BytesToCopy = atoi(sBytesToCopy.c_str());
	}

	if(!Object->Load(FileName.c_str(), Offset, BytesToCopy))
	{
		char *ErrorMessage = (char *)malloc(ObjName.length() + FileName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), FileName.c_str());
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_BINARY_CANNOT_LOAD, ErrorMessage);
		free(ErrorMessage);
		return false;
	}
	return true;
}


//SYSTEM LOCK LOCK
bool NvSession::LockLock(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get lock name that should be in command parameters
	string LockName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(LockName);

	string ResultObject = CommandToExecute->Parameters.GetValue("GOT_LOCK");
	MakeUpper(ResultObject);
	if(CommandToExecute->Parameters.KeyExist("GOT_LOCK"))
		if(!ResultObject.length())
			ResultObject = "GOT_LOCK";

	if(ResultObject.length())
	{
		// The function uses the output container
		if(!CheckOutputContainer(CommandToExecute))
			return false;
		OutContainer->RemoveObject(ResultObject.c_str());
		if(!OutContainer->CreateObject(ResultObject.c_str(), NV_OBJTYPE_BOOLEAN))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ResultObject);
			return false;
		}
		NvObj *Object = OutContainer->GetObject(ResultObject.c_str());
		((NvObjBoolean *)Object)->SetValue(false);
	}

	if(!LockName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LOCK:LOCK:NAME");
		return false;
	}
	
	// Get lock info that may be in command parameters
	string LockInfo = CommandToExecute->Parameters.GetValue("INFO");
	CleanString(LockInfo);

					
	string WaitTimeInCommand = CommandToExecute->Parameters.GetValue("WAIT");
	CleanString(WaitTimeInCommand);
	MakeUpper(WaitTimeInCommand);
	int iWaitTimeInCommand = 300;	// by default we wait 300 seconds
	if(WaitTimeInCommand.length())
		iWaitTimeInCommand = atoi(WaitTimeInCommand.c_str());
	if(iWaitTimeInCommand < 0)
		iWaitTimeInCommand = -1;	// Infinite wait time

					
	string SpeedInCommand = CommandToExecute->Parameters.GetValue("SPEED");
	CleanString(SpeedInCommand);
	MakeUpper(SpeedInCommand);
	int iSpeedInCommand = 1;	// by default the speed is 1 (slow)
	if(SpeedInCommand.length())
		iSpeedInCommand = atoi(SpeedInCommand.c_str());
	if(iSpeedInCommand < 1)
		iSpeedInCommand = 1;
	if(iSpeedInCommand > 10)
		iSpeedInCommand = 10;


	// Get system option
	bool bSystemLock = false;
	string sSystemLock = CommandToExecute->Parameters.GetValue("SYSTEM");
	CleanString(sSystemLock);
	MakeUpper(sSystemLock);
	if(sSystemLock == "YES")
		bSystemLock = true;


	bool RetVal = false;
	if(bSystemLock)
		RetVal = GetMain()->LockList.Lock(LockName, m_sIdent, LockInfo, iWaitTimeInCommand, iSpeedInCommand);
	else
		RetVal = Application->LockList.Lock(LockName, m_sIdent, LockInfo, iWaitTimeInCommand, iSpeedInCommand);

	if(!ResultObject.length())
	{
		if(!RetVal)
		{
			CommandToExecute->Error.SetError("SYSTEM", "LOCK", NV_LOCK_CANNOT_LOCK, LockName);
			return false;
		}
		return true;
	}

	CommandToExecute->Error.SetError();

	NvObj *Object = OutContainer->GetObject(ResultObject.c_str());
	((NvObjBoolean *)Object)->SetValue(RetVal);

	return true;
}


//SYSTEM LOCK UNLOCK
bool NvSession::LockUnlock(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get lock name that should be in command parameters
	string LockName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(LockName);

	if(!LockName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LOCK:UNLOCK:NAME");
		return false;
	}


	// Get system option
	bool bSystemLock = false;
	string sSystemLock = CommandToExecute->Parameters.GetValue("SYSTEM");
	CleanString(sSystemLock);
	MakeUpper(sSystemLock);
	if(sSystemLock == "YES")
		bSystemLock = true;


	// Get force option
	bool bForce = false;
	string sForce = CommandToExecute->Parameters.GetValue("FORCE");
	CleanString(sForce);
	MakeUpper(sForce);
	if(sForce == "YES")
		bForce = true;

	bool RetVal = false;
	if(bSystemLock)
		RetVal = GetMain()->LockList.Unlock(LockName, m_sIdent, bForce);
	else
		RetVal = Application->LockList.Unlock(LockName, m_sIdent, bForce);

	if(!RetVal)
	{
		CommandToExecute->Error.SetError("SYSTEM", "LOCK", NV_LOCK_CANNOT_UNLOCK, LockName);
		return false;
	}
	return RetVal;
}


//SYSTEM LOCK ISLOCKED
bool NvSession::LockIsLocked(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get lock name that should be in command parameters
	string LockName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(LockName);

	if(!LockName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "LOCK:ISLOCKED:NAME");
		return false;
	}


	// Get system option
	bool bSystemLock = false;
	string sSystemLock = CommandToExecute->Parameters.GetValue("SYSTEM");
	CleanString(sSystemLock);
	MakeUpper(sSystemLock);
	if(sSystemLock == "YES")
		bSystemLock = true;

	// We must update the session list because the time out sessions are checked only each 30 seconds
	// SO an object may stay locked 30 seconds after the session is closed
	GetMain()->Sessions.Update();

	bool RetVal = false;
	string Owner = "";
	string Info = "";
	if(bSystemLock)
		RetVal = GetMain()->LockList.Islocked(LockName, Owner, Info);
	else
		RetVal = Application->LockList.Islocked(LockName, Owner, Info);


	OutContainer->RemoveObject("LOCKED");
	OutContainer->RemoveObject("OWNER");
	OutContainer->RemoveObject("INFO");
	if(!OutContainer->CreateObject("LOCKED", NV_OBJTYPE_BOOLEAN))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "LOCKED");
		return false;
	}
	if(!OutContainer->CreateObject("OWNER", NV_OBJTYPE_STRING))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "OWNER");
		OutContainer->RemoveObject("LOCKED");
		return false;
	}
	if(!OutContainer->CreateObject("INFO", NV_OBJTYPE_STRING))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "INFO");
		OutContainer->RemoveObject("LOCKED");
		OutContainer->RemoveObject("OWNER");
		return false;
	}

	NvObj *Object = OutContainer->GetObject("LOCKED");
	((NvObjBoolean *)Object)->SetValue(RetVal);

	Object = OutContainer->GetObject("OWNER");
	((NvObjString *)Object)->SetValue(Owner);

	Object = OutContainer->GetObject("INFO");
	((NvObjString *)Object)->SetValue(Info);

	char Buf[20];
	if(RetVal)
		strcpy(Buf, "YES");
	else
		strcpy(Buf, "NO");
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}


//SYSTEM LOCK INFO
bool NvSession::LockInfo(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get lock name that should be in command parameters
	string LockName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(LockName);

	OutContainer->RemoveObject("INFO");
	if(!OutContainer->CreateObject("INFO", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "INFO");
		return false;
	}

	NvObjTable *LockInfo = (NvObjTable *)(OutContainer->GetObject("INFO"));

	// Get system option
	bool bSystemLock = false;
	string sSystemLock = CommandToExecute->Parameters.GetValue("SYSTEM");
	CleanString(sSystemLock);
	MakeUpper(sSystemLock);
	if(sSystemLock == "YES")
		bSystemLock = true;

	bool RetVal = false;
	if(bSystemLock)
		RetVal = GetMain()->LockList.GetInfo(LockName, LockInfo);
	else
		RetVal = Application->LockList.GetInfo(LockName, LockInfo);

	if(!RetVal)
	{
		OutContainer->RemoveObject("INFO");
		CommandToExecute->Error.SetError("SYSTEM", "LOCK", NV_LOCK_CANNOT_GET_INFO, LockName);
		return false;
	}
	return RetVal;
}



//SYSTEM SEMAPHORE LOCK
bool NvSession::SemaLock(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get sema name that should be in command parameters
	string SemaName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(SemaName);

	if(!SemaName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SEMAPHORE:LOCK:NAME");
		return false;
	}
	

	// Get value
	int iValue = 1;
	string sValue = CommandToExecute->Parameters.GetValue("VALUE");
	CleanString(sValue);
	if(sValue.length())
		iValue = atoi(sValue.c_str());
	if(iValue <= 0)
		iValue = 1;

	// Get system option
	bool bSystemLock = false;
	string sSystemLock = CommandToExecute->Parameters.GetValue("SYSTEM");
	CleanString(sSystemLock);
	MakeUpper(sSystemLock);
	if(sSystemLock == "YES")
		bSystemLock = true;


	bool RetVal = false;
	if(bSystemLock)
		RetVal = GetMain()->SemaList.Lock(SemaName, m_sIdent, iValue);
	else
		RetVal = Application->SemaList.Lock(SemaName, m_sIdent, iValue);

	if(!RetVal)
	{
		CommandToExecute->Error.SetError("SYSTEM", "SEMAPHORE", NV_SEMAPHORE_CANNOT_LOCK, SemaName);
		return false;
	}

	return RetVal;
}


//SYSTEM SEMAPHORE UNLOCK
bool NvSession::SemaUnlock(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get lock name that should be in command parameters
	string SemaName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(SemaName);

	if(!SemaName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SEMAPHORE:UNLOCK:NAME");
		return false;
	}


	// Get system option
	bool bSystemLock = false;
	string sSystemLock = CommandToExecute->Parameters.GetValue("SYSTEM");
	CleanString(sSystemLock);
	MakeUpper(sSystemLock);
	if(sSystemLock == "YES")
		bSystemLock = true;

	bool RetVal = false;
	if(bSystemLock)
		RetVal = GetMain()->SemaList.Unlock(SemaName, m_sIdent);
	else
		RetVal = Application->SemaList.Unlock(SemaName, m_sIdent);

	if(!RetVal)
	{
		CommandToExecute->Error.SetError("SYSTEM", "SEMAPHORE", NV_SEMAPHORE_CANNOT_UNLOCK, SemaName);
		return false;
	}

	return RetVal;
}


//SYSTEM SEMAPHORE REMOVE
bool NvSession::SemaRemove(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get lock name that should be in command parameters
	string SemaName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(SemaName);

	if(!SemaName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "SEMAPHORE:REMOVE:NAME");
		return false;
	}


	// Get system option
	bool bSystemLock = false;
	string sSystemLock = CommandToExecute->Parameters.GetValue("SYSTEM");
	CleanString(sSystemLock);
	MakeUpper(sSystemLock);
	if(sSystemLock == "YES")
		bSystemLock = true;

	bool RetVal = false;
	if(bSystemLock)
		RetVal = GetMain()->SemaList.Remove(SemaName);
	else
		RetVal = Application->SemaList.Remove(SemaName);

	if(!RetVal)
	{
		CommandToExecute->Error.SetError("SYSTEM", "SEMAPHORE", NV_SEMAPHORE_CANNOT_REMOVE, SemaName);
		return false;
	}

	return RetVal;
}


//SYSTEM SEMAPHORE INFO
bool NvSession::SemaInfo(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get lock name that should be in command parameters
	string SemaName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(SemaName);

	OutContainer->RemoveObject("INFO");
	if(!OutContainer->CreateObject("INFO", NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "INFO");
		return false;
	}

	NvObjTable *SemaInfo = (NvObjTable *)(OutContainer->GetObject("INFO"));

	// Get system option
	bool bSystemLock = false;
	string sSystemLock = CommandToExecute->Parameters.GetValue("SYSTEM");
	CleanString(sSystemLock);
	MakeUpper(sSystemLock);
	if(sSystemLock == "YES")
		bSystemLock = true;

	bool RetVal = false;
	if(bSystemLock)
		RetVal = GetMain()->SemaList.GetInfo(SemaName, SemaInfo);
	else
		RetVal = Application->SemaList.GetInfo(SemaName, SemaInfo);

	if(!RetVal)
	{
		OutContainer->RemoveObject("INFO");
		CommandToExecute->Error.SetError("SYSTEM", "SEMAPHORE", NV_SEMAPHORE_CANNOT_GET_INFO, SemaName);
		return false;
	}
	return RetVal;
}




//CONTAINER COPY
bool NvSession::CopyContainer(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get append option
	int AppendMode = NV_CONTAINER_COPY_NO_APPEND;
	string sAppend = CommandToExecute->Parameters.GetValue("APPEND");
	CleanString(sAppend);
	MakeUpper(sAppend);
	if(sAppend == "YES")
	{
		AppendMode = NV_CONTAINER_COPY_APPEND_NO_REPLACE;
		string sReplace = CommandToExecute->Parameters.GetValue("REPLACE");
		CleanString(sReplace);
		MakeUpper(sReplace);
		if(sReplace == "YES")
			AppendMode = NV_CONTAINER_COPY_APPEND;
	}

	// Get subcontainer option
	bool bWithSubContainers = false;
	string sWithSubContainers = CommandToExecute->Parameters.GetValue("SUBCONTAINERS");
	CleanString(sWithSubContainers);
	MakeUpper(sWithSubContainers);
	if(sWithSubContainers == "YES")
		bWithSubContainers = true;

	// Get object names that may be in parameters
	string ObjNames = CommandToExecute->Parameters.GetValue("OBJECTS");
	CleanString(ObjNames);

	// Check the persist parameter
	string PersistTimeFileInCommand = CommandToExecute->Parameters.GetValue("PERSIST");
	CleanString(PersistTimeFileInCommand);
	MakeUpper(PersistTimeFileInCommand);
	int iPersistTimeFileInCommand = 0;	// The file is not persistant by default on the server
	if(PersistTimeFileInCommand.length())
		iPersistTimeFileInCommand = atoi(PersistTimeFileInCommand.c_str());
	if(iPersistTimeFileInCommand < 0)
		iPersistTimeFileInCommand = -1;

	// Get with file option
	bool bWithFiles = true;
	string sWithFiles = CommandToExecute->Parameters.GetValue("FILES");
	CleanString(sWithFiles);
	MakeUpper(sWithFiles);
	if(sWithFiles == "NO")
		bWithFiles = false;

	string FileDir = "";
	if(bWithFiles)
	{
		if(iPersistTimeFileInCommand == -1)
			FileDir = Application->GetFileDir();
		else
			FileDir = Application->GetWorkDir();
		
		
		if((CommandToExecute->m_iCommandSource == NV_SOURCE_PROCEDURE) || (CommandToExecute->m_iCommandSource == NV_SOURCE_SERVICE))
		{
			// from internal, we can change the file directory of new file objects
			string FileDirInCommand = CommandToExecute->Parameters.GetValue("FILEDIR");
			CleanString(FileDirInCommand);
			SetDirName(FileDirInCommand);
			// eliminate last slash
			if(FileDirInCommand.size() && FileDirInCommand[FileDirInCommand.size()-1] == '/')
				FileDirInCommand.erase(FileDirInCommand.size()-1, 1);
			// eliminate last backslash
			if(FileDirInCommand.size() && FileDirInCommand[FileDirInCommand.size()-1] == '\\')
				FileDirInCommand.erase(FileDirInCommand.size()-1, 1);
			if(FileDirInCommand.length())
				FileDir = FileDirInCommand;
			if(iPersistTimeFileInCommand == -1)
			{
				// We get application work dir
				string sWorkDir = "";
				sWorkDir = Application->GetWorkDir();
				if(CompareWorkDir(FileDir, sWorkDir))
				{
					CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_SET_PERSISTANT, FileDir);
					return false;
				}
			}
		}
	}


	// Get with data option option
	bool bWithData = true;
	string sWithData = CommandToExecute->Parameters.GetValue("WITH_DATA");
	CleanString(sWithData);
	MakeUpper(sWithData);
	if(sWithData == "NO")
		bWithData = false;

	const char *ObjsForCommand = NULL;
	if(CommandToExecute->Parameters.KeyExist("OBJECTS"))
		ObjsForCommand = ObjNames.c_str();


	// Verify that we do not break the brach we sit on
	if(bWithSubContainers)
	{
		NvContainer *ContainerHere = OutContainer;
		bool bRecurseFound = false;
		while(ContainerHere != NULL)
		{
			ContainerHere = ContainerHere->GetParent();
			if(ContainerHere == InContainer)
			{
				bRecurseFound = true;
				break;
			}
		}
		if(bRecurseFound)
		{
			char *ErrorMessage = (char *)malloc(InContainer->GetName().length() + OutContainer->GetName().length() + 100);
			sprintf(ErrorMessage, "%s - %s", InContainer->GetName().c_str(), OutContainer->GetName().c_str());
			CommandToExecute->Error.SetError("SYSTEM", "CONTAINER", NV_CONTAINER_CANNOT_COPY, ErrorMessage);
			free(ErrorMessage);
			return false;
		}
		
		if(AppendMode == NV_CONTAINER_COPY_NO_APPEND)
		{
			ContainerHere = InContainer;
			while(ContainerHere != NULL)
			{
				ContainerHere = ContainerHere->GetParent();
				if(ContainerHere == OutContainer)
				{
					bRecurseFound = true;
					break;
				}
			}
			if(bRecurseFound)
			{
				char *ErrorMessage = (char *)malloc(InContainer->GetName().length() + OutContainer->GetName().length() + 100);
				sprintf(ErrorMessage, "%s - %s", InContainer->GetName().c_str(), OutContainer->GetName().c_str());
				CommandToExecute->Error.SetError("SYSTEM", "CONTAINER", NV_CONTAINER_CANNOT_COPY, ErrorMessage);
				free(ErrorMessage);
				return false;
			}
		}
	}

	if(!OutContainer->CopyFrom(InContainer, AppendMode, ObjsForCommand, bWithSubContainers, bWithData, FileDir.c_str(), iPersistTimeFileInCommand))
	{
		char *ErrorMessage = (char *)malloc(InContainer->GetName().length() + OutContainer->GetName().length() + 100);
		sprintf(ErrorMessage, "%s - %s", InContainer->GetName().c_str(), OutContainer->GetName().c_str());
		CommandToExecute->Error.SetError("SYSTEM", "CONTAINER", NV_CONTAINER_CANNOT_COPY, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	return true;
}




//CONTAINER REMOVE
bool NvSession::RemoveContainer(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get container name that should be in command parameters
	string ContainerName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ContainerName);

	NvContainer *Container = GetContainer(ContainerName.c_str(), false, CommandToExecute);

	if(Container == NULL)
		return true;	// nothing to do in this case

	if((Container == InContainer) || (Container == OutContainer))
	{
		// We cannot remove the incontainer or outcontainer
		Container->Clear();
		FreeContainer(Container, ContainerName.c_str(), false, CommandToExecute);
		return true;
	}

	FreeContainer(Container, ContainerName.c_str(), false, CommandToExecute);


	string::size_type index = ContainerName.find(':');
	if(index == string::npos)
	{
		// This is a session container
		Containers.RemoveContainer(ContainerName.c_str());
		return true;
	}

	string sLevel = ContainerName.substr(0, index);
	ContainerName = ContainerName.substr(index+1);
	CleanString(sLevel, true);
	MakeUpper(sLevel);

	int Level = CONTAINER_LEVEL_APPLICATION;

	NvContainerList *ContainerList = NULL;
	if(Application != NULL)
		ContainerList = &(Application->Containers);
	if(sLevel == "SYSTEM")
		Level = CONTAINER_LEVEL_SYSTEM;
	else if(sLevel == "[NV_PARENT]")
		Level = CONTAINER_LEVEL_PARENT;
	else if(sLevel.length())
		Level = CONTAINER_LEVEL_SERVICE;

	NvService *Service = NULL;
	if(Level == CONTAINER_LEVEL_SYSTEM)
		ContainerList = &(GetMain()->Containers);		
	if(Level == CONTAINER_LEVEL_SERVICE)
	{
		Service = GetMain()->Services.GetService(sLevel.c_str(), true, true);
		if(Service == NULL)
			return true;	// Nothing to do, the service doesn't exist
		ContainerList = &(Service->Containers);		
	}
	if(Level == CONTAINER_LEVEL_PARENT)
	{
		// This is parent container level
		// This can occur when a named or normal session has been called by
		// a parent session. The called session can then directly access
		// the containers of its parent.
		// If there is no parent, the actual session container list is used
		NvSession* Parent = (NvSession*)(CommandToExecute->m_pParentSession);
		if(Parent == NULL)
			ContainerList = &Containers;		
		else
			ContainerList = &(Parent->Containers);
	}

	ContainerList->RemoveContainer(ContainerName.c_str());
	
	if(Service != NULL)
		GetMain()->Services.FreeService(sLevel.c_str());	// We must free the service in this case

	return true;
}




//CONTAINER LIST
bool NvSession::ListContainers(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	
	string ObjName = CommandToExecute->Parameters.GetValue("LIST");
	CleanString(ObjName);
	if(!ObjName.length())
		ObjName = "LIST";
	OutContainer->RemoveObject(ObjName.c_str());
	if(!OutContainer->CreateObject(ObjName.c_str(), NV_OBJTYPE_STRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjName);
		return false;
	}
	
	NvObjStringList *ContainerListResult = (NvObjStringList *)(OutContainer->GetObject(ObjName.c_str()));

	string sLevel = CommandToExecute->Parameters.GetValue("LEVEL");
	CleanString(sLevel);
	MakeUpper(sLevel);
	if(!sLevel.length())
		sLevel = "SESSION";

	NvContainerList *ContainerList = NULL;
	if(sLevel == "SESSION")
		ContainerList = &Containers;
	if(sLevel == "APPLICATION")
		if(Application != NULL)
			ContainerList = &(Application->Containers);
	if(sLevel == "SYSTEM")
		ContainerList = &(GetMain()->Containers);		
	if(sLevel == "PARENT")
	{
		NvSession* Parent = (NvSession*)(CommandToExecute->m_pParentSession);
		if(Parent == NULL)
			ContainerList = &Containers;		
		else
			ContainerList = &(Parent->Containers);
	}
	NvService *Service = NULL;
	if(sLevel == "SERVICE")
	{
		string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
		CleanString(ServiceName);
		Service = GetMain()->Services.GetService(sLevel.c_str(), true, true);
		if(Service == NULL)
			return true;	// Nothing to do, the service doesn't exist
		ContainerList = &(Service->Containers);		
	}

	if(ContainerList == NULL)
		return true;

	vector<string> vContainers;
	ContainerList->GetContainerNames(&vContainers);

	if(Service != NULL)
		GetMain()->Services.FreeService(sLevel.c_str());	// We must free the service in this case

	int i;
	for(i=0;i<(int)(vContainers.size());i++)
		ContainerListResult->Add(vContainers[i]);

	return true;
}


//CONTAINER GET_NUM_SUBCONTAINERS
bool NvSession::GetNumSubContainers(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	char Buf[50];
	sprintf(Buf, "%d", InContainer->m_mSubcontainers.size());
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}



//CONTAINER GET_SUBCONTAINER_NAME
bool NvSession::GetSubContainerName(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// Get object index that should be in command parameters
	int iSubContainerIndex = 0;
	string SubContainerIndex = CommandToExecute->Parameters.GetValue("INDEX");
	CleanString(SubContainerIndex);
	iSubContainerIndex = atoi(SubContainerIndex.c_str());
	
	if((iSubContainerIndex < 1) || (iSubContainerIndex > (int)(InContainer->m_mSubcontainers.size())))
	{
		CommandToExecute->Error.SetError("SYSTEM", "CONTAINER", NV_INVALID_CONTAINER_INDEX, SubContainerIndex);
		return false;
	}

	map<string,void *>::iterator pos;
	int i;
	NvContainer* Container = NULL;
	for(i=0,pos = InContainer->m_mSubcontainers.begin();i<iSubContainerIndex;i++,++pos)
	{
		Container = (NvContainer*)(pos->second);
	}
	string ContainerName = "";
	if(Container != NULL)
		ContainerName = Container->GetName();

	GetStringValue(CommandToExecute, ContainerName.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}



//CONTAINER GET_PARENT_NAME
bool NvSession::GetContainerParentName(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	string ParentName = "";
	NvContainer *Container = InContainer->GetParent();
	if(Container != NULL)
		ParentName = Container->GetName();

	GetStringValue(CommandToExecute, ParentName.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}



//CONTAINER EXPORT
bool NvSession::ExportContainer(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	

	// Get export file name that may be in parameters
	string FileName = CommandToExecute->Parameters.GetValue("FILENAME");
	SetDirName(FileName);
	if(!FileName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "CONTAINER:EXPORT:FILENAME");
		return false;
	}

	// Get subcontainer option
	bool bWithSubContainers = false;
	string sWithSubContainers = CommandToExecute->Parameters.GetValue("SUBCONTAINERS");
	CleanString(sWithSubContainers);
	MakeUpper(sWithSubContainers);
	if(sWithSubContainers == "YES")
		bWithSubContainers = true;

	// Get object names that may be in parameters
	string ObjNames = CommandToExecute->Parameters.GetValue("OBJECTS");
	CleanString(ObjNames);

	const char *ObjsForCommand = NULL;
	if(CommandToExecute->Parameters.KeyExist("OBJECTS"))
		ObjsForCommand = ObjNames.c_str();

	if(!InContainer->Export(FileName.c_str(), ObjsForCommand, bWithSubContainers))
	{
		CommandToExecute->Error.SetError("SYSTEM", "CONTAINER", NV_CONTAINER_CANNOT_EXPORT, FileName);
		return false;
	}

	return true;
}





//CONTAINER IMPORT
bool NvSession::ImportContainer(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	
	// Get import file name that may be in parameters
	string FileName = CommandToExecute->Parameters.GetValue("FILENAME");
	SetDirName(FileName);
	if(!FileName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "CONTAINER:IMPORT:FILENAME");
		return false;
	}


	// Get append option
	int AppendMode = NV_CONTAINER_COPY_NO_APPEND;
	string sAppend = CommandToExecute->Parameters.GetValue("APPEND");
	CleanString(sAppend);
	MakeUpper(sAppend);
	if(sAppend == "YES")
	{
		AppendMode = NV_CONTAINER_COPY_APPEND_NO_REPLACE;
		string sReplace = CommandToExecute->Parameters.GetValue("REPLACE");
		CleanString(sReplace);
		MakeUpper(sReplace);
		if(sReplace == "YES")
			AppendMode = NV_CONTAINER_COPY_APPEND;
	}


	// Check the persist parameter
	string PersistTimeFileInCommand = CommandToExecute->Parameters.GetValue("PERSIST");
	CleanString(PersistTimeFileInCommand);
	MakeUpper(PersistTimeFileInCommand);
	int iPersistTimeFileInCommand = 0;	// The file is not persistant by default on the server
	if(PersistTimeFileInCommand.length())
		iPersistTimeFileInCommand = atoi(PersistTimeFileInCommand.c_str());
	if(iPersistTimeFileInCommand < 0)
		iPersistTimeFileInCommand = -1;

	// Get with file option
	bool bWithFiles = true;
	string sWithFiles = CommandToExecute->Parameters.GetValue("FILES");
	CleanString(sWithFiles);
	MakeUpper(sWithFiles);
	if(sWithFiles == "NO")
		bWithFiles = false;

	string FileDir = "";
	if(bWithFiles)
	{
		if(iPersistTimeFileInCommand == -1)
			FileDir = Application->GetFileDir();
		else
			FileDir = Application->GetWorkDir();
		if((CommandToExecute->m_iCommandSource == NV_SOURCE_PROCEDURE) || (CommandToExecute->m_iCommandSource == NV_SOURCE_SERVICE))
		{
			// from internal, we can change the file directory of new file objects
			string FileDirInCommand = CommandToExecute->Parameters.GetValue("FILEDIR");
			CleanString(FileDirInCommand);
			SetDirName(FileDirInCommand);
			// eliminate last slash
			if(FileDirInCommand.size() && FileDirInCommand[FileDirInCommand.size()-1] == '/')
				FileDirInCommand.erase(FileDirInCommand.size()-1, 1);
			// eliminate last backslash
			if(FileDirInCommand.size() && FileDirInCommand[FileDirInCommand.size()-1] == '\\')
				FileDirInCommand.erase(FileDirInCommand.size()-1, 1);
			if(FileDirInCommand.length())
				FileDir = FileDirInCommand;
			if(iPersistTimeFileInCommand == -1)
			{
				// We get application work dir
				string sWorkDir = "";
				sWorkDir = Application->GetWorkDir();
				if(CompareWorkDir(FileDir, sWorkDir))
				{
					CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_SET_PERSISTANT, FileDir);
					return false;
				}
			}
		}
	}

	// Get eventual specific object name
	string ObjectName = CommandToExecute->Parameters.GetValue("OBJECT");
	CleanString(ObjectName);

	// Check if an offset and a number of bytes has been requested
	// They are used for file and binary types
	NvInt64 Offset = 0;
	NvInt64 BytesToCopy = -1;
	if(CommandToExecute->Parameters.KeyExist("OFFSET"))
	{
		string sOffset = CommandToExecute->Parameters.GetValue("OFFSET");
		CleanString(sOffset);
		Offset = NvAtoi64(sOffset.c_str());
	}
	if(CommandToExecute->Parameters.KeyExist("NUM_BYTES"))
	{
		string sBytesToCopy = CommandToExecute->Parameters.GetValue("NUM_BYTES");
		CleanString(sBytesToCopy);
		BytesToCopy = NvAtoi64(sBytesToCopy.c_str());
	}

	NvInt64 Size = 0;
	if(!OutContainer->Import(FileName.c_str(), AppendMode, FileDir.c_str(), iPersistTimeFileInCommand, ObjectName.c_str(), &Size, Offset, BytesToCopy))
	{
		CommandToExecute->Error.SetError("SYSTEM", "CONTAINER", NV_CONTAINER_CANNOT_IMPORT, FileName);
		return false;
	}

	char Buf[50];
	sprintf(Buf, "%llu", Size);
	GetStringValue(CommandToExecute, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//CONTAINER VIEW
bool NvSession::ViewContainer(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;
	
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;
	
	string ObjName = CommandToExecute->Parameters.GetValue("INFO");
	CleanString(ObjName);
	if(!ObjName.length())
		ObjName = "INFO";
	OutContainer->RemoveObject(ObjName.c_str());
	if(!OutContainer->CreateObject(ObjName.c_str(), NV_OBJTYPE_TABLE))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjName);
		return false;
	}

	NvObjTable *ContainerInfo = (NvObjTable *)(OutContainer->GetObject(ObjName.c_str()));

	// Get own name option
	bool bWithRootName = true;
	string sWithRootName = CommandToExecute->Parameters.GetValue("WITH_ROOT_NAME");
	CleanString(sWithRootName);
	MakeUpper(sWithRootName);
	if(sWithRootName == "NO")
		bWithRootName = false;

	InContainer->View(ContainerInfo, bWithRootName);
	return true;
}



//SYSTEM CONTAINER ENCODE
bool NvSession::EncodeContainer(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	string sSrcEncoding = CommandToExecute->Parameters.GetValue("SRC_ENCODING");
	CleanString(sSrcEncoding, true);
	MakeUpper(sSrcEncoding);
	if(!sSrcEncoding.length())
		sSrcEncoding = "INTERNAL";
	bool bSrcUnicode = false;
	if(sSrcEncoding == "UTF-8")
		bSrcUnicode = true;
	if(sSrcEncoding == "INTERNAL")
		bSrcUnicode = GetMain()->IsUnicode();

	string sDestEncoding = CommandToExecute->Parameters.GetValue("DEST_ENCODING");
	CleanString(sDestEncoding, true);
	MakeUpper(sDestEncoding);
	if(!sDestEncoding.length())
		sDestEncoding = "INTERNAL";
	bool bDestUnicode = false;
	if(sDestEncoding == "UTF-8")
		bDestUnicode = true;
	if(sDestEncoding == "INTERNAL")
		bDestUnicode = GetMain()->IsUnicode();

	if(bSrcUnicode == bDestUnicode)
		return true;	// nothing to do

	string sWithName = CommandToExecute->Parameters.GetValue("WITH_NAME");
	CleanString(sWithName, true);
	MakeUpper(sWithName);
	bool bWithName = false;
	if(sWithName == "YES")
		bWithName = true;

	bool bWithSubContainers = true;
	string sWithSubContainers = CommandToExecute->Parameters.GetValue("WITH_SUBCONTAINERS");
	CleanString(sWithSubContainers);
	MakeUpper(sWithSubContainers);
	if(sWithSubContainers == "YES")
		bWithSubContainers = true;

	InContainer->Encode(bSrcUnicode, bDestUnicode, bWithSubContainers, bWithName);
	CommandToExecute->m_sInContainer = InContainer->GetName();
	CommandToExecute->m_sOutContainer = OutContainer->GetName();

	return true;
}


// Return the registry corresponding to the requested key
// and NULL if not authorized or invalid registry
// This is an internal function only
NvRegistryContainer* NvSession::GetRegistry(NvCommand *CommandToExecute, string& Key, string& ServiceName, bool& AppRegistry, bool& UserRegistry)
{
	ServiceName = "";
	UserRegistry = false;
		
	NvRegistryContainer *RegistryContainer =  &(Application->AppRegistry);
	AppRegistry = true;

	// Get key name that may be in parameters
	Key = CommandToExecute->Parameters.GetValue("KEY");
	CleanString(Key);
	MakeLower(Key);
	Key = NvContainerList::CleanContainerName(Key);

	ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);

	// Get registry name (user registry)
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	NvRegistryContainerList::CleanRegistryName(Name);

	NvService *Service = NULL;

	// Check if this is a system or service registry key (key is enclosed in parentheses preceded eventuall by the service name)
	// ex: Toto(zut.titi) -> key zut.titi for registry service toto
	string::size_type index = Key.find('(');
	bool bSystemRegistry = false;
	if((!ServiceName.length()) && (index != string::npos))
	{
		if(Key[Key.length() - 1] != ')')
			return NULL;

		// this is a system or service key
		if(index != 0)
		{
			// it's probably a service key
			// we must get the service name
			ServiceName = Key.substr(0, index);
			CleanString(ServiceName);
			MakeUpper(ServiceName);
			Key = Key.substr(index);
		}

		// We must remove the parentheses
		Key = Key.substr(1, Key.length() - 2);
		
		// And get the service registry
		if(!ServiceName.length())
			ServiceName = "SYSTEM";
	}

	// And get the service registry
	if(ServiceName.length())
	{
		if(ServiceName == "SYSTEM")
		{
			bSystemRegistry = true;
			RegistryContainer =  &(GetMain()->SystemRegistry);	// This is the system registry
		}
		else
		{
			Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
			if(Service == NULL)
			{
				ServiceName = "";
				return NULL;
			}
			RegistryContainer =  &(Service->SrvRegistry);	// This is the service registry
		}
	}

	if(Name.length())
	{
		UserRegistry = true;
		// If this is a user registry, we try to get it from corresponding level
		// First check if its ever opened
		string RegistryIdent = ServiceName;
		if(bSystemRegistry)
			RegistryIdent = "SYSTEM";
		CleanString(RegistryIdent);
		MakeUpper(RegistryIdent);
		RegistryIdent += "::::";
		RegistryIdent += Name;
		if(m_oRegistries.find(RegistryIdent) == m_oRegistries.end())
		{
			// The registry is not opened
			if(ServiceName.length())
				GetMain()->Services.FreeService(ServiceName.c_str());
			ServiceName = "";
			return NULL;
		}
		if(bSystemRegistry)
			RegistryContainer = GetMain()->Registries.GetRegistry(Name);
		else if(ServiceName.length())
		{
			RegistryContainer = Service->Registries.GetRegistry(Name);
			if(RegistryContainer == NULL)
			{
				if(ServiceName.length())
					GetMain()->Services.FreeService(ServiceName.c_str());
				ServiceName = "";
			}
		}
		else
			RegistryContainer = Application->Registries.GetRegistry(Name);
	}
	else
	{
		CleanString(Key);
		NvContainerList::CleanContainerName(Key);
		string RootKey = Key;
		index = RootKey.find('.');
		if(index != string::npos)
			RootKey = RootKey.substr(0, index);
		CleanString(RootKey);
		MakeLower(RootKey);

		if(RootKey == "system")
		{
			// this is a reserved registry key (only for internal use). Its use depends of the original command source
			if(bSystemRegistry)	// Forbidden to access system system registry key
				return NULL;

			if(ServiceName.length())
			{
				// Authorized only if the command comes from the same service
				if(CommandToExecute->m_iCommandSource != NV_SOURCE_SERVICE)
				{
					GetMain()->Services.FreeService(ServiceName.c_str());
					ServiceName = "";
					return NULL;
				}
				string ServiceCommand = CommandToExecute->m_sSourceService;
				MakeUpper(ServiceCommand);
				if(ServiceCommand != ServiceName)
				{
					GetMain()->Services.FreeService(ServiceName.c_str());
					ServiceName = "";
					return NULL;
				}
			}
			else
			{
				// Authorized only if the command comes from the application procedure
				if(CommandToExecute->m_iCommandSource != NV_SOURCE_PROCEDURE)
					return NULL;
				if(m_sProcName.find('(') != string::npos)	// This is not an application procedure
					return NULL;
			}
		}
	}

	if(ServiceName.length() || bSystemRegistry)
		AppRegistry = false;

	return RegistryContainer;
}




//REGISTRY GET
bool NvSession::GetRegistryKey(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get key name that may be in parameters
	string Key = CommandToExecute->Parameters.GetValue("KEY");
	CleanString(Key);
	MakeLower(Key);
	Key = NvContainerList::CleanContainerName(Key);


	// Get append option
	int AppendMode = NV_CONTAINER_COPY_NO_APPEND;
	string sAppend = CommandToExecute->Parameters.GetValue("APPEND");
	CleanString(sAppend);
	MakeUpper(sAppend);
	if(sAppend == "YES")
	{
		AppendMode = NV_CONTAINER_COPY_APPEND_NO_REPLACE;
		string sReplace = CommandToExecute->Parameters.GetValue("REPLACE");
		CleanString(sReplace);
		MakeUpper(sReplace);
		if(sReplace == "YES")
			AppendMode = NV_CONTAINER_COPY_APPEND;
	}

	// Get subkey option
	bool bWithSubKeys = false;
	string sWithSubKeys = CommandToExecute->Parameters.GetValue("SUBKEYS");
	CleanString(sWithSubKeys);
	MakeUpper(sWithSubKeys);
	if(sWithSubKeys == "YES")
		bWithSubKeys = true;

	// Get recurse option
	bool bRecurse = true;
	string sRecurse = CommandToExecute->Parameters.GetValue("RECURSE");
	CleanString(sRecurse);
	MakeUpper(sRecurse);
	if(sRecurse == "NO")
		bRecurse = false;

	// Get object names that may be in parameters
	string EntryNames = CommandToExecute->Parameters.GetValue("ENTRIES");
	CleanString(EntryNames);

	// Check the persist parameter
	string PersistTimeFileInCommand = CommandToExecute->Parameters.GetValue("PERSIST");
	CleanString(PersistTimeFileInCommand);
	MakeUpper(PersistTimeFileInCommand);
	int iPersistTimeFileInCommand = 0;	// The file is not persistant by default on the server
	if(PersistTimeFileInCommand.length())
		iPersistTimeFileInCommand = atoi(PersistTimeFileInCommand.c_str());
	if(iPersistTimeFileInCommand < 0)
		iPersistTimeFileInCommand = -1;

	// Get with file option
	bool bWithFiles = true;
	string sWithFiles = CommandToExecute->Parameters.GetValue("FILES");
	CleanString(sWithFiles);
	MakeUpper(sWithFiles);
	if(sWithFiles == "NO")
		bWithFiles = false;

	string FileDir = "";
	if(bWithFiles)
	{
		if(iPersistTimeFileInCommand == -1)
			FileDir = Application->GetFileDir();
		else
			FileDir = Application->GetWorkDir();
		if((CommandToExecute->m_iCommandSource == NV_SOURCE_PROCEDURE) || (CommandToExecute->m_iCommandSource == NV_SOURCE_SERVICE))
		{
			// from internal, we can change the file directory of new file objects
			string FileDirInCommand = CommandToExecute->Parameters.GetValue("FILEDIR");
			CleanString(FileDirInCommand);
			SetDirName(FileDirInCommand);
			// eliminate last slash
			if(FileDirInCommand.size() && FileDirInCommand[FileDirInCommand.size()-1] == '/')
				FileDirInCommand.erase(FileDirInCommand.size()-1, 1);
			// eliminate last backslash
			if(FileDirInCommand.size() && FileDirInCommand[FileDirInCommand.size()-1] == '\\')
				FileDirInCommand.erase(FileDirInCommand.size()-1, 1);
			if(FileDirInCommand.length())
				FileDir = FileDirInCommand;
			if(iPersistTimeFileInCommand == -1)
			{
				// We get application work dir
				string sWorkDir = "";
				sWorkDir = Application->GetWorkDir();
				if(CompareWorkDir(FileDir, sWorkDir))
				{
					CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_SET_PERSISTANT, FileDir);
					return false;
				}
			}
		}
	}

	string ServiceName = "";
	bool AppRegistry = true;
	bool UserRegistry = true;
	NvRegistryContainer *RegistryContainer =  GetRegistry(CommandToExecute, Key, ServiceName, AppRegistry, UserRegistry);
	if(RegistryContainer == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "REGISTRY", NV_REGISTRY_CANNOT_GET_KEY, Key);
		return false;
	}
	
	// Check security
	if(!UserRegistry)
	{
		if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
		{
			if(AppRegistry)
			{
				if(!CheckPermission(CommandToExecute, "SYSTEM", "REGISTRY_APPLICATION_READ_CLIENT"))
				{
					if(ServiceName.length())
						GetMain()->Services.FreeService(ServiceName.c_str());
					return false;
				}
			}
			else
			{
				if(!CheckPermission(CommandToExecute, "SYSTEM", "REGISTRY_SYSTEM_READ_CLIENT"))
				{
					if(ServiceName.length())
						GetMain()->Services.FreeService(ServiceName.c_str());
					return false;
				}

			}
		}
	}
	// Get with data option option
	bool bWithData = true;
	string sWithData = CommandToExecute->Parameters.GetValue("WITH_DATA");
	CleanString(sWithData);
	MakeUpper(sWithData);
	if(sWithData == "NO")
		bWithData = false;

	const char *EntriesForCommand = NULL;
	if(CommandToExecute->Parameters.KeyExist("ENTRIES"))
		EntriesForCommand = EntryNames.c_str();

	if(!RegistryContainer->GetKey(Key.c_str(), OutContainer, AppendMode, EntriesForCommand, bWithSubKeys, bWithData, FileDir.c_str(), iPersistTimeFileInCommand, bRecurse))
	{
		CommandToExecute->Error.SetError("SYSTEM", "REGISTRY", NV_REGISTRY_CANNOT_GET_KEY, Key);
		if(ServiceName.length())
			GetMain()->Services.FreeService(ServiceName.c_str());
		return false;
	}

	if(ServiceName.length())
		GetMain()->Services.FreeService(ServiceName.c_str());

	return true;
}


//REGISTRY SET
bool NvSession::SetRegistryKey(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	
	// The function uses the output container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	// Get key name that may be in parameters
	string Key = CommandToExecute->Parameters.GetValue("KEY");
	CleanString(Key);
	MakeLower(Key);
	Key = NvContainerList::CleanContainerName(Key);


	// Get append option
	int AppendMode = NV_CONTAINER_COPY_NO_APPEND;
	string sAppend = CommandToExecute->Parameters.GetValue("APPEND");
	CleanString(sAppend);
	MakeUpper(sAppend);
	if(sAppend == "YES")
	{
		AppendMode = NV_CONTAINER_COPY_APPEND_NO_REPLACE;
		string sReplace = CommandToExecute->Parameters.GetValue("REPLACE");
		CleanString(sReplace);
		MakeUpper(sReplace);
		if(sReplace == "YES")
			AppendMode = NV_CONTAINER_COPY_APPEND;
	}

	// Get subkey option
	bool bWithSubKeys = false;
	string sWithSubKeys = CommandToExecute->Parameters.GetValue("SUBKEYS");
	CleanString(sWithSubKeys);
	MakeUpper(sWithSubKeys);
	if(sWithSubKeys == "YES")
		bWithSubKeys = true;

	// Get object names that may be in parameters
	string EntryNames = CommandToExecute->Parameters.GetValue("ENTRIES");
	CleanString(EntryNames);



	string ServiceName = "";
	bool AppRegistry = true;
	bool UserRegistry = true;
	NvRegistryContainer *RegistryContainer =  GetRegistry(CommandToExecute, Key, ServiceName, AppRegistry, UserRegistry);
	if(RegistryContainer == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "REGISTRY", NV_REGISTRY_CANNOT_SET_KEY, Key);
		return false;
	}

	// Check security
	if(!UserRegistry)
	{
		if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
		{
			if(AppRegistry)
			{
				if(!CheckPermission(CommandToExecute, "SYSTEM", "REGISTRY_APPLICATION_WRITE_CLIENT"))
				{
					if(ServiceName.length())
						GetMain()->Services.FreeService(ServiceName.c_str());
					return false;
				}

			}
			else
			{
				if(!CheckPermission(CommandToExecute, "SYSTEM", "REGISTRY_SYSTEM_WRITE_CLIENT"))
				{
					if(ServiceName.length())
						GetMain()->Services.FreeService(ServiceName.c_str());
					return false;
				}

			}
		}
	}
	// Get with data option option
	bool bWithData = true;
	string sWithData = CommandToExecute->Parameters.GetValue("WITH_DATA");
	CleanString(sWithData);
	MakeUpper(sWithData);
	if(sWithData == "NO")
		bWithData = false;

	const char *EntriesForCommand = NULL;
	if(CommandToExecute->Parameters.KeyExist("ENTRIES"))
		EntriesForCommand = EntryNames.c_str();

	if(!RegistryContainer->SetKey(Key.c_str(), InContainer, AppendMode, EntriesForCommand, bWithSubKeys, bWithData))
	{
		CommandToExecute->Error.SetError("SYSTEM", "REGISTRY", NV_REGISTRY_CANNOT_SET_KEY, Key);
		if(ServiceName.length())
			GetMain()->Services.FreeService(ServiceName.c_str());
		return false;
	}

	if(ServiceName.length())
		GetMain()->Services.FreeService(ServiceName.c_str());
	return true;
}





//REGISTRY REMOVE
bool NvSession::RemoveRegistryKey(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{


	// Get key name that may be in parameters
	string Key = CommandToExecute->Parameters.GetValue("KEY");
	CleanString(Key);
	MakeLower(Key);
	Key = NvContainerList::CleanContainerName(Key);

	// Get subkey option
	bool bWithSubKeys = false;
	string sWithSubKeys = CommandToExecute->Parameters.GetValue("SUBKEYS");
	CleanString(sWithSubKeys);
	MakeUpper(sWithSubKeys);
	if(sWithSubKeys == "YES")
		bWithSubKeys = true;

	// Get object names that may be in parameters
	string EntryNames = CommandToExecute->Parameters.GetValue("ENTRIES");
	CleanString(EntryNames);


	string ServiceName = "";
	bool AppRegistry = true;
	bool UserRegistry = true;
	NvRegistryContainer *RegistryContainer =  GetRegistry(CommandToExecute, Key, ServiceName, AppRegistry, UserRegistry);
	if(RegistryContainer == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "REGISTRY", NV_REGISTRY_CANNOT_REMOVE_KEY, Key);
		return false;
	}

	// Check security
	if(!UserRegistry)
	{
		if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
		{
			if(AppRegistry)
			{
				if(!CheckPermission(CommandToExecute, "SYSTEM", "REGISTRY_APPLICATION_WRITE_CLIENT"))
				{
					if(ServiceName.length())
						GetMain()->Services.FreeService(ServiceName.c_str());
					return false;
				}

			}
			else
			{
				if(!CheckPermission(CommandToExecute, "SYSTEM", "REGISTRY_SYSTEM_WRITE_CLIENT"))
				{
					if(ServiceName.length())
						GetMain()->Services.FreeService(ServiceName.c_str());
					return false;
				}

			}
		}
	}
	const char *EntriesForCommand = NULL;
	if(CommandToExecute->Parameters.KeyExist("ENTRIES"))
		EntriesForCommand = EntryNames.c_str();

	if(!RegistryContainer->RemoveKey(Key.c_str(), EntriesForCommand, bWithSubKeys))
	{
		CommandToExecute->Error.SetError("SYSTEM", "REGISTRY", NV_REGISTRY_CANNOT_REMOVE_KEY, Key);
		if(ServiceName.length())
			GetMain()->Services.FreeService(ServiceName.c_str());
		return false;
	}

	if(ServiceName.length())
		GetMain()->Services.FreeService(ServiceName.c_str());

	return true;
}



//REGISTRY CREATE
bool NvSession::CreateRegistryKey(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get key name that may be in parameters
	string Key = CommandToExecute->Parameters.GetValue("KEY");
	CleanString(Key);
	MakeLower(Key);
	Key = NvContainerList::CleanContainerName(Key);

	string ServiceName = "";
	bool AppRegistry = true;
	bool UserRegistry = true;
	NvRegistryContainer *RegistryContainer =  GetRegistry(CommandToExecute, Key, ServiceName, AppRegistry, UserRegistry);
	if(RegistryContainer == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "REGISTRY", NV_REGISTRY_CANNOT_CREATE_KEY, Key);
		return false;
	}

	// Check security
	if(!UserRegistry)
	{
		if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
		{
			if(AppRegistry)
			{
				if(!CheckPermission(CommandToExecute, "SYSTEM", "REGISTRY_APPLICATION_WRITE_CLIENT"))
				{
					if(ServiceName.length())
						GetMain()->Services.FreeService(ServiceName.c_str());
					return false;
				}

			}
			else
			{
				if(!CheckPermission(CommandToExecute, "SYSTEM", "REGISTRY_SYSTEM_WRITE_CLIENT"))
				{
					if(ServiceName.length())
						GetMain()->Services.FreeService(ServiceName.c_str());
					return false;
				}

			}
		}
	}
	if(!RegistryContainer->SetKey(Key.c_str(), NULL))
	{
		CommandToExecute->Error.SetError("SYSTEM", "REGISTRY", NV_REGISTRY_CANNOT_CREATE_KEY, Key);
		if(ServiceName.length())
			GetMain()->Services.FreeService(ServiceName.c_str());
		return false;
	}

	if(ServiceName.length())
		GetMain()->Services.FreeService(ServiceName.c_str());
	return true;
}




//REGISTRY EXIST
bool NvSession::RegistryKeyExist(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	
	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Get key name that may be in parameters
	string Key = CommandToExecute->Parameters.GetValue("KEY");
	CleanString(Key);
	MakeLower(Key);
	Key = NvContainerList::CleanContainerName(Key);

	bool Found = false;

	string ServiceName = "";
	bool AppRegistry = true;
	bool UserRegistry = true;
	NvRegistryContainer *RegistryContainer =  GetRegistry(CommandToExecute, Key, ServiceName, AppRegistry, UserRegistry);

	// Check security
	if(!UserRegistry)
	{
		if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
		{
			if(AppRegistry)
			{
				if(!CheckPermission(CommandToExecute, "SYSTEM", "REGISTRY_APPLICATION_READ_CLIENT"))
				{
					if(ServiceName.length())
						GetMain()->Services.FreeService(ServiceName.c_str());
					return false;
				}

			}
			else
			{
				if(!CheckPermission(CommandToExecute, "SYSTEM", "REGISTRY_SYSTEM_READ_CLIENT"))
				{
					if(ServiceName.length())
						GetMain()->Services.FreeService(ServiceName.c_str());
					return false;
				}

			}
		}
	}
	
	if(RegistryContainer != NULL)
	{
		if(RegistryContainer->GetKey(Key.c_str(), NULL))
			Found = true;

		if(ServiceName.length())
			GetMain()->Services.FreeService(ServiceName.c_str());
	}

	OutContainer->RemoveObject("EXIST");

	if(!OutContainer->CreateObject("EXIST", NV_OBJTYPE_BOOLEAN))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "EXIST");
		return false;
	}

	NvObj *Object = OutContainer->GetObject("EXIST");
	((NvObjBoolean *)Object)->SetValue(Found);

	return true;
}



//REGISTRY EXPORT
bool NvSession::ExportRegistryKey(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	
	// Get key name that may be in parameters
	string Key = CommandToExecute->Parameters.GetValue("KEY");
	CleanString(Key);
	MakeLower(Key);
	Key = NvContainerList::CleanContainerName(Key);
	
	// Get export file name that may be in parameters
	string FileName = CommandToExecute->Parameters.GetValue("FILENAME");
	SetDirName(FileName);
	if(!FileName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "REGISTRY:EXPORT:FILENAME");
		return false;
	}

	// Get subkey option
	bool bWithSubKeys = false;
	string sWithSubKeys = CommandToExecute->Parameters.GetValue("SUBKEYS");
	CleanString(sWithSubKeys);
	MakeUpper(sWithSubKeys);
	if(sWithSubKeys == "YES")
		bWithSubKeys = true;


	string ServiceName = "";
	bool AppRegistry = true;
	bool UserRegistry = true;
	NvRegistryContainer *RegistryContainer =  GetRegistry(CommandToExecute, Key, ServiceName, AppRegistry, UserRegistry);
	if(RegistryContainer == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "REGISTRY", NV_REGISTRY_CANNOT_EXPORT_KEY, Key);
		return false;
	}

	// Check security
	if(!UserRegistry)
	{
		if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
		{
			if(AppRegistry)
			{
				if(!CheckPermission(CommandToExecute, "SYSTEM", "REGISTRY_APPLICATION_READ_CLIENT"))
				{
					if(ServiceName.length())
						GetMain()->Services.FreeService(ServiceName.c_str());
					return false;
				}

			}
			else
			{
				if(!CheckPermission(CommandToExecute, "SYSTEM", "REGISTRY_SYSTEM_READ_CLIENT"))
				{
					if(ServiceName.length())
						GetMain()->Services.FreeService(ServiceName.c_str());
					return false;
				}

			}
		}
	}

	if(!RegistryContainer->Export(Key.c_str(), FileName.c_str(), bWithSubKeys))
	{
		CommandToExecute->Error.SetError("SYSTEM", "REGISTRY", NV_REGISTRY_CANNOT_EXPORT_KEY, Key);
		if(ServiceName.length())
			GetMain()->Services.FreeService(ServiceName.c_str());
		return false;
	}

	if(ServiceName.length())
		GetMain()->Services.FreeService(ServiceName.c_str());

	return true;
}





//REGISTRY IMPORT
bool NvSession::ImportRegistryKey(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get key name that may be in parameters
	string Key = CommandToExecute->Parameters.GetValue("KEY");
	CleanString(Key);
	MakeLower(Key);
	Key = NvContainerList::CleanContainerName(Key);
	
	// Get import file name that may be in parameters
	string FileName = CommandToExecute->Parameters.GetValue("FILENAME");
	SetDirName(FileName);
	if(!FileName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "REGISTRY:IMPORT:FILENAME");
		return false;
	}


	// Get append option
	int AppendMode = NV_CONTAINER_COPY_NO_APPEND;
	string sAppend = CommandToExecute->Parameters.GetValue("APPEND");
	CleanString(sAppend);
	MakeUpper(sAppend);
	if(sAppend == "YES")
	{
		AppendMode = NV_CONTAINER_COPY_APPEND_NO_REPLACE;
		string sReplace = CommandToExecute->Parameters.GetValue("REPLACE");
		CleanString(sReplace);
		MakeUpper(sReplace);
		if(sReplace == "YES")
			AppendMode = NV_CONTAINER_COPY_APPEND;
	}


	string ServiceName = "";
	bool AppRegistry = true;
	bool UserRegistry = true;
	NvRegistryContainer *RegistryContainer =  GetRegistry(CommandToExecute, Key, ServiceName, AppRegistry, UserRegistry);
	if(RegistryContainer == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "REGISTRY", NV_REGISTRY_CANNOT_IMPORT_KEY, Key);
		return false;
	}

	// Check security
	if(!UserRegistry)
	{
		if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
		{
			if(AppRegistry)
			{
				if(!CheckPermission(CommandToExecute, "SYSTEM", "REGISTRY_APPLICATION_WRITE_CLIENT"))
				{
					if(ServiceName.length())
						GetMain()->Services.FreeService(ServiceName.c_str());
					return false;
				}

			}
			else
			{
				if(!CheckPermission(CommandToExecute, "SYSTEM", "REGISTRY_SYSTEM_WRITE_CLIENT"))
				{
					if(ServiceName.length())
						GetMain()->Services.FreeService(ServiceName.c_str());
					return false;
				}

			}
		}
	}

	if(!RegistryContainer->Import(Key.c_str(), FileName.c_str(), AppendMode))
	{
		CommandToExecute->Error.SetError("SYSTEM", "REGISTRY", NV_REGISTRY_CANNOT_IMPORT_KEY, Key);
		if(ServiceName.length())
			GetMain()->Services.FreeService(ServiceName.c_str());
		return false;
	}

	if(ServiceName.length())
		GetMain()->Services.FreeService(ServiceName.c_str());
	return true;
}


//REGISTRY CLEAR_CACHE
bool NvSession::ClearRegistryCache(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// This command is authorized only from procedure or service
	if((CommandToExecute->m_iCommandSource != NV_SOURCE_PROCEDURE) && (CommandToExecute->m_iCommandSource != NV_SOURCE_SERVICE) && (CommandToExecute->m_iCommandSource != NV_SOURCE_SERVER))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:REGISTRY:CLEAR_CACHE");
		return false;
	}

	bool bServiceRegistry = false;
	bool bSystemRegistry = false;
	bool bApplicationRegistry = true;

	NvService *Service = NULL;
	
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);
	if(ServiceName.length())
	{
		bApplicationRegistry = false;
		if(ServiceName == "SYSTEM")
			bSystemRegistry = true;
		else
			bServiceRegistry = true;
	}
		
	// Get registry name (user registry)
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	NvRegistryContainerList::CleanRegistryName(Name);

	if(bServiceRegistry && ServiceName.length())
	{
		Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
		if(Service == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
			return false;
		}
	}

	NvRegistryContainer *RegistryContainer = &(Application->AppRegistry);
	if(bSystemRegistry)
		RegistryContainer = &(GetMain()->SystemRegistry);
	if(bServiceRegistry)
		RegistryContainer = &(Service->SrvRegistry);

	if(Name.length())
	{
		// If this is a user registry, we try to get it from corresponding level
		// First check if its ever opened
		string RegistryIdent = ServiceName;
		if(bSystemRegistry)
			RegistryIdent = "SYSTEM";
		CleanString(RegistryIdent);
		MakeUpper(RegistryIdent);
		RegistryIdent += "::::";
		RegistryIdent += Name;
		if(m_oRegistries.find(RegistryIdent) == m_oRegistries.end())
		{
			// The registry is not opened so nothing to do
			if(Service != NULL)
				GetMain()->Services.FreeService(ServiceName.c_str());
			return true;
		}
		if(bSystemRegistry)
			RegistryContainer = GetMain()->Registries.GetRegistry(Name);
		else if(ServiceName.length())
			RegistryContainer = Service->Registries.GetRegistry(Name);
		else
			RegistryContainer = Application->Registries.GetRegistry(Name);
	}


	if(RegistryContainer != NULL)
		RegistryContainer->Clear();

	if(Service != NULL)
		GetMain()->Services.FreeService(ServiceName.c_str());

	return true;
}


//REGISTRY OPEN_REGISTRY
bool NvSession::OpenRegistry(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	bool bServiceRegistry = false;
	bool bSystemRegistry = false;
	bool bApplicationRegistry = true;

	NvService *Service = NULL;
	
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);
	if(ServiceName.length())
	{
		bApplicationRegistry = false;
		if(ServiceName == "SYSTEM")
			bSystemRegistry = true;
		else
			bServiceRegistry = true;
	}
		
	// Get registry name (user registry)
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	NvRegistryContainerList::CleanRegistryName(Name);
	if(!Name.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "REGISTRY:OPEN_REGISTRY:NAME");
		return false;
	}
		
	// Get registry directory (user registry)
	string Directory = CommandToExecute->Parameters.GetValue("DIRECTORY");
	CleanString(Directory);
	if(!Directory.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "REGISTRY:OPEN_REGISTRY:DIRECTORY");
		return false;
	}

	if(bServiceRegistry && ServiceName.length())
	{
		Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
		if(Service == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
			return false;
		}
	}

	NvRegistryContainer *RegistryContainer = NULL;
	if(bSystemRegistry)
		RegistryContainer = GetMain()->Registries.OpenRegistry(Name, Directory);
	else if(ServiceName.length())
		RegistryContainer = Service->Registries.OpenRegistry(Name, Directory);
	else
		RegistryContainer = Application->Registries.OpenRegistry(Name, Directory);

	if(RegistryContainer == NULL)
	{
		if(Service != NULL)
			GetMain()->Services.FreeService(ServiceName.c_str());
		string Info = ServiceName;
		Info += ":";
		Info += Name;
		CommandToExecute->Error.SetError("SYSTEM", "REGISTRY", NV_REGISTRY_CANNOT_OPEN, Info);
		return false;
	}

	string RegistryIdent = ServiceName;
	if(bSystemRegistry)
		RegistryIdent = "SYSTEM";
	CleanString(RegistryIdent);
	MakeUpper(RegistryIdent);
	RegistryIdent += "::::";
	RegistryIdent += Name;
	int NumInstances = 0;
	if(m_oRegistries.find(RegistryIdent) != m_oRegistries.end())
		NumInstances = m_oRegistries[RegistryIdent];
	NumInstances ++;
	m_oRegistries[RegistryIdent] = NumInstances;

	if(Service != NULL)
		GetMain()->Services.FreeService(ServiceName.c_str());

	return true;
}


//REGISTRY CLOSE_REGISTRY
bool NvSession::CloseRegistry(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	bool bServiceRegistry = false;
	bool bSystemRegistry = false;
	bool bApplicationRegistry = true;

	NvService *Service = NULL;
	
	string ServiceName = CommandToExecute->Parameters.GetValue("SERVICE");
	CleanString(ServiceName);
	MakeUpper(ServiceName);
	if(ServiceName.length())
	{
		bApplicationRegistry = false;
		if(ServiceName == "SYSTEM")
			bSystemRegistry = true;
		else
			bServiceRegistry = true;
	}
		
	// Get registry name (user registry)
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	NvRegistryContainerList::CleanRegistryName(Name);
	if(!Name.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "REGISTRY:OPEN_REGISTRY:NAME");
		return false;
	}

	if(bServiceRegistry && ServiceName.length())
	{
		Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
		if(Service == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
			return false;
		}
	}


	bool RetVal = true;
	if(bSystemRegistry)
		RetVal = GetMain()->Registries.CloseRegistry(Name);
	else if(ServiceName.length())
		RetVal = Service->Registries.CloseRegistry(Name);
	else
		RetVal = Application->Registries.CloseRegistry(Name);

	if(RetVal)
	{
		string RegistryIdent = ServiceName;
		if(bSystemRegistry)
			RegistryIdent = "SYSTEM";
		CleanString(RegistryIdent);
		MakeUpper(RegistryIdent);
		RegistryIdent += "::::";
		RegistryIdent += Name;
		int NumInstances = 0;
		if(m_oRegistries.find(RegistryIdent) != m_oRegistries.end())
			NumInstances = m_oRegistries[RegistryIdent];
		if(NumInstances > 0)
			NumInstances --;
		if(NumInstances == 0)
		{
			if(m_oRegistries.find(RegistryIdent) != m_oRegistries.end())
				m_oRegistries.erase(RegistryIdent);
		}
		else
			m_oRegistries[RegistryIdent] = NumInstances;
	}

	if(Service != NULL)
		GetMain()->Services.FreeService(ServiceName.c_str());

	return true;
}


// FILEDB OPEN
bool NvSession::FileDbOpen(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(Name);
	MakeUpper(Name);
	string FileName = CommandToExecute->Parameters.GetValue("FILENAME");
	if(!FileName.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "FILEDB:OPEN:FILENAME");
		return false;
	}

	if(SqLite.Open(Name.c_str(), FileName.c_str()) != SQLITE_ERROR_NONE)
	{
		CommandToExecute->Error.SetError("SYSTEM", "FILEDB", NV_FILEDB_CANNOT_OPEN_DB, SqLite.LastErrorInfo);
		return false;
	}

	return true;
}


// FILEDB CLOSE
bool NvSession::FileDbClose(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(Name);
	MakeUpper(Name);

	SqLite.Close(Name.c_str());
	return true;
}


// FILEDB SQL
bool NvSession::FileDbSql(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string Name = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(Name);
	MakeUpper(Name);

	string Sql = CommandToExecute->Parameters.GetValue("SQL");
	if(!Sql.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, "FILEDB:SQL:SQL");
		return false;
	}

	// The function uses the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	string ObjResult = CommandToExecute->Parameters.GetValue("RESULT");
	CleanString(ObjResult);
	MakeUpper(ObjResult);
	if(!ObjResult.length())
		ObjResult = "RECORDS";

	NvObjTable* Records = NULL;
	if(ObjResult != "NONE")
	{
		OutContainer->RemoveObject(ObjResult.c_str());
		if(!OutContainer->CreateObject(ObjResult.c_str(), NV_OBJTYPE_TABLE))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjResult);
			return false;
		}
		Records = (NvObjTable *)(OutContainer->GetObject(ObjResult.c_str()));
	}

	int TimeOut = 60;
	string sTimeOut = CommandToExecute->Parameters.GetValue("TIMEOUT");
	CleanString(sTimeOut);
	MakeUpper(sTimeOut);
	if(sTimeOut.length())
		TimeOut = atoi(sTimeOut.c_str());
	if(TimeOut < 0)
		TimeOut = -1;

	int rc = SqLite.Exec(Name.c_str(), Sql.c_str(), Records, TimeOut);
	if(rc == SQLITE_ERROR_NONE)
		return true;
	
	if(rc == SQLITE_ERROR_NOTCONNECTED)
		CommandToExecute->Error.SetError("SYSTEM", "FILEDB", NV_FILEDB_NOT_CONNECTED, SqLite.LastErrorInfo);
	else if(rc == SQLITE_ERROR_TIMEOUT)
		CommandToExecute->Error.SetError("SYSTEM", "FILEDB", NV_FILEDB_TIMEOUT, SqLite.LastErrorInfo);
	else
		CommandToExecute->Error.SetError("SYSTEM", "FILEDB", NV_FILEDB_CANNOT_EXECUTE_SQL, SqLite.LastErrorInfo);
	
	return false;
}

